/*
 * ********************************************************* {{{1 **********
 *   Copyright © 2005 … 2021  "Martin Krischik" «krischik@users.sourceforge.net»
 * *************************************************************************
 *   This program is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * *************************************************************************
 *   $Author: krischik $
 *   $Revision: 7684 $
 *   $Date: 2021-09-13 20:49:22 +0200 (Mo, 13. Sep 2021) $
 *   $Id: Mock_Canvas.kt 7684 2021-09-13 18:49:22Z krischik $
 * ********************************************************** }}}1 **********
 *
 */

package net.sourceforge.uiq3.hp45

import org.jetbrains.annotations.NotNull
import org.jetbrains.annotations.TestOnly

/**
 * <p>A Canvas Mock class</p>
 * @param keystrokes Keystrokes to send to the calculator
 * @author "Martin Krischik" «krischik@users.sourceforge.net»
 */
@android.annotation.TargetApi(android.os.Build.VERSION_CODES.P)
class Mock_Canvas(keystrokes: List<Short>) :
   net.sourceforge.uiq3.ui.ICalculatorCanvas
{
   companion object
   {
      private val TAG = net.sourceforge.uiq3.Log.createTag(Mock_Canvas::class.java)
   }

   private var currentKey = keystrokes.iterator()
   private var displayText = Array(size = 3) { ShortArray(size = 16) }
   private var printerText = ""

   /**
    * Get calculator key code of the currently pressed key.
    *
    * @return calculator key code
    */
   override var pressedKey = net.sourceforge.uiq3.calculator.IKey.No_Key

   /**
    * Get calculator key code of the last key (de)pressed.
    *
    * @return calculator key code
    */
   override var readKey: Short
      get()
      {
	 val Retval = if (currentKey.hasNext())
	 {
	    currentKey.next()
	 }
	 else
	 {
	    net.sourceforge.uiq3.calculator.IKey.No_Key
	 } // if

	 return Retval
      } // Get_Read_Key
      set(value)
      {
	 this.keystrokes = listOf(value)
      }

   /**
    * set new kestrokes for enhanced testing
    * @param Keystrokes new keystrokes
    */
   var keystrokes: List<Short> = keystrokes
      get() = field
      set(keystrokes)
      {
	 field = keystrokes
	 this.currentKey = field.iterator()
      } // setKeystrokes

   /**
    * <p>Check if a calculator key code is available</p>
    *
    * @return true when a key is available.
    */
   @get:TestOnly
   override val isReadKeyAvailable
      get(): Boolean = currentKey.hasNext()

   /**
    * Set the indicators (Shift, Alpha, Deg, Grad, Gra, etc. pp.)
    *
    * @param Indicators
    * indicators to be shown
    */
   override var indicators = BooleanArray (size = 19)

   /**
    * Set text to be displayed (as java string, used by RPN-45 only).
    */
   override var javaText = ""

   /**
    * <p>Set calculator Thread so the canvas can wake up the calculator after key / touch screen
    * clicks.</p>
    *
    * @param CPU_Thread
    * CPU Thread.
    * @author "Martin Krischik" «krischik@users.sourceforge.net»
    */
   override fun bindCPUThread(CPU_Thread: Thread?)
   {
      // do nothing
      return
   } // bindCPUThread

   /**
    * <p>Set text to be displayed (calculator alpha codes). Null implementation for calculators
    * without alpha display.</p>
    *
    * @param Row
    * Display row to set
    * @param Text
    * to be displayed
    */
   override fun setDisplayText(Row: Int, Text: ShortArray)
   {
      displayText[Row] = Text
   } // Set_Text

   override fun getDisplayText(Row: Int): ShortArray
   {
      return displayText[Row]
   }

   override fun getJavaText(Row: Int): String
   {
      return printerText;
   }

   /**
    * <p>Toggle Keyboard help on and off</p>
    */
   override fun toggleHelp()
   {
      // do nothing
   } // Toggle_Help

   /**
    * <p>display additional information on the error. this is device and setup dependent.</p>
    *
    * @param Exception
    *    Exception to display
    */
   override fun displayException(@NotNull Exception: Throwable)
   {
      net.sourceforge.uiq3.Log.d(TAG, "displayException", arrayOf(Exception))

      net.sourceforge.uiq3.Log.wtf(TAG, "calculator exception", Exception)

      net.sourceforge.uiq3.Log.d(TAG, "displayException")
      return;
   } // displayException

   /**
    * <p> set the calculators on of status. </p>
    *
    * @param on
    *    calculator is switched on
    */
   override fun setOnOff(on: Boolean)
   {
      return
   }
}

// ********************************************************* {{{1 **********
// vim: set nowrap tabstop=8 shiftwidth=3 softtabstop=3 expandtab textwidth=120 :
// vim: set fileencoding=utf-8 filetype=kotlin foldmethod=indent spell spelllang=en_gb :
