/*
 * Copyright (C) 2010-2012 Zutubi Pty Ltd
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.zutubi.android.junitreport;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.io.Writer;
import java.util.Locale;

import junit.framework.AssertionFailedError;
import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestListener;

import org.xmlpull.v1.XmlSerializer;

import android.content.Context;
import android.util.Log;
import android.util.Xml;

/**
 * Custom test listener that outputs test results to XML files. The files use a similar format
 * to the Ant JUnit task XML formatter, with a few of caveats:
 * <ul>
 * <li>By default, multiple suites are all placed in a single file under a root
 * &lt;testsuites&gt; element. In multiFile mode a separate file is created for each suite,
 * which may be more compatible with existing tools.</li>
 * <li>Redundant information about the number of nested cases within a suite is omitted.</li>
 * <li>Durations are omitted from suites.</li>
 * <li>Neither standard output nor system properties are included.</li>
 * </ul>
 * The differences mainly revolve around making this reporting as lightweight as possible. The
 * report is streamed as the tests run, making it impossible to, e.g. include the case count in
 * a &lt;testsuite&gt; element.
 */
@SuppressWarnings ("WeakerAccess")
public class JUnitReportListener
   implements
      TestListener
{
   /**
    * Wrapper around a print writer that filters out common noise from stack traces, making it
    * easier to see the actual failure.
    */
   private static class FilteringWriter
      extends
         PrintWriter
   {
      public FilteringWriter (final Writer out)
      {
         super (out);
      }

      @Override
      public void println (final String s)
      {
         for (final String filtered : JUnitReportListener.DEFAULT_TRACE_FILTERS)
         {
            if (s.contains (filtered))
            {
               return;
            }
         }

         super.println (s);
      }
   }

   private static final String ATTRIBUTE_CLASS = "classname";

   private static final String ATTRIBUTE_MESSAGE = "message";
   private static final String ATTRIBUTE_NAME = "name";

   private static final String ATTRIBUTE_TIME = "time";
   private static final String ATTRIBUTE_TYPE = "type";
   // With thanks to org.apache.tools.ant.taskdefs.optional.junit.JUnitTestRunner.
   // Trimmed some entries, added others for Android.
   private static final String [] DEFAULT_TRACE_FILTERS = new String [] {
      "junit.framework.TestCase",
      "junit.framework.TestResult",
      "junit.framework.TestSuite",
      "junit.framework.Assert.", // don't filter AssertionFailure
      "java.lang.reflect.Method.invoke(",
      "sun.reflect.",
      // JUnit 4 support:
      "org.junit.",
      "junit.framework.JUnit4TestAdapter",
      " more",
      // Added for Android
      "android.test.",
      "android.app.Instrumentation",
      "java.lang.reflect.Method.invokeNative", };
   private static final String ENCODING_UTF_8 = "utf-8";
   private static final String LOG_TAG = JUnitReportListener.class.getSimpleName ();

   private static final String TAG_CASE = "testcase";
   private static final String TAG_ERROR = "error";
   private static final String TAG_FAILURE = "failure";
   private static final String TAG_SUITE = "testsuite";
   private static final String TAG_SUITES = "testsuites";

   public static final String TOKEN_EXTERNAL = "__external__";

   public static final String TOKEN_SUITE = "__suite__";
   private String mCurrentSuite;
   private final boolean mFilterTraces;
   private final boolean mMultiFile;
   private FileOutputStream mOutputStream;
   private String mReportDir;
   private final String mReportFile;
   private XmlSerializer mSerializer;

   private final Context mTargetContext;
   private long mTestStartTime;

   // simple time tracking
   private boolean mTimeAlreadyWritten = false;

   /**
    * Creates a new listener.
    *
    * @param context
    *           context of the test application
    * @param targetContext
    *           context of the application under test
    * @param reportFile
    *           name of the report file(s) to create
    * @param reportDir
    *           path of the directory under which to write files (may be null in which case
    *           files are written under the context using
    *           {@link Context#openFileOutput(String, int)}).
    * @param filterTraces
    *           if true, stack traces will have common noise (e.g. framework methods) omitted
    *           for clarity
    * @param multiFile
    *           if true, use a separate file for each test suite
    */
   public JUnitReportListener (
      final Context context,
      final Context targetContext,
      final String reportFile,
      final String reportDir,
      final boolean filterTraces,
      final boolean multiFile)
   {
      Log.i (
         JUnitReportListener.LOG_TAG,
         "Listener created with arguments:\n" + "  report file  : '" + reportFile + "'\n" + "  report dir   : '" +
            reportDir + "'\n" + "  filter traces: " + filterTraces + "\n" + "  multi file   : " + multiFile);

      this.mTargetContext = targetContext;
      this.mReportFile = reportFile;
      this.mReportDir = reportDir;
      this.mFilterTraces = filterTraces;
      this.mMultiFile = multiFile;
   }

   @Override
   public void addError (final Test test, final Throwable error)
   {
      this.addProblem (JUnitReportListener.TAG_ERROR, error);
   }

   @Override
   public void addFailure (final Test test, final AssertionFailedError error)
   {
      this.addProblem (JUnitReportListener.TAG_FAILURE, error);
   }

   private void addProblem (final String tag, final Throwable error)
   {
      try
      {
         this.recordTestTime ();

         this.mSerializer.startTag ("", tag);
         this.mSerializer.attribute ("", JUnitReportListener.ATTRIBUTE_MESSAGE, this.safeMessage (error));
         this.mSerializer.attribute ("", JUnitReportListener.ATTRIBUTE_TYPE, error.getClass ().getName ());
         final StringWriter w = new StringWriter ();
         error.printStackTrace (
            this.mFilterTraces
               ? new FilteringWriter (w)
               : new PrintWriter (w));
         this.mSerializer.text (w.toString ());
         this.mSerializer.endTag ("", tag);
         this.mSerializer.flush ();
      }
      catch (final IOException e)
      {
         Log.e (JUnitReportListener.LOG_TAG, this.safeMessage (e));
      }
   }

   private void checkForNewSuite (final TestCase testCase)
      throws IOException
   {
      final String suiteName = testCase.getClass ().getName ();
      if ((this.mCurrentSuite == null) || !this.mCurrentSuite.equals (suiteName))
      {
         if (this.mCurrentSuite != null)
         {
            if (this.mMultiFile)
            {
               this.close ();
            }
            else
            {
               this.mSerializer.endTag ("", JUnitReportListener.TAG_SUITE);
               this.mSerializer.flush ();
            }
         }

         this.openIfRequired (suiteName);

         this.mSerializer.startTag ("", JUnitReportListener.TAG_SUITE);
         this.mSerializer.attribute ("", JUnitReportListener.ATTRIBUTE_NAME, suiteName);
         this.mCurrentSuite = suiteName;
      }
   }

   /**
    * Releases all resources associated with this listener. Must be called when the listener is
    * finished with.
    */
   public void close ()
   {
      if (this.mSerializer != null)
      {
         try
         {
            // Do this just in case endTest() was not called due to a crash in native code.
            if (JUnitReportListener.TAG_CASE.equals (this.mSerializer.getName ()))
            {
               this.mSerializer.endTag ("", JUnitReportListener.TAG_CASE);
            }

            if (this.mCurrentSuite != null)
            {
               this.mSerializer.endTag ("", JUnitReportListener.TAG_SUITE);
            }

            if (!this.mMultiFile)
            {
               this.mSerializer.endTag ("", JUnitReportListener.TAG_SUITES);
            }
            this.mSerializer.endDocument ();
            this.mSerializer.flush ();
            this.mSerializer = null;
         }
         catch (final IOException e)
         {
            Log.e (JUnitReportListener.LOG_TAG, this.safeMessage (e));
         }
      }

      if (this.mOutputStream != null)
      {
         try
         {
            this.mOutputStream.close ();
            this.mOutputStream = null;
         }
         catch (final IOException e)
         {
            Log.e (JUnitReportListener.LOG_TAG, this.safeMessage (e));
         }
      }
   }

   @Override
   public void endTest (final Test test)
   {
      try
      {
         if (test instanceof TestCase)
         {
            this.recordTestTime ();
            this.mSerializer.endTag ("", JUnitReportListener.TAG_CASE);
            this.mSerializer.flush ();
         }
      }
      catch (final IOException e)
      {
         Log.e (JUnitReportListener.LOG_TAG, this.safeMessage (e));
      }
   }

   private void ensureDirectoryExists (final String path)
      throws IOException
   {
      final File dir = new File (path);
      if (!dir.isDirectory () && !dir.mkdirs ())
      {
         final String message = "Cannot create directory '" + path + "'";
         Log.e (JUnitReportListener.LOG_TAG, message);
         throw new IOException (message);
      }
   }

   private void openIfRequired (final String suiteName)
   {
      try
      {
         if (this.mSerializer == null)
         {
            this.mOutputStream = this.openOutputStream (this.resolveFileName (suiteName));
            this.mSerializer = Xml.newSerializer ();
            this.mSerializer.setOutput (this.mOutputStream, JUnitReportListener.ENCODING_UTF_8);
            this.mSerializer.startDocument (JUnitReportListener.ENCODING_UTF_8, true);
            if (!this.mMultiFile)
            {
               this.mSerializer.startTag ("", JUnitReportListener.TAG_SUITES);
            }
         }
      }
      catch (final IOException e)
      {
         Log.e (JUnitReportListener.LOG_TAG, this.safeMessage (e));
         throw new RuntimeException ("Unable to open serializer: " + e.getMessage (), e);
      }
   }

   private FileOutputStream openOutputStream (final String fileName)
      throws IOException
   {
      if (this.mReportDir == null)
      {
         Log.d (
            JUnitReportListener.LOG_TAG,
            "No reportDir specified. Opening report file '" + fileName + "' in internal storage of app under test");
         return this.mTargetContext.openFileOutput (fileName, Context.MODE_WORLD_READABLE);
      }
      else
      {
         if (this.mReportDir.contains (JUnitReportListener.TOKEN_EXTERNAL))
         {
            final File externalDir = Compatibility.getExternalFilesDir (this.mTargetContext, null);
            if (externalDir == null)
            {
               Log.e (
                  JUnitReportListener.LOG_TAG,
                  "reportDir references external storage, but external storage is not available (check mounting and permissions)");
               throw new IOException ("Cannot access external storage");
            }

            String externalPath = externalDir.getAbsolutePath ();
            if (externalPath.endsWith ("/"))
            {
               externalPath = externalPath.substring (0, externalPath.length () - 1);
            }

            this.mReportDir = this.mReportDir.replace (JUnitReportListener.TOKEN_EXTERNAL, externalPath);
         }

         this.ensureDirectoryExists (this.mReportDir);

         final File outputFile = new File (this.mReportDir, fileName);
         Log.d (JUnitReportListener.LOG_TAG, "Opening report file '" + outputFile.getAbsolutePath () + "'");
         return new FileOutputStream (outputFile);
      }
   }

   private void recordTestTime ()
      throws IOException
   {
      if (!this.mTimeAlreadyWritten)
      {
         this.mTimeAlreadyWritten = true;
         this.mSerializer.attribute (
            "",
            JUnitReportListener.ATTRIBUTE_TIME,
            String.format (Locale.ENGLISH, "%.3f", (System.currentTimeMillis () - this.mTestStartTime) / 1000.));
      }
   }

   private String resolveFileName (final String suiteName)
   {
      String fileName = this.mReportFile;
      if (this.mMultiFile)
      {
         fileName = fileName.replace (JUnitReportListener.TOKEN_SUITE, suiteName);
      }
      return fileName;
   }

   private String safeMessage (final Throwable error)
   {
      final String message = error.getMessage ();
      return error.getClass ().getName () + ": " + (message == null
         ? "<null>"
         : message);
   }

   @Override
   public void startTest (final Test test)
   {
      try
      {
         if (test instanceof TestCase)
         {
            final TestCase testCase = (TestCase) test;
            this.checkForNewSuite (testCase);
            this.mSerializer.startTag ("", JUnitReportListener.TAG_CASE);
            this.mSerializer.attribute ("", JUnitReportListener.ATTRIBUTE_CLASS, this.mCurrentSuite);
            this.mSerializer.attribute ("", JUnitReportListener.ATTRIBUTE_NAME, testCase.getName ());

            this.mTimeAlreadyWritten = false;
            this.mTestStartTime = System.currentTimeMillis ();
         }
      }
      catch (final IOException e)
      {
         Log.e (JUnitReportListener.LOG_TAG, this.safeMessage (e));
      }
   }
}
