/*
 * Copyright (C) 2010-2012 Zutubi Pty Ltd
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.zutubi.android.junitreport;

import android.os.Bundle;
import android.test.AndroidTestRunner;
import android.test.InstrumentationTestRunner;
import android.util.Log;

/**
 * Custom test runner that adds a {@link JUnitReportListener} to the underlying test runner in
 * order to capture test results in an XML report. You may use this class in place of
 * {@link InstrumentationTestRunner} in your test project's manifest, and/or specify it to your
 * Ant build using the test.runner property.
 * <p/>
 * This runner behaves identically to the default, with the added side-effect of producing JUnit
 * XML reports. The report format is similar to that produced by the Ant JUnit task's XML
 * formatter, making it compatible with existing tools that can process that format. See
 * {@link JUnitReportListener} for further details.
 * <p/>
 * This runner accepts arguments specified by the ARG_* constants. For details refer to the
 * README.
 */
@SuppressWarnings ("ALL")
public class JUnitReportTestRunner
   extends
      InstrumentationTestRunner
{
   /**
    * If true, stack traces in the report will be filtered to remove common noise (e.g.
    * framework methods).
    */
   private static final String ARG_FILTER_TRACES = "filterTraces";
   /**
    * If true, produce a separate file for each test suite. By default a single report is
    * created for all suites.
    */
   private static final String ARG_MULTI_FILE = "multiFile";
   /**
    * If specified, path of the directory to write report files to. May start with __external__.
    * If not set files are written to the internal storage directory of the app under test.
    */
   private static final String ARG_REPORT_DIR = "reportDir";
   /**
    * Name of the report file(s) to write, may contain __suite__ in multiFile mode.
    */
   private static final String ARG_REPORT_FILE = "reportFile";
   /**
    * Default name pattern for multiple report files.
    */
   private static final String DEFAULT_MULTI_REPORT_FILE = "junit-report-" + JUnitReportListener.TOKEN_SUITE + ".xml";
   /**
    * Default name of the single report file.
    */
   private static final String DEFAULT_SINGLE_REPORT_FILE = "junit-report.xml";

   private static final String LOG_TAG = JUnitReportTestRunner.class.getSimpleName ();

   private boolean mFilterTraces = true;
   private JUnitReportListener mListener;
   private boolean mMultiFile = false;
   private String mReportDir;
   private String mReportFile;

   @Override
   public void finish (final int resultCode, final Bundle results)
   {
      if (this.mListener != null)
      {
         this.mListener.close ();
      }

      super.finish (resultCode, results);
   }

   @Override
   protected AndroidTestRunner getAndroidTestRunner ()
   {
      final AndroidTestRunner runner = this.makeAndroidTestRunner ();
      this.mListener = new JUnitReportListener (
         this.getContext (),
         this.getTargetContext (),
         this.mReportFile,
         this.mReportDir,
         this.mFilterTraces,
         this.mMultiFile);
      runner.addTestListener (this.mListener);
      return runner;
   }

   private boolean getBooleanArgument (final Bundle arguments, final String name, final boolean defaultValue)
   {
      final String value = arguments.getString (name);
      if (value == null)
      {
         return defaultValue;
      }
      else
      {
         return Boolean.parseBoolean (value);
      }
   }

   /**
    * Subclass and override this if you want to use a different TestRunner type.
    *
    * @return the test runner to use
    */
   protected AndroidTestRunner makeAndroidTestRunner ()
   {
      return new AndroidTestRunner ();
   }

   @Override
   public void onCreate (final Bundle arguments)
   {
      if (arguments != null)
      {
         Log.i (JUnitReportTestRunner.LOG_TAG, "Created with arguments: " + arguments.keySet ());
         this.mReportFile = arguments.getString (JUnitReportTestRunner.ARG_REPORT_FILE);
         this.mReportDir = arguments.getString (JUnitReportTestRunner.ARG_REPORT_DIR);
         this.mFilterTraces = this.getBooleanArgument (arguments, JUnitReportTestRunner.ARG_FILTER_TRACES, true);
         this.mMultiFile = this.getBooleanArgument (arguments, JUnitReportTestRunner.ARG_MULTI_FILE, false);
      }
      else
      {
         Log.i (JUnitReportTestRunner.LOG_TAG, "No arguments provided");
      }

      if (this.mReportFile == null)
      {
         this.mReportFile = this.mMultiFile
            ? JUnitReportTestRunner.DEFAULT_MULTI_REPORT_FILE
            : JUnitReportTestRunner.DEFAULT_SINGLE_REPORT_FILE;
         Log.i (JUnitReportTestRunner.LOG_TAG, "Defaulted report file to '" + this.mReportFile + "'");
      }

      super.onCreate (arguments);
   }
}
