/*
 * ******************************************************** {{{1 ************
 *   Copyright © 2005 … 2017 "Martin Krischik" «krischik@users.sourceforge.net»
 * **************************************************************************
 *   This program is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see «http://www.gnu.org/licenses».
 *  ***************************************************************************
 *   $Author: krischik $
 *   $Revision: 7681 $
 *   $Date: 2021-09-02 16:15:29 +0200 (Do, 02. Sep 2021) $
 *   $Id: BalloonItemizedOverlay.java 7681 2021-09-02 14:15:29Z krischik $
 *  ********************************************************** }}}1 ***********
 */

package net.sourceforge.uiq3.maps;

/**
 * An abstract extension of ItemizedOverlay for displaying an information balloon upon
 * screen-tap of each marker overlay.
 *
 * @author Jeff Gilfelt
 * @author "Martin Krischik" «krischik@users.sourceforge.net»
 * @version 1.2 $Revision: 7681 $
 * @since 1.2
 */
@SuppressWarnings ({
   "UnusedDeclaration" })
@android.annotation.TargetApi (android.os.Build.VERSION_CODES.UPSIDE_DOWN_CAKE)
public abstract class BalloonItemizedOverlay <Item extends com.google.android.maps.OverlayItem>
   extends
      com.google.android.maps.ItemizedOverlay <Item>
{

   private BalloonOverlayView <Item> balloonView;
   private final int closeButtonId;
   private int currentFocussedIndex;
   private Item currentFocussedItem;
   private final android.graphics.drawable.Drawable focusedDrawable;
   private final int innerLayoutid;
   private final int itemSnippetId;
   private final int itemTitleId;

   private final int mainLayoutId;
   @org.jetbrains.annotations.NotNull
   private final com.google.android.maps.MapView mapView;
   private final com.google.android.maps.MapController mc;
   private final int overlayId;
   private final android.graphics.drawable.Drawable unFocusedDrawable;
   private int viewOffset;

   /**
    * Create a new BalloonItemizedOverlay
    *
    * @param defaultMarker
    *           - A bounded Drawable to be drawn on the map for each item in the overlay.
    * @param mapView
    * @param overlayFocusedId
    * @param overlayUnfocusedId
    * @param mainLayoutId
    * @param innerLayoutid
    * @param overlayId
    * @param itemTitleId
    * @param itemSnippetId
    * @param closeButtonId
    */
   protected BalloonItemizedOverlay (
      final android.graphics.drawable.Drawable defaultMarker,
      @org.jetbrains.annotations.NotNull final com.google.android.maps.MapView mapView,
      final int overlayFocusedId,
      final int overlayUnfocusedId,
      final int mainLayoutId,
      final int innerLayoutid,
      final int overlayId,
      final int itemTitleId,
      final int itemSnippetId,
      final int closeButtonId)
   {
      super (defaultMarker);
      this.mainLayoutId = mainLayoutId;
      this.innerLayoutid = innerLayoutid;
      this.overlayId = overlayId;
      this.itemTitleId = itemTitleId;
      this.itemSnippetId = itemSnippetId;
      this.closeButtonId = closeButtonId;

      this.mapView = mapView;
      this.viewOffset = 0;
      this.mc = mapView.getController ();

      final android.content.Context context = mapView.getContext ();
      final android.content.res.Resources resources = context.getResources ();

      this.focusedDrawable = resources.getDrawable (overlayFocusedId);
      this.unFocusedDrawable = resources.getDrawable (overlayUnfocusedId);

   }

   /**
    * Creates the balloon view. Override to create a sub-classed view that can populate
    * additional sub-views.
    */
   @org.jetbrains.annotations.NotNull
   protected BalloonOverlayView <Item> createBalloonOverlayView ()
   {
      @org.jetbrains.annotations.NotNull
      final com.google.android.maps.MapView view = this.getMapView ();

      return new BalloonOverlayView <Item> (
         view.getContext (),
         this.getBalloonBottomOffset (),
         this.overlayId,
         this.itemTitleId,
         this.itemSnippetId,
         this.closeButtonId);
   }

   /**
    * Sets the onTouchListener for the balloon being displayed, calling the overridden
    * {@link #onBalloonTap} method.
    */
   @org.jetbrains.annotations.NotNull
   private android.view.View.OnTouchListener createBalloonTouchListener ()
   {
      return new android.view.View.OnTouchListener ()
      {
         @SuppressWarnings ({
            "MethodWithMultipleReturnPoints" })
         @Override
         public boolean onTouch (
            @org.jetbrains.annotations.NotNull final android.view.View v,
            @org.jetbrains.annotations.NotNull final android.view.MotionEvent event)
         {

            final android.view.View mainLayout = ((android.view.View) v.getParent ()).findViewById (
               BalloonItemizedOverlay.this.mainLayoutId);
            final android.graphics.drawable.Drawable background = mainLayout.getBackground ();

            if (event.getAction () == android.view.MotionEvent.ACTION_DOWN)
            {
               mainLayout.setBackgroundDrawable (BalloonItemizedOverlay.this.focusedDrawable);
               background.invalidateSelf ();
               // int[] states = { android.R.attr.state_pressed };
               // if (background.setState(states)) {
               // background.invalidateSelf();
               // }
               return true;
            }
            else if (event.getAction () == android.view.MotionEvent.ACTION_UP)
            {
               mainLayout.setBackgroundDrawable (BalloonItemizedOverlay.this.unFocusedDrawable);
               background.invalidateSelf ();
               // int newStates[] = {};
               // if (background.setState(newStates)) {
               // background.invalidateSelf();
               // }
               // call overridden method
               BalloonItemizedOverlay.this.onBalloonTap (
                  BalloonItemizedOverlay.this.currentFocussedIndex,
                  BalloonItemizedOverlay.this.currentFocussedItem);
               return true;
            }
            else
            {
               return false;
            }

         }
      };
   }

   public void dispose ()
   {
      if (this.balloonView != null)
      {
         this.balloonView.setVisibility (android.view.View.GONE);
         this.mapView.removeView (this.balloonView);
      }
   }

   public int getBalloonBottomOffset ()
   {
      return this.viewOffset;
   }

   /**
    * Expose map view to subclasses. Helps with creation of balloon views.
    */
   @org.jetbrains.annotations.NotNull
   protected com.google.android.maps.MapView getMapView ()
   {
      return this.mapView;
   }

   /**
    * Sets the visibility of this overlay's balloon view to GONE.
    */
   protected void hideBalloon ()
   {
      if (this.balloonView != null)
      {
         this.balloonView.setVisibility (android.view.View.GONE);
      }
   }

   /**
    * Hides the balloon view for any other BalloonItemizedOverlay instances that might be
    * present on the MapView.
    *
    * @param overlays
    *           - list of overlays (including this) on the MapView.
    */
   private void hideOtherBalloons (
      @org.jetbrains.annotations.NotNull final Iterable <com.google.android.maps.Overlay> overlays)
   {

      for (final com.google.android.maps.Overlay overlay : overlays)
      {
         if ((overlay instanceof BalloonItemizedOverlay <?>) && (overlay != this))
         {
            ((BalloonItemizedOverlay <?>) overlay).hideBalloon ();
         }
      }

   }

   /**
    * Override this method to handle a "tap" on a balloon. By default, does nothing and returns
    * false.
    *
    * @param index
    *           - The index of the item whose balloon is tapped.
    * @param item
    *           - The item whose balloon is tapped.
    * @return true if you handled the tap, otherwise false.
    */
   protected boolean onBalloonTap (final int index, final Item item)
   {
      return false;
   }

   /*
    * (non-Javadoc)
    *
    * @see com.google.android.maps.ItemizedOverlay#onTap(int)
    */
   @Override
   protected final boolean onTap (final int index)
   {

      this.currentFocussedIndex = index;
      this.currentFocussedItem = this.createItem (index);

      final boolean isRecycled;

      if (this.balloonView == null)
      {
         this.balloonView = this.createBalloonOverlayView ();
         final android.view.View clickRegion = this.balloonView.findViewById (this.innerLayoutid);
         clickRegion.setOnTouchListener (this.createBalloonTouchListener ());
         isRecycled = false;
      }
      else
      {
         isRecycled = true;
      }
      this.balloonView.setVisibility (android.view.View.GONE);

      final java.util.List <com.google.android.maps.Overlay> mapOverlays = this.mapView.getOverlays ();

      if (mapOverlays.size () > 1)
      {
         this.hideOtherBalloons (mapOverlays);
      }
      this.balloonView.setData (this.currentFocussedItem);

      final com.google.android.maps.GeoPoint point = this.currentFocussedItem.getPoint ();
      @org.jetbrains.annotations.NotNull
      final com.google.android.maps.MapView.LayoutParams params = new com.google.android.maps.MapView.LayoutParams (
         android.view.ViewGroup.LayoutParams.WRAP_CONTENT,
         android.view.ViewGroup.LayoutParams.WRAP_CONTENT,
         point,
         com.google.android.maps.MapView.LayoutParams.BOTTOM_CENTER);

      params.mode = com.google.android.maps.MapView.LayoutParams.MODE_MAP;

      this.balloonView.setVisibility (android.view.View.VISIBLE);

      if (isRecycled)
      {
         this.balloonView.setLayoutParams (params);
      }
      else
      {
         this.mapView.addView (this.balloonView, params);
      }
      this.mc.animateTo (point);

      return true;
   }

   /**
    * Set the horizontal distance between the marker and the bottom of the information balloon.
    * The default is 0 which works well for center bounded markers. If your marker is
    * center-bottom bounded, call this before adding overlay items to ensure the balloon hovers
    * exactly above the marker.
    *
    * @param pixels
    *           - The padding between the center point and the bottom of the information
    *           balloon.
    */
   public void setBalloonBottomOffset (final int pixels)
   {
      this.viewOffset = pixels;
   }

}
