/*
 * ************************************************************ {{{1 **********
 * Copyright © 2005 … 2017  "Martin Krischik" «krischik@users.sourceforge.net»
 * **************************************************************************
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see «http://www.gnu.org/licenses»..
 * **************************************************************************
 * $Author: krischik $
 * $Revision: 7137 $
 * $Date: 2020-09-22 10:50:39 +0200 (Di, 22. Sep 2020) $
 * $Id: IProperties.java 7137 2020-09-22 08:50:39Z krischik $
 * $HeadURL: svn+ssh://krischik@svn.code.sf
 * .net/p/uiq3/code/trunk/Java/Calculator/Desktop/src/main/java/net/sourceforge/uiq3/calculator/IProperties.java $
 * ************************************************************ }}}1 **********
 */
package net.sourceforge.uiq3.calculator

@java.beans.JavaBean(description = "Calculator properties")
interface IProperties :
   net.sourceforge.uiq3.IPropertyNotifier
{
   /**
    * Looks and feel scaling. Small is 80% and large is 125% or normal.
    */
   enum class Look_And_Feel_Size_Type(val scale: Float)
   {
      Small(0.8f),
      Medium(1.0f),
      Large(1.25f);

      /**
       * derive a new font using the current scaling factor.
       */
      fun deriveFont(original: java.awt.Font) = original.deriveFont(original.size2D * scale)

      /**
       * derive a new dimension using the current scaling factor.
       */
      fun deriveDimension(original: java.awt.Dimension) = Utils.resize(
	 original = original,
	 scaleWidth = scale.toDouble(),
	 scaleHeight = scale.toDouble()
      )
   }

   object Defaults
   {
      const val Look_And_Feel_Name: String = "com.formdev.flatlaf.FlatLightLaf"
      const val Font_Base_Size = 12.0f

      val Look_And_Feel_Size = Look_And_Feel_Size_Type.Medium
      val Font_Small_Scale = Look_And_Feel_Size_Type.Small.scale
      val Font_Large_Scale = Look_And_Feel_Size_Type.Large.scale

      private val Mono_Base_Font = java.awt.Font(
	 /* name  = */ "DejaVu Sans Mono",
	 /* style = */ java.awt.Font.PLAIN,
	 /* size  = */ this.Font_Base_Size.toInt()
      )
      private val Sans_Base_Font = java.awt.Font(
	 /* name  = */ "DejaVu Sans",
	 /* style = */ java.awt.Font.PLAIN,
	 /* size  = */ this.Font_Base_Size.toInt()
      )
      private val Serif_Base_Font = java.awt.Font(
	 /* name  = */ "DejaVu Serif",
	 /* style = */ java.awt.Font.PLAIN,
	 /* size  = */ this.Font_Base_Size.toInt()
      )

      val Button_Size = java.awt.Dimension(72, 18)
      val Button_Arc = 8
      val Button_Style = "rect"
      val Mono_Font = Mono_Base_Font.deriveFont(Font_Base_Size)
      val Mono_Small_Font = Mono_Base_Font.deriveFont(Font_Base_Size * Font_Small_Scale)
      val Mono_Large_Font = Mono_Base_Font.deriveFont(Font_Base_Size * Font_Large_Scale)
      val Dialog_Font = Sans_Base_Font.deriveFont(Font_Base_Size)
      val Dialog_Small_Font = Sans_Base_Font.deriveFont(Font_Base_Size * Font_Small_Scale)
      val Dialog_Large_Font = Sans_Base_Font.deriveFont(Font_Base_Size * Font_Large_Scale)
      val Text_Font = Serif_Base_Font.deriveFont(Font_Base_Size)
      val Text_Small_Font = Serif_Base_Font.deriveFont(Font_Base_Size * Font_Small_Scale)
      val Text_Large_Font = Serif_Base_Font.deriveFont(Font_Base_Size * Font_Large_Scale)

      /**
       * The simulators home directory. The usual positions are:
       *
       *  * On Windows: "C:/Users/user_id/FX-602P/"
       *  * On OS X: "/Users/user_id/FX-602P/"
       *  * On Unix: "/home/user_id/FX-602P/"
       *  * On Symbian: "D:/others/user_id/FX-602P/"
       *
       * TODO: use more modern location for each OS.
       *
       * @return The home directory to be used.
       */
      val Home_Directory = lazy {
	 logger.entering(TAG, "Home_Directory")

	 var Retval: String

	 try
	 {
	    Retval = System.getProperty(HOME_DIRECTORY_ID) + "/FX-602P"

	    val Home = java.io.File(Retval)

	    Home.mkdirs()
	 }
	 catch (Exception: SecurityException)
	 {
	    logger.log(
	       /* level  = */ java.util.logging.Level.WARNING,
	       /* msg    = */"LOG00580: Can't access Home Directory, probably running in with Webstart",
	       /* thrown = */ Exception
	    )
	    Retval = "FX-602P"
	 } // try

	 logger.exiting(TAG, "Home_Directory", Retval)
	 return@lazy Retval
      } // Home_Directory
   }

   companion object
   {
      /**
       * class logger instance
       */
      private val TAG: String

      /**
       * class logger instance
       */
      private val logger: java.util.logging.Logger

      /**
       *  parse numbers in default format.
       */
      private val Root_Format = java.text.NumberFormat.getInstance(java.util.Locale.ROOT)

      /**
       * property change events. Also used as names in the property file.
       */
      const val LOOK_AND_FEEL_NAME_ID: String = "Style.name"
      const val LOOK_AND_FEEL_SIZE_ID: String = "Style.size"
      const val MONO_FONT_ID = java.awt.Font.MONOSPACED + ".font"
      const val DIALOG_FONT_ID = java.awt.Font.DIALOG + ".font"
      const val DIALOG_LARGE_FONT_ID = java.awt.Font.DIALOG + "_Large.font"
      const val DIALOG_SMALL_FONT_ID = java.awt.Font.DIALOG + "_Small.font"
      const val MONO_LARGE_FONT_ID = java.awt.Font.MONOSPACED + "_Large.font"
      const val MONO_SMALL_FONT_ID = java.awt.Font.MONOSPACED + "_Small.font"
      const val TEXT_FONT_ID = java.awt.Font.SERIF + ".font"
      const val TEXT_LARGE_FONT_ID = java.awt.Font.SERIF + "_Large.font"
      const val TEXT_SMALL_FONT_ID = java.awt.Font.SERIF + "_Small.font"
      const val BUTTON_SIZE_ID = "Button.size"
      const val HOME_DIRECTORY_ID = "user.home"

      private const val Bundle_Name = "net/sourceforge/uiq3/calculator/Bundle"
      private val Calculator_Bundle = lazy { java.util.ResourceBundle.getBundle(Bundle_Name) }

      init
      {
	 this.TAG = IProperties::class.java.name
	 this.logger = java.util.logging.Logger.getLogger(this.TAG)
      }

      /**
       * Encode a full font name
       */
      fun encode(Default: java.awt.Font) =
	 Default.name + "-" +
	    when (Default.style)
	    {
	       1    -> "BOLD"
	       2    -> "ITALIC"
	       3    -> "BOLDITALIC"
	       else -> "PLAIN"
	    } + "-" +
	    String.format(java.util.Locale.ROOT, format = "%.2f", Default.size2D)
     
      fun longToColor(Default: Long) = java.awt.Color(
	 /* rgba     = */ Default.toInt(),
	 /* hasalpha = */ true
      )

      /**
       * get color from properties.
       *
       * @return a font size
       */
      fun readColor(P: java.util.Properties, name: String, default: Long): java.awt.Color
      {
	 this.logger.entering(this.TAG, "readColor", arrayOf(P, name, default))

	 var retval: java.awt.Color

	 try
	 {
	    val Value_Text = P.getProperty(name)
	    val Value_Int: Long

	    if (Value_Text != null)
	    {
	       Value_Int = java.lang.Long.valueOf(Value_Text, 16)
	    }
	    else
	    {
	       Value_Int = default
	       P.setProperty(name, java.lang.Long.toString(Value_Int, 16))
	    }

	    retval = this.longToColor(Value_Int)
	 }
	 catch (Exception: Exception)
	 {
	    this.logger.log(
	       java.util.logging.Level.WARNING,
	       "LOG00570: Color ${name} Default color ${default} size will be used!",
	       Exception
	    )
	    retval = this.longToColor(default)
	    P.setProperty(name, java.lang.Long.toString(default, 16))
	 } // try

	 this.logger.exiting(this.TAG, "readColor", retval)
	 return retval
      } // readColor

      /**
       * get color from properties.
       *
       * @return a font size
       */
      fun readDimension(P: java.util.Properties, name: String, default: java.awt.Dimension): java.awt.Dimension
      {
	 this.logger.entering(this.TAG, "readDimension", arrayOf(P, name, default))

	 val height = this.readInteger(P, name + ".height", default.height)
	 val width = this.readInteger(P, name + ".width", default.width)
	 val retval = java.awt.Dimension(width, height)

	 this.logger.exiting(this.TAG, "readDimension", retval)
	 return retval
      } // readColor

      /**
       * get color from properties.
       *
       * @return a font size
       */
      fun writeDimension(P: java.util.Properties, name: String, value: java.awt.Dimension)
      {
	 this.logger.entering(this.TAG, "readDimension", arrayOf(P, name, value))

	 P.setProperty(name + ".height", Integer.toString(value.height, 10))
	 P.setProperty(name + ".width", Integer.toString(value.width, 10))

	 this.logger.exiting(this.TAG, "readDimension")
	 return
      } // readColor

      /**
       * get color from properties.
       *
       * @return a font size
       */
      fun readFont(P: java.util.Properties, name: String, default: java.awt.Font): java.awt.Font
      {
	 this.logger.entering(this.TAG, "readFont", arrayOf(P, name, default))

	 var retval: java.awt.Font

	 try
	 {
	    val Text = P.getProperty(name)

	    if (Text != null)
	    {
	       val parts = Text.split("-")
	       val size = Root_Format.parse(parts[2]).toFloat()

	       retval = java.awt.Font.decode(parts[0] + "-" + parts[1] + "-12").deriveFont(size)
	    }
	    else
	    {
	       P.setProperty(name, this.encode(default))

	       retval = default
	    }
	 }
	 catch (Exception: Exception)
	 {
	    this.logger.log(
	       java.util.logging.Level.WARNING,
	       "LOG00570: Font ${name} Default ${default} will be used!",
	       Exception
	    )

	    P.setProperty(name, this.encode(default))
	    retval = default
	 } // try

	 this.logger.exiting(this.TAG, "readFont", retval)
	 return retval
      } // readFont

      /**
       * get integer from properties.
       *
       * @return a integer
       */
      fun readInteger(P: java.util.Properties, name: String, default: Int): Int
      {
	 this.logger.entering(this.TAG, "readInteger", arrayOf(name, default))

	 var Retval: Int

	 try
	 {
	    val Value_Text = P.getProperty(name)
	    val Value_Int: Int

	    if (Value_Text != null)
	    {
	       Value_Int = Integer.valueOf(Value_Text, 10)
	    }
	    else
	    {
	       Value_Int = default
	       P.setProperty(name, Integer.toString(Value_Int, 10))
	    }

	    Retval = Value_Int
	 }
	 catch (Exception: Exception)
	 {
	    this.logger.log(
	       java.util.logging.Level.WARNING,
	       "LOG00570: Color ${name} Default color ${default} size will be used!",
	       Exception
	    )
	    Retval = default
	    P.setProperty(name, Integer.toString(default, 16))
	 } // try

	 this.logger.exiting(this.TAG, "readInteger", Retval)
	 return Retval
      } // readInteger

      /**
       * get color from properties.
       *
       * @return a font size
       */
      fun readString(P: java.util.Properties, name: String, default: String): String
      {
	 this.logger.entering(this.TAG, "readString", arrayOf(name, default))

	 var Retval: String

	 try
	 {
	    val Text = P.getProperty(name)

	    if (Text != null)
	    {
	       Retval = Text
	    }
	    else
	    {
	       P.setProperty(name, default)
	       Retval = default
	    }
	 }
	 catch (Exception: Exception)
	 {
	    this.logger.log(
	       /* level = */ java.util.logging.Level.WARNING,
	       /* msg = */ "LOG00570: could not get string ${name} default „${default}“ will be used!",
	       /* thrown = */ Exception
	    )
	    P.setProperty(name, default)
	    Retval = default
	 } // try

	 this.logger.exiting(this.TAG, "readString", Retval)
	 return Retval
      } // readString
   } //  companion object

   /**
    *  Library resource bundle
    */
   val libraryBundle: java.util.ResourceBundle
      get() = Calculator_Bundle.value

   /**
    *  Application resource bundle
    */
   val applicationBundle: java.util.ResourceBundle

   /**
    * Get the raw properties.
    *
    * @return properties
    */
   val data: java.util.Properties

   val Properties_Filename: java.io.File

   val Accent_Color: Lazy<java.awt.Color>

   val Background_Dark_Color: Lazy<java.awt.Color>

   val Background_Light_Color: Lazy<java.awt.Color>

   /**
    * Font used for monospaced text.
    */
   @get:java.beans.BeanProperty(bound = true, description = "Font used for monospaced text.")
   var Mono_Font: java.awt.Font

   /**
    * Large font used for monospaced text.
    */
   @get:java.beans.BeanProperty(bound = true, description = "Large font used for monospaced text.")
   var Mono_Large_Font: java.awt.Font

   /**
    * Small font used for monospaced text.
    */
   @get:java.beans.BeanProperty(bound = true, description = "Small font used for monospaced text.")
   var Mono_Small_Font: java.awt.Font

   /**
    * Font used for dialog text.
    */
   @get:java.beans.BeanProperty(bound = true, description = "Font used for dialog text.")
   var Dialog_Font: java.awt.Font

   /**
    * Large font used for dialog text.
    */
   @get:java.beans.BeanProperty(bound = true, description = "Large font used for dialog text.")
   var Dialog_Large_Font: java.awt.Font

   /**
    * Small font used for dialog text.
    */
   @get:java.beans.BeanProperty(bound = true, description = "Small font used for dialog text.")
   var Dialog_Small_Font: java.awt.Font

   /**
    * Ffont used for body text.
    */
   @get:java.beans.BeanProperty(bound = true, description = "Font used for body text.")
   var Text_Font: java.awt.Font

   /**
    * Large font used for body text.
    */
   @get:java.beans.BeanProperty(bound = true, description = "Large font used for body text.")
   var Text_Large_Font: java.awt.Font

   /**
    * Small font used for body text.
    */
   @get:java.beans.BeanProperty(bound = true, description = "Small font used for body text.")
   var Text_Small_Font: java.awt.Font

   /**
    * Minimum size for calculator buttons
    */
   @get:java.beans.BeanProperty(bound = true, description = "Minimum size for calculator buttons")
   var Button_Size: java.awt.Dimension

   /**
    * Define Button corner arc
    */
   @get:java.beans.BeanProperty(bound = true, description = "Radius for calculator buttons")
   val Button_Arc: Lazy<Int>

   /**
    * Define Button corner style
    */
   @get:java.beans.BeanProperty(bound = true, description = "Style for calculator buttons")
   val Button_Style: Lazy<String>

   /**
    * more information's for error messages
    */
   val Exception_Info: String

   /**
    * The simulators home directory. The usual positions are:
    *
    *  * On Windows: "C:/Users/user_id/FX-603P/"
    *  * On OS X: "/Users/user_id/FX-602P/"
    *  * On Unix: "/home/user_id/FX-602P/"
    *  * On Symbian: "D:/others/user_id/FX-602P/"
    *
    *
    * @return The home directory to be used.
    */
   var Home_Directory: String

   /**
    * Look and Feel class name
    */
   var Look_And_Feel_Name: String

   /**
    * Looks and feel scaling. Small is 80% and large is 125% of normal.
    */
   var Look_And_Feel_Size: Look_And_Feel_Size_Type

   /**
    * get color from properties.
    *
    * @return a font size
    */
   fun readDimension(name: String, default: java.awt.Dimension) = readDimension(this.data, name, default)

   /**
    * get color from properties.
    *
    * @return a font size
    */
   fun readColor(name: String, default: Long) = readColor(this.data, name, default)

   /**
    * get color from properties.
    *
    * @return a font size
    */
   fun readFont(name: String, default: java.awt.Font) = readFont(this.data, name, default)

   /**
    * get integer from properties.
    *
    * @return a integer
    */
   fun readInteger(name: String, default: Int) = readInteger(this.data, name, default)

   /**
    * get string from properties.
    *
    * @return a string size
    */
   fun readString(name: String, default: String): String = readString(this.data, name, default)

   /**
    * Save Property file
    */
   fun saveProperties()

   /**
    * Restore calculator status
    *
    * @param calculatorEngine
    * calculator
    */
   fun restoreCalculatorStatus(calculatorEngine: ICalculator)

   /**
    * Restore window status
    *
    * @param window
    * windows to save.
    */
   fun restoreWindowStatus(window: java.awt.Dialog)
   {
      logger.entering(TAG, "restoreWindowStatus", window)

      try
      {
	 val P = this.data
	 val Window_Name = window.name
	 val Location = java.awt.Point(
	    /* x = */ P.getProperty("${Window_Name}.x").toInt(),
	    /* y = */ P.getProperty("${Window_Name}.y").toInt()
	 )
	 val Size = java.awt.Dimension(
	    /* width  = */ P.getProperty("${Window_Name}.width").toInt(),
	    /* height = */ P.getProperty("${Window_Name}.height").toInt()
	 )

	 window.location = Location
	 window.size = Size
      }
      catch (Exception: Exception)
      {
	 window.pack()
	 logger.log(java.util.logging.Level.WARNING, "LOG00640: Default window size will be used!", Exception)
      } // try

      logger.exiting(TAG, "restoreWindowStatus")
      return
   } // Restore_Window_Status

   /**
    * Restore window status
    *
    * @param window
    * windows to save.
    */
   fun restoreWindowStatus(window: java.awt.Frame)
   {
      logger.entering(TAG, "restoreWindowStatus", window)

      val P = this.data

      try
      {
	 val Window_Name = window.name
	 val Location = java.awt.Point(
	    P.getProperty("${Window_Name}.x").toInt(),
	    P.getProperty("${Window_Name}.y").toInt()
	 )
	 val Size = java.awt.Dimension(
	    P.getProperty("${Window_Name}.width").toInt(),
	    P.getProperty("${Window_Name}.height").toInt()
	 )
	 val Visible = java.lang.Boolean.parseBoolean(P.getProperty("${Window_Name}.visible"))

	 window.focusableWindowState = false
	 window.location = Location
	 window.size = Size
	 window.isVisible = Visible
	 window.focusableWindowState = true
      }
      catch (Exception: Exception)
      {
	 window.pack()
	 logger.log(
	    java.util.logging.Level.SEVERE,
	    "LOG00650: Window Position not read. " + this.Exception_Info,
	    Exception
	 )
      } // try

      logger.exiting(TAG, "restoreWindowStatus")
      return
   } // Restore_Window_Status

   /**
    * Save split plane location
    *
    * @param split
    * split pane to save.
    */
   fun restoreSplitStatus(split: javax.swing.JSplitPane)
   {
      val windowName = split.name

      val location = readInteger("${windowName}.location", -1)
      if (location > 0)
      {
	 split.dividerLocation = location
      }
   } // restoreSplitStatus

   /**
    * Save calculator status
    *
    * @param calculatorEngine
    * Calculator
    */
   fun saveCalculatorStatus(calculatorEngine: ICalculator)

   /**
    * Save window status
    *
    * @param window
    * windows to save.
    */
   fun saveWindowStatus(window: java.awt.Dialog)
   {
      val P = this.data
      val Window_Name = window.name
      val Location = window.location
      val Size = window.size

      P.setProperty("${Window_Name}.x", Integer.toString(Location.x))
      P.setProperty("${Window_Name}.y", Integer.toString(Location.y))
      P.setProperty("${Window_Name}.height", Integer.toString(Size.height))
      P.setProperty("${Window_Name}.width", Integer.toString(Size.width))

      return
   } // Save_Window_Status

   /**
    * Save window status
    *
    * @param window
    * windows to save.
    */
   fun saveWindowStatus(window: java.awt.Frame)
   {
      val P = this.data
      val Window_Name = window.name
      val Location = window.location
      val Size = window.size
      val Visible = window.isVisible

      P.setProperty("${Window_Name}.x", Integer.toString(Location.x))
      P.setProperty("${Window_Name}.y", Integer.toString(Location.y))
      P.setProperty("${Window_Name}.height", Integer.toString(Size.height))
      P.setProperty("${Window_Name}.width", Integer.toString(Size.width))
      P.setProperty("${Window_Name}.visible", java.lang.Boolean.toString(Visible))

      return
   } // Save_Window_Status

   /**
    * Save split plane location
    *
    * @param split
    * split pane to save.
    */
   fun saveSplitStatus(split: javax.swing.JSplitPane)
   {
      val P = this.data
      val windowName = split.name
      val location = split.dividerLocation

      P.setProperty("${windowName}.location", Integer.toString(location))

      return
   } // saveSplitStatus
} // IProperties

// vim: set nowrap tabstop=8 shiftwidth=4 softtabstop=4 expandtab :
// vim: set textwidth=0 filetype=java foldmethod=syntax nospell :

