/*
 * ************************************************************ {{{1 **********
 * Copyright © 2005 … 2024  "Martin Krischik" «krischik@users.sourceforge.net»
 * **************************************************************************
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see «http://www.gnu.org/licenses»..
 * ************************************************************ }}}1 **********
 */
package net.sourceforge.uiq3.calculator

import net.sourceforge.uiq3.ui.ICalculatorCanvas
import java.beans.JavaBean
import java.beans.PropertyChangeListener
import java.beans.PropertyChangeSupport
import java.util.logging.Logger

/**
 * Java standard edition keyboard
 *
 * @author "Martin Krischik" «krischik@users.sourceforge.net»
 * @param keyNr GUI identification. Either the number in the keyboard array or the Android widget ID.
 * @param calculatorKeyCode Key Code used inside the Calculator
 * @param javaKeyCode Key Code used inside the Java
 */
@JavaBean
abstract class Key
protected constructor(
   override val keyNr: Int,
   override val calculatorKeyCode: Short,
   override val javaKeyCode: Int) :
   IKey
{
   companion object
   {
      /**
       * class logger instance
       */
      private val logger: Logger

      /**
       * class logger tag.
       */
      private val TAG: String = Key::class.java.name

      init
      {
	 logger = Logger.getLogger(TAG)
      }
   }

   /**
    * property change support.
    */
   protected val propertyChangeSupport = PropertyChangeSupport(this)

   /**
    * canvas owning the key.
    * Canvas attached to the Key
    *
    * @return the canvas
    */
   var canvas: ICalculatorCanvas? = null
      set(canvas)
      {
	 logger.entering(TAG, "setCanvas", canvas)

	 val oldCanvas = field
	 field = canvas

	 propertyChangeSupport.firePropertyChange("canvas", oldCanvas, canvas)

	 logger.exiting(TAG, "setCanvas")
	 return
      } // setCanvas

   fun addPropertyChangeListener(listener: PropertyChangeListener?)
   {
      propertyChangeSupport.addPropertyChangeListener(listener)
   }

   fun removePropertyChangeListener(listener: PropertyChangeListener?)
   {
      propertyChangeSupport.removePropertyChangeListener(listener)
   }

   /**
    * check if key with Java Key code has been hit.
    *
    * @param Java_Key_Code
    * Key code from the Java virtual machine
    * @return true when key was hit
    */
   override fun isKeyHit(Java_Key_Code: Int): Boolean
   {
      return this.javaKeyCode == Java_Key_Code
   } // isKeyHit

   override fun toString(): String
   {
      return "Key{" +
	 "calculatorKeyCode=" + calculatorKeyCode +
	 ", javaKeyCode=" + javaKeyCode +
	 '}'
   }

} // SE_Key
// vim: set nowrap tabstop=8 shiftwidth=4 softtabstop=4 expandtab :
// vim: set textwidth=0 filetype=java foldmethod=syntax nospell :

