/*
 * ********************************************************* {{{1 ************
 *   Copyright © 2005 … 2012 "Martin Krischik" «krischik@users.sourceforge.net»
 * ***************************************************************************
 *   This program is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see «http://www.gnu.org/licenses».
 * ********************************************************** }}}1 ***********
 */
package net.sourceforge.uiq3.calculator

/**
 * Application properties singleton.
 *
 * @author Martin Krischik
 */
abstract class Properties :
   IProperties
{

   /**
    * class logger instance
    */
   private val Logger: java.util.logging.Logger

   /**
    * Handle property changes. We don't use the full feature with oldValue and new newValue as that would be overkill.
    */
   protected val propertyChangeSupport = java.beans.PropertyChangeSupport(this);

   /**
    * class logger tag.
    */
   private val TAG: String

   init
   {
      this.TAG = Properties::class.java.name
      this.Logger = java.util.logging.Logger.getLogger(this.TAG)

      this.addPropertyChangeListener { event ->
	 val size = event.newValue

	 if (size is IProperties.Look_And_Feel_Size_Type)
	 {
	    lookAndFeelChanged(size)
	 }
      }
   }

   /**
    * Get the properties.
    */
   protected val self = lazy {
      this.Logger.entering(this.TAG, "self")

      val retval = java.util.Properties()

      try
      {
	 java.io.FileInputStream(this.Properties_Filename).use { Reader ->
	    retval.load(Reader)
	 }
      }
      catch (Exception: java.io.FileNotFoundException)
      {
	 this.Logger.log(
	    /* level  = */ java.util.logging.Level.SEVERE,
	    /* msg    = */ "LOG00600:" + this.Exception_Info,
	    /* thrown = */ Exception
	 )
      }
      catch (Exception: java.io.IOException)
      {
	 this.Logger.log(
	    /* level  = */ java.util.logging.Level.SEVERE,
	    /* msg    = */ "LOG00610:" + this.Exception_Info,
	    /* thrown = */ Exception
	 )
      } // try

      this.Logger.exiting(this.TAG, "self", retval)

      return@lazy retval
   } // self

   /**
    * more information's for error messages
    */
   override val Exception_Info: String
      get() = "In property file " + this.Properties_Filename

   /**
    * Get the raw properties.
    *
    * @return properties
    */
   override val data: java.util.Properties
      get() = this.self.value

   /**
    * The simulators home directory. The usual positions are:
    *
    *  * On Windows: "C:/Users/user_id/FX-603P/"
    *  * On OS X: "/Users/user_id/FX-602P/"
    *  * On Unix: "/home/user_id/FX-602P/"
    *  * On Symbian: "D:/others/user_id/FX-602P/"
    *
    *
    * @return The home directory to be used.
    */
   override var Home_Directory
      get() = IProperties.readString(
	 this.self.value,
	 IProperties.HOME_DIRECTORY_ID,
	 IProperties.Defaults.Home_Directory.value
      )
      set(value)
      {
	 val oldValue = this.Home_Directory
	 this.self.value.setProperty(IProperties.HOME_DIRECTORY_ID, value)
	 this.propertyChangeSupport.firePropertyChange(IProperties.HOME_DIRECTORY_ID, oldValue, value);
      }


   /**
    * making the buttons smaller will make it unreadable.
    */
   override val Button_Arc = lazy { this.readInteger("Button.arc", IProperties.Defaults.Button_Arc) }

   /**
    * making the buttons smaller will make it unreadable.
    */
   override val Button_Style = lazy { this.readString("Button.style", IProperties.Defaults.Button_Style) }

   /**
    * Looks and feel scaling. Small is 80% and large is 120% of normal.
    */
   @get:java.beans.BeanProperty(bound = true, description = "Looks and feel scaling.")
   override var Look_And_Feel_Size
      get() = IProperties.Look_And_Feel_Size_Type.valueOf(
	 IProperties.readString(
	    this.self.value,
	    IProperties.LOOK_AND_FEEL_SIZE_ID,
	    IProperties.Defaults.Look_And_Feel_Size.name
	 )
      )
      set(value)
      {
	 val oldValue = this.Look_And_Feel_Size
	 this.self.value.setProperty(IProperties.LOOK_AND_FEEL_SIZE_ID, value.name)
	 this.propertyChangeSupport.firePropertyChange(IProperties.LOOK_AND_FEEL_SIZE_ID, oldValue, value);
      }

   /**
    * making the buttons smaller will make it unreadable.
    */
   @get:java.beans.BeanProperty(bound = true, description = "Requested button minimum size.")
   override var Button_Size
      get() = this.readDimension(IProperties.BUTTON_SIZE_ID, IProperties.Defaults.Button_Size)
      set(value)
      {
	 val oldValue = this.Button_Size
	 IProperties.writeDimension(this.self.value, IProperties.BUTTON_SIZE_ID, value)
	 this.propertyChangeSupport.firePropertyChange(IProperties.BUTTON_SIZE_ID, oldValue, value);
      }

   /**
    * Font used for monospaced text.
    */
   @get:java.beans.BeanProperty(bound = true, description = "Font used for monospaced text.")
   override var Mono_Font: java.awt.Font
      get() = this.readFont(IProperties.MONO_FONT_ID, IProperties.Defaults.Mono_Font)
      set(value)
      {
	 val oldValue = this.Mono_Font
	 this.self.value.setProperty(IProperties.MONO_FONT_ID, IProperties.encode(value))
	 this.propertyChangeSupport.firePropertyChange(IProperties.MONO_FONT_ID, oldValue, value);
      }

   /**
    * Large font used for monospaced text.
    */
   @get:java.beans.BeanProperty(bound = true, description = "Large font used for monospaced text.")
   override var Mono_Large_Font: java.awt.Font
      get() = this.readFont(IProperties.MONO_LARGE_FONT_ID, IProperties.Defaults.Mono_Large_Font)
      set(value)
      {
	 val oldValue = this.Mono_Large_Font
	 this.self.value.setProperty(IProperties.MONO_LARGE_FONT_ID, IProperties.encode(value))
	 this.propertyChangeSupport.firePropertyChange(IProperties.MONO_LARGE_FONT_ID, oldValue, value);
      }

   /**
    * Small font used for monospaced text.
    */
   @get:java.beans.BeanProperty(bound = true, description = "Small font used for monospaced text.")
   override var Mono_Small_Font: java.awt.Font
      get() = this.readFont(IProperties.MONO_SMALL_FONT_ID, IProperties.Defaults.Mono_Small_Font)
      set(value)
      {
	 val oldValue = this.Mono_Small_Font
	 this.self.value.setProperty(IProperties.MONO_SMALL_FONT_ID, IProperties.encode(value))
	 this.propertyChangeSupport.firePropertyChange(IProperties.MONO_SMALL_FONT_ID, oldValue, value);
      }

   /**
    * Font used for dialog text.
    */
   @get:java.beans.BeanProperty(bound = true, description = "Font used for dialog text.")
   override var Dialog_Font: java.awt.Font
      get() = this.readFont(IProperties.DIALOG_FONT_ID, IProperties.Defaults.Dialog_Font)
      set(value)
      {
	 val oldValue = this.Dialog_Font
	 this.self.value.setProperty(IProperties.DIALOG_FONT_ID, IProperties.encode(value))
	 this.propertyChangeSupport.firePropertyChange(IProperties.DIALOG_FONT_ID, oldValue, value);
      }

   /**
    * Large font used for dialog text.
    */
   @get:java.beans.BeanProperty(bound = true, description = "Large font used for dialog text.")
   override var Dialog_Large_Font: java.awt.Font
      get() = this.readFont(IProperties.DIALOG_LARGE_FONT_ID, IProperties.Defaults.Dialog_Large_Font)
      set(value)
      {
	 val oldValue = this.Dialog_Large_Font
	 this.self.value.setProperty(IProperties.DIALOG_LARGE_FONT_ID, IProperties.encode(value))
	 this.propertyChangeSupport.firePropertyChange(IProperties.DIALOG_LARGE_FONT_ID, oldValue, value);
      }

   /**
    * Small font used for dialog text.
    */
   @get:java.beans.BeanProperty(bound = true, description = "Small font used for dialog text.")
   override var Dialog_Small_Font: java.awt.Font
      get() = this.readFont(IProperties.DIALOG_SMALL_FONT_ID, IProperties.Defaults.Dialog_Small_Font)
      set(value)
      {
	 val oldValue = this.Dialog_Small_Font
	 this.self.value.setProperty(IProperties.DIALOG_SMALL_FONT_ID, IProperties.encode(value))
	 this.propertyChangeSupport.firePropertyChange(IProperties.DIALOG_SMALL_FONT_ID, oldValue, value);
      }

   /**
    * Ffont used for body text.
    */
   @get:java.beans.BeanProperty(bound = true, description = "Lont used for body text.")
   override var Text_Font: java.awt.Font
      get() = this.readFont(IProperties.TEXT_FONT_ID, IProperties.Defaults.Text_Font)
      set(value)
      {
	 val oldValue = this.Text_Font
	 this.self.value.setProperty(IProperties.TEXT_FONT_ID, IProperties.encode(value))
	 this.propertyChangeSupport.firePropertyChange(IProperties.TEXT_FONT_ID, oldValue, value);
      }

   /**
    * Large font used for body text.
    */
   @get:java.beans.BeanProperty(bound = true, description = "Large font used for body text.")
   override var Text_Large_Font: java.awt.Font
      get() = this.readFont(IProperties.TEXT_LARGE_FONT_ID, IProperties.Defaults.Text_Large_Font)
      set(value)
      {
	 val oldValue = this.Text_Large_Font
	 this.self.value.setProperty(IProperties.TEXT_LARGE_FONT_ID, IProperties.encode(value))
	 this.propertyChangeSupport.firePropertyChange(IProperties.TEXT_LARGE_FONT_ID, oldValue, value);
      }

   /**
    * Small font used for body text.
    */
   @get:java.beans.BeanProperty(bound = true, description = "Small font used for body text.")
   override var Text_Small_Font: java.awt.Font
      get() = this.readFont(IProperties.TEXT_SMALL_FONT_ID, IProperties.Defaults.Text_Small_Font)
      set(value)
      {
	 val oldValue = this.Text_Small_Font
	 this.self.value.setProperty(IProperties.TEXT_SMALL_FONT_ID, IProperties.encode(value))
	 this.propertyChangeSupport.firePropertyChange(IProperties.TEXT_SMALL_FONT_ID, oldValue, value);
      }

   /**
    * Look and feel was changed
    */
   protected open fun lookAndFeelChanged(size: IProperties.Look_And_Feel_Size_Type)
   {
      this.Logger.entering(this.TAG, "Look_And_Feel_Changed", size)

      Mono_Font = size.deriveFont(IProperties.Defaults.Mono_Font)
      Mono_Large_Font = size.deriveFont(IProperties.Defaults.Mono_Large_Font)
      Mono_Small_Font = size.deriveFont(IProperties.Defaults.Mono_Small_Font)
      Dialog_Font = size.deriveFont(IProperties.Defaults.Dialog_Font)
      Dialog_Large_Font = size.deriveFont(IProperties.Defaults.Dialog_Large_Font)
      Dialog_Small_Font = size.deriveFont(IProperties.Defaults.Dialog_Small_Font)
      Text_Font = size.deriveFont(IProperties.Defaults.Text_Font)
      Text_Large_Font = size.deriveFont(IProperties.Defaults.Text_Large_Font)
      Text_Small_Font = size.deriveFont(IProperties.Defaults.Text_Small_Font)
      Button_Size = size.deriveDimension(IProperties.Defaults.Button_Size)

      this.Logger.entering(this.TAG, "Look_And_Feel_Changed")
      return;
   } // lookAndFeelChanged

   override fun addPropertyChangeListener(listener: java.beans.PropertyChangeListener)
   {
      propertyChangeSupport.addPropertyChangeListener(listener)
   } // addPropertyChangeListener

   override fun removePropertyChangeListener(listener: java.beans.PropertyChangeListener)
   {
      propertyChangeSupport.removePropertyChangeListener(listener)
   } // removePropertyChangeListener
} // Properties

// vim: set nowrap tabstop=8 shiftwidth=4 softtabstop=4 expandtab :
// vim: set textwidth=0 filetype=kotlin foldmethod=syntax nospell :
