/*
 * ************************************************************ {{{1 **********
 * Copyright © 2005 … 2024  "Martin Krischik" «krischik@users.sourceforge.net»
 * **************************************************************************
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see «http://www.gnu.org/licenses»..
 * ************************************************************ }}}1 **********
 */
package net.sourceforge.uiq3.calculator

/**
 * @author "Martin Krischik" «krischik@users.sourceforge.net»
 */
object Utils
{
   /**
    *
    *  class logger instance
    */
   private val logger: java.util.logging.Logger

   /**
    *
    *  class logger tag.
    */
   private val TAG: String = Utils::class.java.name

   init
   {
      logger = java.util.logging.Logger.getLogger(TAG)
   }

   /**
    * Read image from jar file
    *
    * @param File_Name
    * Filename of the Image
    * @return font read in background.
    */
   @JvmStatic
   fun createFont(File_Name: String): java.awt.Font?
   {
      var Retval: java.awt.Font?

      try
      {
	 Utils::class.java.getResourceAsStream(File_Name).use { File_Stream ->
	    Retval = java.awt.Font.createFont(
	       /* fontFormat */java.awt.Font.TRUETYPE_FONT,
	       /* fontStream */File_Stream
	    )
	 }
      }
      catch (Exception: java.awt.FontFormatException)
      {
	 logger.log(java.util.logging.Level.SEVERE, "LOG01180:Error in font file format.", Exception)
	 Retval = null
      }
      catch (Exception: java.io.IOException)
      {

	 logger.log(java.util.logging.Level.SEVERE, "LOG01190:Could not read font file", Exception)
	 Retval = null
      }
      catch (Exception: NullPointerException)
      {
	 logger.log(java.util.logging.Level.SEVERE, "LOG01200:Could not create font.", Exception)
	 Retval = null
      } // try

      return Retval
   } // createFont

   /**
    * Read image from jar file
    *
    * @param Caller
    * Object to read the image for
    * @param File_Name
    * Filename of the Image
    * @return image read in background.
    */
   @JvmStatic
   fun createImage(Caller: java.awt.Component, File_Name: String): java.awt.Image?
   {
      logger.entering(TAG, "createImage", File_Name)

      var Retval: java.awt.Image?

      try
      {
	 val Tracker = java.awt.MediaTracker(Caller)
	 val Toolkit = java.awt.Toolkit.getDefaultToolkit()
	 val Caller_Class: Class<out java.awt.Component?> = Caller.javaClass
	 val Loader = Caller_Class.classLoader
	 val File_Location = Loader.getResource(File_Name)

	 Retval = Toolkit.getImage(File_Location)
	 Tracker.addImage(Retval, 0)
	 Tracker.waitForAll(500)
      }
      catch (Exception: NullPointerException)
      {
	 logger.log(java.util.logging.Level.SEVERE, "LOG01210: Failed to load image ${File_Name}")
	 Retval = null
      }
      catch (Exception: InterruptedException)
      {
	 logger.log(java.util.logging.Level.SEVERE, "LOG01220: Failed to load image ${File_Name}")
	 Retval = null
      } // try


      logger.exiting(TAG, "createImage", Retval)
      return Retval
   } // createImage

   /**
    * Paint image using lazy read.
    *
    * @param graphics
    * Graphic object to paint to
    * @param Caller
    * Object to read the image for
    * @param Image
    * Image to paint - if null read image from jar file
    * @param File_Name
    * Filename if lazy read is needed
    * @param x
    * x position
    * @param y
    * y position
    * @return original Image or lazy read immage
    */
   @JvmStatic
   fun drawImage(
      graphics: java.awt.Graphics,
      Caller: java.awt.Component,
      Image: java.awt.Image?,
      File_Name: String,
      x: Int,
      y: Int
   ): java.awt.Image?
   {
      logger.entering(TAG, "Utils.drawImage", File_Name)
      var Retval = Image

      try
      {
	 if (Retval == null)
	 {
	    logger.log(java.util.logging.Level.FINE, "> File_Name: $File_Name")

	    Retval = createImage( /* Canvas    => */
	       Caller,  /* File_Name => */
	       File_Name
	    )
	 } // if


	 if (Retval != null)
	 {
	    graphics.drawImage( /* img      => */Retval,  /* x        => */x,  /* y        => */y,  /* observer =>
	    */null
	    )
	 } // if
      }
      catch (Exception: NullPointerException)
      {
	 logger.log(java.util.logging.Level.SEVERE, "LOG01230:", Exception)
	 Retval = null
      } // try


      logger.exiting(TAG, "Utils.drawImage ()")
      return Retval
   } // drawImage

   /**
    * Resize a dimension by given factors. Note that java.awt.Dimension stores sizes in integer and the
    * resized dimension will be rounded up to the next integer value.
    *
    * @param original dimension to resize
    * @param scaleWidth width scale factor
    * @param scaleHeight height scale factor
    * @return the resized dimension
    */
   @JvmStatic
   fun resize(original: java.awt.Dimension, scaleWidth: Double, scaleHeight: Double) =
      java.awt.Dimension().apply {
	 setSize(
	    /* width  = */ original.getWidth() * scaleWidth,
	    /* height = */ original.getHeight() * scaleHeight
	 )
      }

   /**
    * Resize a dimension by given factors. Note that java.awt.Dimension stores sizes in integer and the
    * resized dimension will be rounded up to the next integer value. Adds a border size to the result
    *
    * @param original dimension to resize
    * @param scaleWidth width scale factor
    * @param scaleHeight height scale factor
    * @return the resized dimension
    */
   @JvmStatic
   fun resize(
      original: java.awt.Dimension,
      scaleWidth: Double,
      scaleHeight: Double,
      borderSize: java.awt.Dimension
   ) = java.awt.Dimension().apply {
      setSize(
	 /* width  = */ original.getWidth() * scaleWidth + borderSize.getWidth(),
	 /* height = */ original.getHeight() * scaleHeight + borderSize.getHeight()
      )
   }

   /**
    * Resize a dimension by adding the width and using the maximum height)
    *
    * @param size1 first size to add
    * @param size2 second size to add
    * @return the resized dimension
    */
   @JvmStatic
   fun addWidth(size1: java.awt.Dimension, size2: java.awt.Dimension) = java.awt.Dimension(
      /* width  = */ size1.width + size2.width,
      /* height = */ if (size1.height > size2.height)
      {
	 size1.height
      }
      else
      {
	 size2.height
      }
   )

   /**
    * Resize a dimension by adding the width and using the maximum height)
    *
    * @param size1 first size to add
    * @param size2 second size to add
    * @param borderSize extra border to add
    * @return the resized dimension
    */
   @JvmStatic
   fun addWidth(
      size1: java.awt.Dimension,
      size2: java.awt.Dimension,
      borderSize: java.awt.Dimension
   ) = java.awt.Dimension(
      /* width  = */ size1.width + size2.width + borderSize.width,
      /* height = */ if (size1.height > size2.height)
      {
	 size1.height
      }
      else
      {
	 size2.height
      } + borderSize.height
   )


} // Utils
// vim: set nowrap tabstop=8 shiftwidth=3 softtabstop=3 expandtab :
// vim: set textwidth=0 filetype=java foldmethod=syntax nospell :

