/*
 * ************************************************************ {{{1 **********
 * Copyright © 2005 … 2024  "Martin Krischik" «krischik@users.sourceforge.net»
 * **************************************************************************
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see «http://www.gnu.org/licenses»..
 * ************************************************************ }}}1 **********
 */
package net.sourceforge.uiq3.calculator

import net.sourceforge.uiq3.Asserted
import net.sourceforge.uiq3.Required
import net.sourceforge.uiq3.ui.ICalculatorCanvas
import org.jetbrains.annotations.TestOnly
import java.io.Serializable
import java.util.*
import java.util.concurrent.ArrayBlockingQueue
import java.util.logging.Level
import java.util.logging.Logger
import javax.swing.ImageIcon
import javax.swing.JPanel

/**
 * An display canvas for an Calculator
 *
 * @author "Martin Krischik" «krischik@users.sourceforge.net»
 */
abstract class Canvas
protected constructor() : JPanel(), ICalculatorCanvas, Serializable
{
   companion object
   {
      /**
       * class logger instance
       */
      private val Logger: Logger

      /**
       *
       */
      private const val serialVersionUID = 699482833610225281L

      /**
       * class logger tag.
       */
      private val TAG: String = Canvas::class.java.name

      /**
       * queue length for key input.
       */
      private const val Max_Queue_Length = 256

      protected const val Extra_Width: Int = 4
      protected const val Extra_Height: Int = 4

      /**
       * <p>extra border für split view
       */
      val border :java.awt.Dimension

      init
      {
	 Logger = java.util.logging.Logger.getLogger(TAG)
	 border = java.awt.Dimension(Companion.Extra_Width, Companion.Extra_Height)
      }
   }

   /**
    * Thread in which the virtual calculator CPU is running.
    */
   private var CPU_Thread: Thread? = null

   /**
    * Toggle Keyboard help on and off
    */
   private var Help = false

   /**
    * The emulated / simulated keyboard layout
    */
   protected lateinit var keyboard: Array<Key>

   /**
    * All keys in the keyboard.
    */
   @JvmField
   protected var Keys_Count: Int = 0

   /**
    * The Key last read from the keyboard.
    */
   private val Read_Key: ArrayBlockingQueue<Int>

   /**
    * Text to be displayed on the calculator display.
    */
   protected var Utf16_Text: String = ""

   /**
    *
    * The “Font” images.
    */
   @JvmField
   protected var Font: Array<ImageIcon?>

   /**
    * Create new Calculator AWT_Canvas
    */
   init
   {
      val listener = CanvasListener(this)

      this.addMouseListener(listener)
      this.addKeyListener(listener)

      this.Font = arrayOfNulls(256)
      this.Read_Key = ArrayBlockingQueue(Max_Queue_Length)
   } // Swing_Canvas

   /**
    * Set calculator Thread so the canvas can wake up the calculator after key / touch screen clicks.
    *
    * @param CPU_Thread
    * CPU Thread.
    */
   override fun bindCPUThread(CPU_Thread: Thread?)
   {
      this.CPU_Thread = CPU_Thread

      return
   }

   /**
    * Called when a key is released
    *
    * @param calculatorKeyCode
    * calculator key code of real key released
    */
   fun buttonReleased(calculatorKeyCode: Short)
   {
      Logger.entering(TAG, "buttonReleased", calculatorKeyCode)

      val stream = Arrays.stream(keyboard)
      val first = stream
	 .filter { key: Key -> key.calculatorKeyCode == calculatorKeyCode }
	 .findFirst()

      first.ifPresent { key: Key -> buttonReleased(key.keyNr) }

      Logger.exiting(TAG, "buttonReleased")
      return
   } // buttonReleased

   /**
    * Called when a key is released
    *
    * @param KeyNr
    * Java key code of real key released
    */
   fun buttonReleased(KeyNr: Int)
   {
      Logger.entering(TAG, "buttonReleased", KeyNr)

      if (!Read_Key.offer(KeyNr))
      {
	 Logger.log(Level.WARNING, "LOG03100: Could not store key in queue!")
      }

      if (this.CPU_Thread != null)
      {
	 synchronized(CPU_Thread!!) {
	    CPU_Thread!!.interrupt()
	 } // synchronized
      }

      Logger.exiting(TAG, "buttonReleased")
      return
   } // buttonReleased

   @set:TestOnly
   override var pressedKey: Short
      /**
       * The Key currently pressed
       *
       * @see net.sourceforge.uiq3.ui.ICalculatorCanvas.pressedKey
       */
      get()
      {
	 val Retval = IKey.No_Key.toInt()

	 return Retval.toShort()
      } // Get_Pressed_Key
      set(key)
      {
	 throw NotImplementedError(Constants.Not_Needed_For_Swing)
      }

   /**
    * Get calculator key code of the last key (de)pressed.
    *
    * @return Calculator key code or No_Key if no key pressed
    */
   fun getKey(Nr: Int): Key
   {
      Logger.entering(TAG, "getKey", Nr)

      Required.isNatural(Nr, "Nr")
      Asserted.notNull(this.keyboard, "Keyboard")

      val Retval = keyboard[Nr]

      Asserted.notNull(Retval, "Key")

      Logger.exiting(TAG, "getKey", Retval)
      return Retval
   } // getKey

   @set:TestOnly
   override var readKey: Short
      /**
       * Get calculator key code of the last key (de)pressed.
       *
       * @return Calculator key code or No_Key if no key pressed
       */
      get()
      {
	 Logger.entering(TAG, "getReadKey")

	 val javaNr = Read_Key.poll()

	 val Retval: Short

	 if (javaNr != null)
	 {
	    val key = this.getKey(javaNr)
	    Retval = key.calculatorKeyCode
	 }
	 else
	 {
	    Retval = IKey.No_Key
	 } // if


	 Logger.exiting(TAG, "getReadKey", Retval)
	 return Retval
      } // getReadKey
      set(key)
      {
	 throw NotImplementedError(Constants.Not_Needed_For_Swing)
      }

   /**
    * Called when a key is released
    *
    * @param javaKey
    * Java key code of real key released
    */
   fun keyReleased(javaKey: Int)
   {
      Logger.entering(TAG, "keyReleased", javaKey)

      checkNotNull(this.keyboard) { "keyboard is not null after initialisation" }
      var i = 0

      Search_Key@while (i < keyboard.size)
	 {
	    val key = getKey(i)

	    if (key.isKeyHit(javaKey))
	    {
	       if (!Read_Key.offer(i))
	       {
		  Logger.log(Level.WARNING, "LOG03050: Could not store key in queue!")
	       }

	       break@Search_Key
	    } // if

	    i = i + 1
      } // Search_Key

      if (this.CPU_Thread != null)
      {
	 synchronized(CPU_Thread!!) {
	    CPU_Thread!!.interrupt()
	 } // synchronized
      }

      Logger.exiting(TAG, "keyReleased")
      return
   } // keyReleased

   @get:TestOnly
   override val isReadKeyAvailable: Boolean
      /**
       *
       *
       * Get calculator key code of the last key (de)pressed.
       *
       *
       * @return true when a key is available.
       */
      get()
      {
	 val size = Read_Key.size
	 val retval = size > 0
	 return retval
      } // isReadKeyAvailable

   @get:TestOnly
   override var javaText: String
      /**
       * Get text currently displayed (as one java string).
       */
      get() = Utf16_Text
      /**
       * Set text to be displayed.
       *
       * @param Text
       * to be displayed
       */
      set(Text)
      {
	 Logger.entering(TAG, "setText", Text)

	 if (Utf16_Text != Text)
	 {
	    this.Utf16_Text = Text
	    this.repaint()
	 } // if


	 Logger.exiting(TAG, "setText")
	 return
      } // Set_Text

   /**
    * Toggle Keyboard help on and off
    */
   override fun toggleHelp()
   {
      this.Help = !this.Help
      this.repaint()
      return
   } // Toggle_Help

   @TestOnly
   override fun toString(): String
   {
      return """
       	Canvas {
       	super = ${super.toString()},
       	CPU_Thread = $CPU_Thread,
       	Help = $Help,
       	keyboard = ${keyboard.contentToString()},
       	Keys_Count = $Keys_Count,
       	Read_Key = $Read_Key,
       	Utf16_Text = „$Utf16_Text“,
       	Font = ${Font.contentToString()}
       	}
       	""".trimIndent()
   }

} // Canvas
// vim: set nowrap tabstop=8 shiftwidth=3 softtabstop=3 expandtab :
// vim: set textwidth=0 filetype=java foldmethod=syntax nospell :

