/*
 * ************************************************************ {{{1 **********
 * Copyright © 2005 … 2022  "Martin Krischik" «krischik@users.sourceforge.net»
 * **************************************************************************
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see «http://www.gnu.org/licenses»..
 * **************************************************************************
 * $Author: krischik $
 * $Revision: 8064 $
 * $Date: 2022-02-10 13:08:36 +0100 (Do, 10. Feb 2022) $
 * $Id: Main.java 8064 2022-02-10 12:08:36Z krischik $
 * ************************************************************ }}}1 **********
 */
package net.sourceforge.uiq3.calculator

import org.jetbrains.annotations.TestOnly
import java.util.logging.Level as Log_Level

/**
 * @author "Martin Krischik" «krischik@users.sourceforge.net»
 */
abstract class Main
protected constructor() : Runnable, IMain
{
   companion object
      : net.sourceforge.uiq3.IPropertyNotifier
   {
      /**
       * class logger tag.
       */
      private val TAG: String = Main::class.java.name

      /**
       * class logger instance
       */
      private val logger: java.util.logging.Logger = java.util.logging.Logger.getLogger(TAG)

      /**
       * Bold font being able to display all special characters needed
       */
      protected var File_Handler: java.util.logging.FileHandler? = null

      const val LOOK_AND_FEEL_CHANGED = "Main::LOOK_AND_FEEL_CHANGED";

      /**
       * Handle property changes. We don't use the full feature with oldValue and new newValue as that would be overkill.
       */
      @JvmStatic
      protected val propertyChangeSupport = java.beans.PropertyChangeSupport(this);

      override fun addPropertyChangeListener(listener: java.beans.PropertyChangeListener)
      {
	 propertyChangeSupport.addPropertyChangeListener(listener)
      }

      override fun removePropertyChangeListener(listener: java.beans.PropertyChangeListener)
      {
	 propertyChangeSupport.removePropertyChangeListener(listener)
      }

      /**
       * directory for cache files
       *
       * @return directroy
       */
      fun Cache_Dir(App_Name: String, Home_Directory: String): java.io.File
      {
	 logger.entering(TAG, "Cache_Dir", arrayOf<Any>(App_Name, Home_Directory))

	 val Cache_Directory = if (com.krischik.ui.MacApplication.macOS.value)
	 {
	    java.io.File(
	       /* parent = */System.getProperty("user.dir"),
	       /* child  = */"Caches"
	    )
	 }
	 else
	 {
	    java.io.File(Home_Directory, "Logs")
	 } // if


	 val Retval = java.io.File( /* parent */Cache_Directory,  /* child */App_Name)

	 // noinspection ResultOfMethodCallIgnored
	 Retval.mkdirs()

	 logger.exiting(TAG, "Cache_Dir", Retval)
	 return Retval
      }

      /**
       * To make widget classes compatible with GUI designer they must be instantiatable from there default constructor.
       * This means that each widget must initialize the LAF in there default constructor.
       */
      @JvmStatic
      fun initializeLAF(
	 properties: IProperties,
	 applicationName: String
      )
      {
	 logger.entering(TAG, "initializeLAF", arrayOf(properties, applicationName))

	 val Skin_Name = properties.Look_And_Feel_Name

	 try
	 {
	    logger.info("LOG01740: Setup Look and Feel to $Skin_Name")

	    com.krischik.ui.MacApplication.setApplicationName(applicationName)

	    System.setProperty("swing.defaultlaf", Skin_Name)
	    javax.swing.UIManager.setLookAndFeel(Skin_Name)
	    javax.swing.UIManager.put("Button.arc", properties.Button_Arc)

	    com.krischik.Resizeable_Nimbus.setAllFonts(
	       properties.Dialog_Font,
	       properties.Dialog_Large_Font,
	       properties.Dialog_Small_Font
	    )
	 }
	 catch (exception: Exception)
	 {
	    logger.log(
	       Log_Level.SEVERE,
	       "LOG00031:Can't change LAF to $Skin_Name",
	       exception
	    )
	 } // try


	 logger.exiting(TAG, "initializeLAF")
	 return
      } // initializeLAF

      @JvmStatic
      fun registerLAF()
      {
	 logger.entering(TAG, "registerLAF")

	 logger.info("LOG01741: Register Look and Feel.")

	 com.formdev.flatlaf.FlatLaf.registerCustomDefaultsSource("net.sourceforge.uiq3.calculator");
	 hp45.setup();
	 fx602p.setup();
	 fx603p.setup();

	 logger.exiting(TAG, "registerLAF")
	 return
      } // registerLAF

      fun registerFonts()
      {
	 logger.entering(TAG, "registerFonts")

	 logger.info("LOG01742: Register fonts.")

	 val Graphics_Environment =
	    java.awt.GraphicsEnvironment.getLocalGraphicsEnvironment()

	 java.util.stream.Stream.of(
	    "/DejaVuSans-Bold.ttf",
	    "/DejaVuSans.ttf",
	    "/DejaVuSansMono-Bold.ttf",
	    "/DejaVuSansMono.ttf",
	    "/DejaVuSerif-Bold.ttf",
	    "/DejaVuSerif.ttf"
	 )
	    .map { fileName -> Utils.createFont(fileName) }
	    .forEach { font ->
	       logger.log(Log_Level.INFO, "LOG01743: register font «{0}».", font)

	       Graphics_Environment.registerFont(font)
	    }

	 logger.exiting(TAG, "registerFonts")
	 return
      } // registerFonts

      /**
       *
       *
       * Setup Java logging.
       *
       *
       * @param App_Class
       * Application calls name
       * @param App_Name
       * Application name
       * @param Home_Directory
       * Appplication (not user) home directory
       */
      @JvmStatic
      protected fun Setup_Logging(
	 App_Class: Class<out Main?>,
	 App_Name: String,
	 Home_Directory: String?
      )
      {
	 logger.entering(TAG, "Setup_Logging", Home_Directory)

	 val Manager = java.util.logging.LogManager.getLogManager()
	 val Properties = App_Class.getResourceAsStream("/logging.properties")

	 if (Properties != null)
	 {
	    logger.log(Log_Level.INFO, "LOG00050:Load log settings from {0}", Properties)

	    try
	    {
	       Manager.readConfiguration(Properties)

	       val Formatter = java.util.logging.SimpleFormatter()

	       if (com.krischik.ui.MacApplication.macOS.value)
	       {
		  File_Handler = java.util.logging.FileHandler(
		     /* pattern = */ "%h/Library/Logs/${App_Name}-%g-%u.log",
		     /* limit   = */ 65536,
		     /* count   = */ 10,
		     /* append  = */ false
		  )
	       }
	       else
	       {
		  val Log_Directory = java.io.File(Home_Directory, "Logs")
		  // noinspection ResultOfMethodCallIgnored
		  Log_Directory.mkdirs()
		  File_Handler = java.util.logging.FileHandler(
		     /* pattern = */ "${Log_Directory}/${App_Name}-%g-%u.log",
		     /* limit   = */ 65536,
		     /* count   = */ 10,
		     /* append  = */ false
		  )
	       } // if


	       File_Handler!!.encoding = "UTF-8"
	       File_Handler!!.formatter = Formatter

	       val Console_Handler = java.util.logging.ConsoleHandler()

	       Console_Handler.level = Log_Level.INFO
	       Console_Handler.formatter = Formatter

	       val Root_Logger = java.util.logging.Logger.getLogger("net.sourceforge.uiq3")

	       Root_Logger.addHandler(File_Handler)
	       Root_Logger.addHandler(Console_Handler)
	       // Root_Logger.setLevel (java.util.logging.Level.FINER);
	    }
	    catch (exception: java.io.IOException)
	    {
	       logger.log(Log_Level.WARNING, "LOG00060:Logging properties not read.", exception)
	    }
	    catch (exception: SecurityException)
	    {
	       logger.log(
		  Log_Level.WARNING,
		  "LOG00070:Logging properties not read, probably running in with Webstart.",
		  exception
	       )
	    } // try
	 } // if


	 logger.exiting(TAG, "Setup_Logging")
	 return
      } // Setup_Logging
   }

   /**
    * Thread in which the calculator runs.
    */
   private var calculatorThread: Thread? = null

   /**
    * Calculator engine
    */
   private var calculator: ICalculator?

   /**
    * Canvas to display the Calculator in
    */
   private var Form: net.sourceforge.uiq3.ui.ICalculatorCanvas?

   /**
    * Program start
    */
   init
   {
      logger.entering(TAG, "Main")

      this.Form = null
      this.calculator = null

      // Set up our application to respond to the Mac OS X application menu
      this.registerForMacOSXEvents()

      logger.exiting(TAG, "Main")
   } // Main

   /**
    * Create Calculator engine.
    *
    * @return New calculator engine
    * @throws IOException
    * I/O Error
    */
   @Throws(java.io.IOException::class)
   protected abstract fun Create_Calculator(): ICalculator?

   /**
    * Create Calculator display.
    *
    * @return new display canvas
    * @throws IOException
    * could not create canvas
    */
   @Throws(java.io.IOException::class)
   protected abstract fun createCanvas(): net.sourceforge.uiq3.ui.ICalculatorCanvas?

   /**
    * Print Error message Stop Calculator and exit program.
    *
    * @param Message
    * Message to be displayed
    */
   protected fun fatalError(Message: String?)
   {
      logger.entering(TAG, "Fatal_Error", Message)

      logger.log(Log_Level.SEVERE, "LOG00080:fatal error: {0}", Message)

      // notifyDestroyed ();
      logger.exiting(TAG, "Fatal_Error")
      return
   } // Fatal_Error

   /**
    * General info dialog; fed to the OSXAdapter as the method to call when "About OSXAdapter" is selected from the
    * application menu
    */
   abstract fun about()

   /**
    * load a file
    *
    * @param Filename
    * filename to load
    */
   abstract fun loadFile(Filename: java.io.File?)

   /**
    * Signals the MIDlet to enter the Paused state.
    */
   override fun pauseApp()
   {
      logger.entering(TAG, "Main.pauseApp")

      if ((this.calculator != null) && calculator!!.isRunning())
      {
	 calculator!!.stop()
      } // if


      // if (Calculator_Thread != null)
      // {
      // try
      // {
      // Calculator_Thread.join (2100);
      // }
      // catch (InterruptedException Exception)
      // {
      // logger.log (java.util.logging.Level.SEVERE,"",Exception);
      // } // try
      // Calculator_Thread = null;
      // } // if

      // System.gc ();
      logger.exiting(TAG, "Main.pauseApp()")
      return
   } // pauseApp

   /**
    * General preferences dialog; fed to the OSXAdapter as the method to call when "About OSXAdapter" is selected from
    * the application menu
    */
   abstract fun preferences()

   /**
    * Generic registration with the Mac OS X application menu Checks the platform, then attempts to register with the
    * Apple EAWT See OSXAdapter.java to see how this is done without directly referencing any Apple APIs
    */
   private fun registerForMacOSXEvents()
   {
      logger.entering(TAG, "registerForMacOSXEvents")

      if (com.krischik.ui.MacApplication.macOS.value)
      {
	 logger.log(Log_Level.INFO, "LOG00300: Running on Mac OS X!")

	 try
	 {
	    com.krischik.ui.MacApplication.setAboutHandler {
	       this.about()
	       null
	    }

	    com.krischik.ui.MacApplication.setQuitHandler {
	       this.quit()
	       null
	    }
	    com.krischik.ui.MacApplication.setPreferencesHandler {
	       this.preferences()
	       null
	    }

	    com.krischik.ui.MacApplication.setOpenFileHandler { firstFile: java.io.File? ->
	       this.loadFile(firstFile)
	       null
	    }
	 }
	 catch (exception: Throwable)
	 {
	    logger.log(
	       Log_Level.SEVERE,
	       "LOG00090: Error while loading the OSXAdapter:",
	       exception
	    )
	 } // try
      } // if

      logger.exiting(TAG, "registerForMacOSXEvents")
      return
   } // registerForMacOSXEvents

   /**
    * Initialise system and run virtual calculator CPU
    */
   override fun run()
   {
      logger.entering(TAG, "run")

      if (this.Form == null)
      {
	 try
	 {
	    this.Form = this.createCanvas()
	 }
	 catch (Exception: java.io.IOException)
	 {
	    logger.log(Log_Level.SEVERE, "LOG00100: Error reading Display Theme.", Exception)
	    this.fatalError("Error reading Display Theme")
	 } // try
      } // if


      Form!!.bindCPUThread(this.calculatorThread)

      if (this.calculator == null)
      {
	 try
	 {
	    this.calculator = this.Create_Calculator()
	    calculator!!.reset()
	    calculator!!.Load()
	 }
	 catch (Exception: java.io.IOException)
	 {
	    logger.log(
	       Log_Level.SEVERE,
	       "LOG00110: Error reading listing firmware file.",
	       Exception
	    )
	    this.fatalError("Error reading listing firmware file.")
	 } // try
      } // if


      // noinspection SynchronizeOnNonFinalField
      synchronized(calculatorThread!!) {
	 logger.log(Log_Level.INFO, "LOG00130: Thread initialisation finished, notify all.")
	 (calculatorThread as Object?)!!.notifyAll()
      } // synchronized

      calculator!!.run()

      logger.exiting(TAG, "run")
      return
   } // run

   /**
    * Signals the Application that it has entered the Active state.
    */
   override fun startApp()
   {
      logger.entering(TAG, "startApp")

      if ((this.calculatorThread == null) || !(this.calculator != null && calculator!!.isRunning()))
      {
	 this.calculatorThread = Thread(this)
	 calculatorThread!!.start()

	 try
	 {
	    logger.log(Log_Level.INFO, "LOG00140: Wait for thread initialisation to finish.")

	    // noinspection SynchronizeOnNonFinalField
	    synchronized(calculatorThread!!) {
	       (calculatorThread as Object?)!!.wait(5000)
	    } // synchronized

	    if (this.calculator != null)
	    {
	       logger.log(
		  Log_Level.INFO,
		  "LOG00280: Wait for calculator initialisation to finish."
	       )

	       // noinspection SynchronizeOnNonFinalField
	       synchronized(calculator!!) {
		  (calculator as Object?)!!.wait(2000)
	       } // synchronized
	    }
	    else
	    {
	       logger.log(Log_Level.INFO, "LOG00511: Calculator missing.")
	    }

	    logger.log(
	       Log_Level.INFO,
	       "LOG00290: all initialisation have finished (or timed out)."
	    )
	 }
	 catch (exception: NullPointerException)
	 {
	    logger.log(Log_Level.WARNING, "LOG00510: Calculator missing.", exception)
	 } // try
	 catch (exception: InterruptedException)
	 {
	    logger.log(Log_Level.WARNING, "LOG00120: Interrupted.", exception)
	 } // try
      } // if


      logger.exiting(TAG, "startApp")
      return
   } // startApp

   @TestOnly
   override fun toString(): String
   {
      return "Main {}"
   }

} // Main
// vim: set nowrap tabstop=8 shiftwidth=3 softtabstop=3 expandtab :
// vim: set textwidth=0 filetype=java foldmethod=syntax nospell :
