/*
 * ************************************************************ {{{1 **********
 * Copyright © 2020 … 2020  "Martin Krischik" «krischik@users.sourceforge.net»
 * **************************************************************************
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see «http://www.gnu.org/licenses»..
 * **************************************************************************
 * $Author: krischik $
 * $Revision: 7953 $
 * $Date: 2022-01-19 07:59:28 +0100 (Mi, 19. Jan 2022) $
 * $Id: StringTokenizer.kt 7953 2022-01-19 06:59:28Z krischik $
 * $HeadURL: svn+ssh://krischik@svn.code.sf.net/p/uiq3/code/trunk/Java/Calculator/Library/src/main/java/net.sourceforge.uiq3/StringTokenizer.kt $
 * ************************************************************ }}}1 **********
 */

package net.sourceforge.uiq3;

import java.util.logging.Level

/**
 * Tokenizer to read calculator files
 */
class StringTokenizer(
   Input_Text: String,
   private val Delegate: MutableList<String>) :
   List<String> by Delegate
{
   companion object
   {
      /**
       * <p>logging tag</p>
       */
      private val TAG = StringTokenizer::class.qualifiedName

      /**
       * <p>logger</p>
       */
      private val logger = java.util.logging.Logger.getLogger(TAG)

      /**
       * The calculator tokenizer always separates on space.
       */
      private const val Separator = " "

      /**
       * The calculator tokenizer always double quotes to signify string which may contain separators.
       */
      private const val Quote = '"'

   } // companion object

   /**
    * used for initial parsing. In the resulting the ALPHA strings need to be merged again.
    */
   private val Tokenizer = java.util.StringTokenizer(Input_Text, Separator, true)

   /**
    * convenience constructor for use with java and scala
    */
   constructor(Input_Text: String) : this(Input_Text, mutableListOf())
   {
      logger.entering(TAG, "constructor", arrayOf(Input_Text, Tokenizer))

      logger.exiting(TAG, "constructor")
   } // constructor

   init
   {
      logger.entering(TAG, "init", arrayOf(Input_Text, Tokenizer, Delegate))

      var In_Quotes = false
      var Alpha_String = ""

      for (Token in Tokenizer)
      {
	 if (Token is String)
	 {
	    // There are 4 valid quote options to be handled:
	    //
	    // • quotes on both ends: single word alpha
	    // • quote at the beginning: start of alpha with spaces
	    // • quote at the end: end of alpha with spaces
	    // • quote on its own: start or end of alpha with spaces
	    // • quote in the middle: parse error
	    // • more than two quotes: parse error

	    val Token_Length = Token.length
	    val Start_Quote = Token.startsWith(Quote) && (Token_Length > 1 || !In_Quotes)
	    val End_Quote = Token.endsWith(Quote) && (Token_Length > 1 || In_Quotes)
	    val Both_Quote = Start_Quote && End_Quote && Token_Length > 1

	    logger.log(
	       Level.FINER,
	       "\n> Token        = „{0}“" +
		  "\n> In_Quotes    = „{1}“" +
		  "\n> Alpha_String = „{2}“" +
		  "\n> Token_Length = „{3}“" +
		  "\n> Start_Quote  = „{4}“" +
		  "\n> End_Quote    = „{5}“" +
		  "\n> Both_Quote   = „{6}“",
	       arrayOf<Any>(Token, In_Quotes, Alpha_String, Token_Length, Start_Quote, End_Quote, Both_Quote))

	    if (Token_Length > 2 && Token.lastIndexOf(Quote, Token_Length - 2) > 0)
	    {
	       throw java.text.ParseException("Unexpected quotes in parsed string.", Delegate.size)
	    }
	    else if (Both_Quote)
	    {
	       Delegate.add(Token)
	    }
	    else if (Start_Quote)
	    {
	       Alpha_String = Token
	       In_Quotes = true
	    }
	    else if (End_Quote)
	    {
	       Alpha_String = Alpha_String + Token
	       Delegate.add(Alpha_String)
	       Alpha_String = ""
	       In_Quotes = false
	    }
	    else if (In_Quotes)
	    {
	       Alpha_String = Alpha_String + Token
	    }
	    else if (!Token.equals(Separator))
	    {
	       Delegate.add(Token)
	    } // if
	 } // if
      } // for

      if (In_Quotes)
      {
	 throw java.text.ParseException("Quotes in parsed string are unbalanced.", Delegate.size)
      } // if

      logger.exiting(TAG, "init", Delegate)
   } // init
} // StringTokenizer

//************************************************************* {{{1 **********
// vim: set nowrap tabstop=8 shiftwidth=4 softtabstop=4 expandtab textwidth=120:
// vim: set textwidth=0 filetype=kotlin foldmethod=indent spell spelllang=en_gb :

