/*
 * ******************************************************** {{{1 ************
 *   Copyright © 2005 … 2018 "Martin Krischik" «krischik@users.sourceforge.net»
 * **************************************************************************
 *   This program is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see «http://www.gnu.org/licenses».
 *  ***************************************************************************
 *   $Author: krischik $
 *   $Revision: 7711 $
 *   $Date: 2021-10-15 11:27:40 +0200 (Fr, 15. Okt 2021) $
 *   $Id: IA_Register.java 7711 2021-10-15 09:27:40Z krischik $
 *  ********************************************************** }}}1 ***********
 */

package net.sourceforge.uiq3.calculator;

import org.jetbrains.annotations.NotNull;

/**
 * Interface for FX-602P and FX-603P alpha register
 *
 * @author Martin Krischik «krischik@users.sourceforge.net»
 * @version 1.0
 * @since 1.0
 */
@SuppressWarnings ("UnnecessaryInterfaceModifier")
public interface IA_Register
   extends
   net.sourceforge.uiq3.calculator.IClear,
   net.sourceforge.uiq3.calculator.IOperation
{
   /**
    * Register mode
    *
    * <dl>
    * <dt>Mode_Normal</dt>
    * <dd>normal numeric entry mode</dd>
    * <dt>Mode_Fraction</dt>
    * <dd>fraction entry mode</dd>
    * <dt>Mode_Exponent</dt>
    * <dd>exponent entry mode</dd>
    * <dt>Mode_Begin_Alpha</dt>
    * <dd>begins alpha mode</dd>
    * <dt>Keyboard_Alpha</dt>
    * <dd>alpha entry mode</dd>
    * </dl>
    */
   public enum Mode_Type
   {
      /**
       * Alpha register in alpha entry mode — insert happens from the front
       */
      Alpha,
      /**
       * Alpha register begins alpha mode — insert happens from the front
       */
      Begin_Alpha,
      /**
       * Keyboard_Alpha register in fraction entry mode entering the numeric part after the dot.
       */
      Decimal_Fraction,
      /**
       * Keyboard_Alpha register in true fraction mode entering the Denominator.
       */
      Denominator,
      /**
       * Alpha register in exponent entry mode.
       */
      Exponent,
      /**
       * Alpha register in display message mode
       */
      Message,
      /**
       * * Keyboard_Alpha register in normal numeric entry mode.
       */
      Normal,
      /**
       * Keyboard_Alpha register in true fraction mode entering the Numerator.
       */
      Numerator
   } // Mode_Type

   /**
    * toggle plus / minus for either mantissa or exponent.
    */
   public void Add_Negative ();

   /**
    * Add a numeric value - this is a non scrolling operation.
    *
    * @param Digit
    *    the digit to add
    * @throws Error
    *    something went wrong
    */
   public void Add_Numeric (final short Digit)
      throws
      Error;

   /**
    * Clear content of Register
    *
    * @param From
    *    First character to clean
    * @param To
    *    Last character to clean
    */
   public void Clear_Display_Value (final int From, final int To);

   /**
    * Clear content of Register
    *
    * @see IClear#Clear(boolean)
    */
   public void Clear_Print_Value ();

   /**
    * Create the optional print value which can be up to 40 characters.
    */
   public void Create_Print_Value ();

   /**
    * Count the amount of characters inserted to the display.
    *
    * @return amount of characters in display
    */
   public int Display_Count ();

   /**
    * Count the amount of characters inserted to the display.
    *
    * @param Char_Count
    *    amount of characters
    */
   public void Display_Count (int Char_Count);

   /**
    * Display an informative message until next key is pressed.
    *
    * @param Text
    *    Test to be displayed
    */
   public void Display_Message (final short @NotNull [] Text);

   /**
    * Get value of the register for use with as filename.
    *
    * @return current value
    */
   public short @NotNull [] Filename_Value ();

   /**
    * Insert content of an String.
    *
    * @param Text
    *    Text to insert without leading spaces and exponent
    * @return Characters inserted
    */
   public int Insert (final short @NotNull [] Text)
      throws
      Error;

   /**
    * Insert content of L0 aka X register into the string.
    */
   public void Insert_X ()
      throws
      Error;

   /**
    * Limit Alpha text to Start_Char
    *
    * @param Num_Char
    *    max char allowed
    */
   public void Limit_Alpha (final int Num_Char);

   /**
    * Get alpha register entry mode
    *
    * @return <dl>
    * <dt>Mode_Normal</dt>
    * <dd>normal numeric entry mode</dd>
    * <dt>Mode_Fraction</dt>
    * <dd>fraction entry mode</dd>
    * <dt>Mode_Exponent</dt>
    * <dd>exponent entry mode</dd>
    * <dt>Mode_Begin_Alpha</dt>
    * <dd>begins alpha mode</dd>
    * <dt>Keyboard_Alpha</dt>
    * <dd>alpha entry mode</dd>
    *         </dl>
    */
   @NotNull
   public Mode_Type Mode ();

   /**
    * Set alpha register entry mode
    *
    * @param Mode
    *    <dl>
    *                             <dt>Mode_Normal</dt>
    *                             <dd>normal numeric entry mode</dd>
    *                             <dt>Mode_Fraction</dt>
    *                             <dd>fraction entry mode</dd>
    *                             <dt>Mode_Exponent</dt>
    *                             <dd>exponent entry mode</dd>
    *                             <dt>Mode_Begin_Alpha</dt>
    *                             <dd>begins alpha mode</dd>
    *                             <dt>Keyboard_Alpha</dt>
    *                             <dd>alpha entry mode</dd>
    *                             </dl>
    */
   public void Mode (@NotNull final Mode_Type Mode);

   /**
    * Get value of the register for use with as BCD.
    *
    * @return current value
    */
   @NotNull
   public net.sourceforge.uiq3.math.Number Numeric_Value ()
      throws
      Error;

   /**
    * Get value of the register for use with printer or tape interface.
    *
    * @return current value
    */
   public short[] Print_Value ();

   /**
    * Set value of the register for use with printer or tape interface. Also contains a backup of the alpha string for
    * the append function
    *
    * @param Value
    *    Value
    * @param Size
    *    Characters used
    */
   public void Print_Value (final short @NotNull [] Value, final int Size);

   /**
    * Count the amount of characters inserted to the printer.
    *
    * @param Char_Count
    *    amount of characters to print
    */
   public void Printer_Count (int Char_Count);

   /**
    * Count the amount of characters inserted to the printer.
    *
    * @return amount of characters in print
    */
   public int Printer_Count ();

   /**
    * Start a new alpha run. This flag is needed for proper handling of the ; (append) token.
    */
   public void Start_Alpha ();

   /**
    * start to enter the part behind the fraction mark point
    */
   public void Start_Decimal_Fraction ();

   /**
    * Start to enter the exponent.
    */
   public void Start_Exponent ();

   /**
    * Get value of the register for use with display.
    *
    * @return current value
    */
   public short[] Value ();

   /**
    * Value of the register.
    *
    * @param Value
    *    new value
    */
   public void Value (final short @NotNull [] Value);
} // IA_Register

// vim: set nowrap tabstop=8 shiftwidth=3 softtabstop=3 expandtab textwidth=96 :
// vim: set fileencoding=utf-8 filetype=java foldmethod=syntax spell spelllang=en_gb :
