/*
 * ******************************************************** {{{1 ************
 *   Copyright © 2005 … 2019 "Martin Krischik" «krischik@users.sourceforge.net»
 * **************************************************************************
 *   This program is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see «http://www.gnu.org/licenses».
 *  ***************************************************************************
 *   $Author: krischik $
 *   $Revision: 7714 $
 *   $Date: 2021-10-21 11:50:10 +0200 (Do, 21. Okt 2021) $
 *   $Id: ICalculator.java 7714 2021-10-21 09:50:10Z krischik $
 *  ********************************************************** }}}1 ***********
 */

package net.sourceforge.uiq3.calculator;

import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.TestOnly;

/**
 * Public calculator interface
 *
 * @author "Martin Krischik" «krischik@users.sourceforge.net»
 */
public interface ICalculator
   extends
   Runnable
{
   /**
    * Alpha register to merge key operations.
    * <dl>
    * <dt>A_Numeric</dt>
    * <dd>Alpha register to enter and display numeric results.</dd>
    * <dt>A_Text</dt>
    * <dd>Alpha register to enter and display textual results.</dd>
    * <dt>A_Message</dt>
    * <dd>Alpha register to display messages.</dd>
    * <dt>A_Key</dt>Alpha register to merge key operations.</dd>
    * </dl>
    */
   enum A_Type
   {
      /**
       * <p>
       * Alpha register to enter and display numeric results.
       * </p>
       */
      Numeric (ICalculator.A_Numeric),
      /**
       * <p>
       * Alpha register to enter and display textual results.
       * </p>
       */
      Text (ICalculator.A_Text),
      /**
       * <p>
       * Alpha register to display messages.
       * </p>
       */
      Message (ICalculator.A_Message),
      /**
       * <p>
       * Alpha register to merge key operations.
       * </p>
       */
      Key (ICalculator.A_Key),
      /**
       * <p>
       * Alpha register to show formatted text values
       * </p>
       */
      Formatted (ICalculator.A_Formatted);
      /**
       * <p>
       * Total count of alpha register
       * </p>
       */
      public static final int Count = A_Type.values ().length;
      /**
       * <p>
       * Integer representation of enum value.
       * </p>
       */
      public final int pos;

      /**
       * <p>
       * get voucher type from database value.
       * </p>
       *
       * @param pos
       *    0 … 3
       * @return the voucher type
       * @throws ArrayIndexOutOfBoundsException
       *    pos was not 0 … 1
       */
      @NotNull
      public static A_Type value (final int pos)
	 throws
	 ArrayIndexOutOfBoundsException
      {
	 final A_Type Retval;

	 Search_Enum:
	 {
	    for (final A_Type i : A_Type.values ())
	    {
	       if (i.pos == pos)
	       {
		  Retval = i;
		  break Search_Enum;
	       } // if
	    } // for

	    throw new IllegalArgumentException ("No enum const " + A_Type.class + " for pos " + pos);
	 } // Search_Enum

	 return Retval;
      } // value

      /**
       * <p>
       * </p>
       *
       * @param pos
       *    numerical pos used for array index
       */
      A_Type (final int pos)
      {
	 this.pos = pos;
      } // A_Typ

      @NotNull
      @Override
      public String toString ()
      {
	 return this.pos + ": " + super.toString ();
      }
   } // A_Type

   /**
    * <p>
    * The numeric Display currently displays the content of an Alpha register.
    * <dl>
    * <dt>Di row splay_Alpha</dt>
    * <dd>Displays the content of an Alpha register.</dd>
    * <dt>Display_L</dt>
    * <dd>Displays the content of the L0 aka X register.</dd>
    * <dt>Display_Formatted</dt>
    * <dd>Displays the content of the L0 aka X register inside an alpha register.</dd>
    * </dl>
    * </p>
    */
   enum Display_Type
   {
      /**
       * Displays the content of an Alpha register — for example on input
       */
      Alpha,
      /**
       * Displays a blank row of text. Only text row of the FX-603P
       */
      Blank,
      /**
       * Displays the content of the L0 aka X register inside an alpha register.
       */
      Formatted,
      /**
       * Displays the content of the L0 aka X register inside an alpha register.
       */
      L
   } // Display_Type

   /**
    * <p>
    * <b>Alpha register to display messages.</b>Convenience and performance constants — so we
    * don't have to go though the enum for constant access
    * </p>
    */
   int A_Formatted = 4;
   /**
    * <p>
    * <b>Alpha register to merge key operations.</b>Convenience and performance constants — so
    * we don't have to go thou the enum for constant access
    * </p>
    */
   int A_Key = 3;
   /**
    * <p>
    * <b>Alpha register to display messages.</b>Convenience and performance constants — so we
    * don't have to go though the enum for constant access
    * </p>
    */
   int A_Message = 2;
   /**
    * <p>
    * <b>Alpha register to enter and display numeric results.</b>Convenience and performance
    * constants — so we don't have to go though the enum for constant access
    * </p>
    */
   int A_Numeric = 0;
   /**
    * <p>
    * <b>Alpha register to enter and display textual results.</b>Convenience and performance
    * constants — so we don't have to go though the enum for constant access
    * </p>
    */
   int A_Text = 1;

   /**
    * Clear all alpha register. And just calling "Clear" is not enough.
    */
   void
   displayClear ();

   /**
    * <p>
    * display additional information's on the error. this is device and setup dependent.
    * </p>
    *
    * @param Exception
    *    Exception to display
    */
   void
   displayException (@NotNull final Throwable Exception);

   /**
    * Display value as formatted text
    *
    * @param Text
    *    to be displayed
    */
   void
   Display_Formatted (final short @NotNull [] Text);

   /**
    * * Display Key merge info.
    *
    * @param Text
    *    info to be displayed
    */
   void
   Display_Key (final short @NotNull [] Text);

   /**
    * Reset current value Display
    */
   void
   Display_L ();

   /**
    * Display texts as dual line message
    *
    * @param Text
    *    to be displayed
    */
   void
   Display_Message (final short @NotNull [] Text);

   /**
    * Current display mode
    * <dl>
    * <dt>Display_Alpha</dt>
    * <dd>Displays the content of the Alpha register.</dd>
    * <dt>Display_L</dt>
    * <dd>Displays the content of the L0 aka X register.</dd>
    * <dt>Display_Formatted</dt>
    * <dd>Displays the content of the L0 aka X register inside an alpha
    * </dl>
    *
    * @return current display mode
    */
   @NotNull Display_Type
   Display_Mode ();

   /**
    * Current display mode
    * <dl>
    * <dt>Display_Alpha</dt>
    * <dd>Displays the content of the Alpha register.</dd>
    * <dt>Display_L</dt>
    * <dd>Displays the content of the L0 aka X register.</dd>
    * <dt>Display_Formatted</dt>
    * <dd>Displays the content of the L0 aka X register inside an alpha
    * </dl>
    *
    * @param Display_Mode
    *    new display mode
    */
   void
   Display_Mode (@NotNull final Display_Type Display_Mode);

   /**
    * @return true when calculator is running.
    */
   boolean
   isRunning ();

   /**
    * is waiting in interactive mode for user input.
    *
    * @return true when in interactive mode.
    */
   @TestOnly
   boolean
   isWaiting ();

   /**
    * <p>locate the last step to display the free steps or for debugging.</p>
    *
    * @return last step
    */
   int
   Last_Step ();

   /**
    * Load Calculator State
    */
   void Load ();

   /**
    * Load the current status of the system
    *
    * @param Filename
    *    Filename to load data from
    */
   void
   Load (@NotNull final String Filename);

   /**
    * Load the current status of the system
    *
    * @param Filename
    *    Filename to load data from
    */
   default void
   Load (@NotNull final java.io.File Filename)
   {
      Load (Filename.toString ());
   }

   /**
    * Save Calculator State to default file
    */
   void
   Save ();

   /**
    * Save Calculator state
    *
    * @param Filename
    *    filename to save to
    */
   void
   Save (@NotNull final String Filename);

   /**
    * Save Calculator state
    *
    * @param Filename
    *    filename to save to
    */
   default void
   Save (@NotNull final java.io.File Filename)
   {
      Save (Filename.toString ());
   }

   /**
    * Reset the calculator
    */
   void
   reset ();

   /**
    * <p>
    * performs a full reset by deleting ram and the cache file as well a reloading the cache file.
    * </p>
    */
   default void
   resetAll ()
   {
      reset ();
   }

   /**
    * <p>
    * Enable or disable energy savings mode. for example by reducing thread priority.
    * </p>
    *
    * @param enable
    *    start saving energy
    */
   void Save_Energy (boolean enable);

   /**
    * Start virtual CPU
    */
   @Override void
   run ();

   /**
    * <p>
    * Stop virtual CPU and the thread it runs in
    * </p>
    */
   void
   stop ();
} // ICalculator

// vim: set nowrap tabstop=8 shiftwidth=3 softtabstop=3 expandtab textwidth=96 :
// vim: set fileencoding=utf-8 filetype=java foldmethod=syntax spell spelllang=en_gb :
