/*
 * ************************************************************ {{{1 **********
 * Copyright © 2005 … 2014  "Martin Krischik" «krischik@users.sourceforge.net» (extensions for FX-602P)
 * Copyright © 2005  Kai G. Schwebke, upn.schwebke.com
 * **************************************************************************
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see «http://www.gnu.org/licenses»..
 * **************************************************************************
 * $Author: krischik $
 * $Revision: 8155 $
 * $Date: 2022-05-06 13:03:09 +0200 (Fr, 06. Mai 2022) $
 * $Id: BCDFloat.java 8155 2022-05-06 11:03:09Z krischik $
 * $HeadURL: svn+ssh://krischik@svn.code.sf.net/p/uiq3/code/trunk/Java/Calculator/Library/src/main/java/net/sourceforge/uiq3/math/BCDFloat.java $
 * ************************************************************ }}}1 **********
 */

package net.sourceforge.uiq3.math;

import org.jetbrains.annotations.Contract;
import org.jetbrains.annotations.NonNls;
import org.jetbrains.annotations.NotNull;

import net.sourceforge.uiq3.Required;

/**
 * Arbitrary Precision Denormalized Floating Point Arithmetic.
 * <p>
 * For the use in an FX-602P Simulator the precision is fixed at 14 decimal digits.
 * </p>
 * <p>
 * Note: Local variables are used in trace, and x and y variables are sometimes used inverted to the parameters declared.
 * </p>
 */
@SuppressWarnings ({
   "UnnecessaryLocalVariable",
   "SuspiciousNameCombination",
   "FeatureEnvy",
   "UnnecessaryLabelOnBreakStatement",
   "ChainedMethodCall",
   "LawOfDemeter",
   "NonAsciiCharacters", "HardcodedLineSeparator"})
public final class BCDFloat
   implements
   Number
{
   /**
    * Floating Point Type
    * <dl>
    * <dt>Finite</dt>
    * <dd>Normal Real Number</dd>
    * <dt>NAN</dt>
    * <dd>Not a Number</dd>
    * <dt>Type_Positive_Infinity</dt>
    * <dd>+∞i</dd>
    * <dt>Type_Negative_Infinity</dt>
    * <dd>-∞</dd>
    * </dl>
    */
   public enum Float_Type
   {
      /**
       * <p>
       * Normal Real Number
       * </p>
       */
      Finite (""),
      /**
       * <p>
       * Not a Number
       * </p>
       */
      NaN ("NaN"),
      /**
       * <p>
       * -∞
       * </p>
       */
      Negative_Infinity ("-Inf"),
      /**
       * <p>
       * +∞
       * </p>
       */
      Positive_Infinity ("+Inf");
      /**
       * <p>
       * Integer representation of enum value.
       * </p>
       */
      @NonNls
      @NotNull
      public final String Image;

      /**
       * <p>
       * get voucher type from database value.
       * </p>
       *
       * @param Image
       *    "", "NaN", "+Inf", "-Inf"
       * @return the floating point type
       * @throws ArrayIndexOutOfBoundsException
       *    not "", "NaN", "+Inf", "-Inf"
       */
      @NotNull
      public static BCDFloat.Float_Type value (final String Image)
	 throws
	 ArrayIndexOutOfBoundsException
      {
	 final BCDFloat.Float_Type Retval;

	 Search_Enum:
	 {
	    for (final BCDFloat.Float_Type i : Float_Type.values ())
	    {
	       if (i.Image.equalsIgnoreCase (Image))
	       {
		  Retval = i;
		  break Search_Enum;
	       } // if
	    } // for

	    throw new IllegalArgumentException ("No enum const " + BCDFloat.Float_Type.class + " for pos " + Image);
	 } // Search_Enum

	 return Retval;
      } // value

      Float_Type (@NotNull final String Image)
      {
	 this.Image = Image;
      } // Float_Type

      @NotNull
      @Override
      public String toString ()
      {
	 return this.Image;
      }
   } // Float_Type

   /**
    * <p>
    * Class logger instance.
    * </p>
    */
   private static final java.util.logging.Logger Logger;
   /**
    * Maximum supported Exponent + 1
    */
   private static final int Max_Exponent = 1000;
   /**
    * 0.5 π (18 digits)
    * <p>
    * Unlike the original BCDFloat from Kai G. Schwebke I use a fixed maximum precision defined at compile time - so
    * there is no need for truly arbitrary irrational constants.
    * </p>
    */
   @NotNull
   private static final BCDFloat Num_0_5_π;
   /**
    * 10
    */
   @NotNull
   private static final BCDFloat Num_10;
   /**
    * 1 divided by Euler's number (18 digits)
    * <p>
    * Unlike the original BCDFloat from Kai G. Schwebke I use a fixed maximum precision defined at compile time - so
    * there is no need for truly arbitrary irrational constants.
    * </p>
    */
   @NotNull
   private static final BCDFloat Num_1_div_e;
   /**
    * 2 π (18 digits)
    * <p>
    * Unlike the original BCDFloat from Kai G. Schwebke I use a fixed maximum precision defined at compile time - so
    * there is no need for truly arbitrary irrational constants.
    * </p>
    */
   @NotNull
   private static final BCDFloat Num_2_π;
   /**
    * Log<sub>e</sub>10 (18 digits)
    * <p>
    * Unlike the original BCDFloat from Kai G. Schwebke I use a fixed maximum precision defined at compile time - so
    * there is no need for truly arbitrary irrational constants.
    * </p>
    */
   @NotNull
   private static final BCDFloat Num_log_e_10;
   /**
    * <p>
    * Class logger tag.
    * </p>
    */
   private static final String TAG;
   /**
    * Not a number
    */
   @NotNull
   public static final BCDFloat NaN;
   /**
    * 0
    */
   @NotNull
   public static final BCDFloat Num_0;
   /**
    * 0.5
    */
   @NotNull
   public static final BCDFloat Num_0_5;
   /**
    * 1
    */
   @NotNull
   public static final BCDFloat Num_1;
   /**
    * 100, for example used for percent
    */
   @NotNull
   public static final BCDFloat Num_100;
   /**
    * 100, for example used for percent
    */
   @NotNull
   public static final BCDFloat Num_1000;
   /**
    * 180, for example used for half a circle in degree
    */
   @NotNull
   public static final BCDFloat Num_180;
   /**
    * 2
    */
   @NotNull
   public static final BCDFloat Num_2;
   /**
    * 200, for example used for half a circle in radiant.
    */
   @NotNull
   public static final BCDFloat Num_200;
   /**
    * 60, For example used for seconds in an hour.
    */
   @NotNull
   public static final BCDFloat Num_3600;
   /**
    * 60, For example used for minutes in an hour.
    */
   @NotNull
   public static final BCDFloat Num_60;
   /**
    * -1
    */
   @NotNull
   public static final BCDFloat Num_Minus_1;
   /**
    * Euler's number (18 digits)
    * <p>
    * Unlike the original BCDFloat from Kai G. Schwebke I use a fixed maximum precision defined at compile time - so
    * there is no need for truly arbitrary irrational constants.
    * </p>
    */
   @NotNull
   public static final BCDFloat Num_e;
   /**
    * π (18 digits)
    * <p>
    * Unlike the original BCDFloat from Kai G. Schwebke I use a fixed maximum precision defined at compile time - so
    * there is no need for truly arbitrary irrational constants.
    * </p>
    */
   @NotNull
   public static final BCDFloat Num_π;
   /**
    * <p>
    * coefficient - while a 18 digits number can also be represented as an long an 18digits multiplication need a
    * 36digits temporary and that is more then a long can do.
    * </p>
    * <p>
    * null for NaN and infinities
    * </p>
    */
   @NotNull
   private BCDInteger Coefficient;
   /**
    * current exponent - note that the exponent is based on the internal representation where the radix point is right
    * most. For example π is 314159265358979324×10<sup>-17</sup>.
    */
   private int Exponent;
   /**
    * Numeric Type
    * <dl>
    * <dt>Finite</dt>
    * <dd>Normal Real Number</dd>
    * <dt>NaN</dt>
    * <dd>Not a Number</dd>
    * <dt>Type_Positive_Infinity</dt>
    * <dd>+∞dd>
    * <dt>Type_Negative_Infinity</dt>
    * <dd>-∞</dd>
    * </dl>
    */
   @NotNull
   private Float_Type Type;

   /**
    * Create empty value
    */
   public BCDFloat ()
   {
      super ();

      this.Exponent = 0;
      this.Coefficient = new BCDInteger ();
      this.Type = BCDFloat.Float_Type.Finite;

      return;
   } // BCDFloat

   /**
    * create new value from existing one.
    *
    * @param Value
    *    value to init the instance from
    */
   public BCDFloat (@NotNull final BCDFloat Value)
   {
      super ();
      // BCDFloat.Logger.entering (BCDFloat.TAG, "BCDFloat", Value);

      Required.notNull (Value);

      this.Coefficient = new net.sourceforge.uiq3.math.BCDInteger (Value.Coefficient);
      this.Exponent = Value.Exponent;
      this.Type = Value.Type;

      // BCDFloat.Logger.exiting (BCDFloat.TAG, "BCDFloat", this);
      return;
   } // BCDFloat

   /**
    * create new value from existing one.
    *
    * @param Value
    *    value to init the instance from
    */
   public BCDFloat (@NotNull final BCDInteger Value)
   {
      super ();

      Required.notNull (Value);

      this.Coefficient = new BCDInteger (Value);
      this.Exponent = 0;
      this.Type = BCDFloat.Float_Type.Finite;

      return;
   } // BCDFloat

   /**
    * create from java long
    *
    * @param Value
    *    value to init the instance from
    */
   public BCDFloat (final long Value)
   {
      super ();

      this.Exponent = 0;
      this.Coefficient = new BCDInteger (Value);
      this.Type = BCDFloat.Float_Type.Finite;

      return;
   } // BCDFloat

   /**
    * create from Coefficient and Exponent
    *
    * @param Coefficient
    *    Coefficient
    * @param Exponent
    *    Exponent
    */
   public BCDFloat (final long Coefficient, final int Exponent)
   {
      super ();

      Required.range (Exponent, -BCDFloat.Max_Exponent, +BCDFloat.Max_Exponent);

      this.Exponent = Exponent;
      this.Coefficient = new BCDInteger (Coefficient);
      this.Type = BCDFloat.Float_Type.Finite;

      return;
   } // BCDFloat

   /**
    * create from string
    *
    * @param Value
    *    string to create float form
    */
   public BCDFloat (final String Value)
   {
      super ();
      // BCDFloat.Logger.entering (BCDFloat.TAG, "BCDFloat", Value);

      Required.notNull (Value);

      @NonNls
      String s = Value;

      if (s.equalsIgnoreCase (BCDFloat.Float_Type.NaN.Image))
      {
	 this.Coefficient = new BCDInteger ();
	 this.Type = BCDFloat.Float_Type.NaN;
      }
      else if (s.equalsIgnoreCase (BCDFloat.Float_Type.Positive_Infinity.Image))
      {
	 this.Coefficient = new BCDInteger ();
	 this.Type = BCDFloat.Float_Type.Positive_Infinity;
      }
      else if (s.equalsIgnoreCase (BCDFloat.Float_Type.Negative_Infinity.Image))
      {
	 this.Coefficient = new BCDInteger ();
	 this.Type = BCDFloat.Float_Type.Negative_Infinity;
      }
      else
      {
	 // search for exponent, parse, cut
	 int Index_E = s.indexOf ('e');

	 if (Index_E == (-1))
	 {
	    Index_E = s.indexOf ('E');
	 } // if

	 if (Index_E != (-1))
	 {
	    final String es;

	    // Numbers formatted for the serial port might have a space after the E
	    // character
	    if (s.charAt (Index_E + 1) == ' ')
	    {
	       es = s.substring (Index_E + 2);
	    }
	    else
	    {
	       es = s.substring (Index_E + 1);
	    }

	    try
	    {
	       this.Exponent = Integer.parseInt (es);
	    }
	    catch (@NotNull final NumberFormatException Exception)
	    {
	       BCDFloat.Logger.log (java.util.logging.Level.SEVERE, "LOG01060:", Exception);
	    } // try
	    s = s.substring (0, Index_E);
	 }

	 // search for decimal point, cut
	 final int Index_Dot = s.indexOf ('.');

	 if (Index_Dot != (-1))
	 {
	    this.Exponent -= s.length () - Index_Dot - 1;
	    s = s.substring (0, Index_Dot) + s.substring (Index_Dot + 1);
	 }

	 this.Coefficient = new BCDInteger (s);
	 this.Type = BCDFloat.Float_Type.Finite;
      } // if

      // BCDFloat.Logger.entering (BCDFloat.TAG, "BCDFloat", this);
      return;
   } // BCDFloat

   /**
    * x ÷ y
    *
    * @param Y
    *    divisor
    * @return x ÷ y
    */
   @Contract (pure = true)
   @NotNull
   @Override
   public final Number $div (@NotNull final Number Y)
   {
      final Number Retval;

      if (Y instanceof BCDFraction)
      {
	 if (this.Is_Integer ())
	 {
	    Retval = this.As_BCDFraction ().$div (Y);
	 }
	 else
	 {
	    Retval = this.$div (Y.As_BCDFloat ());
	 } // if
      }
      else
      {
	 final BCDFloat Result = new BCDFloat ();
	 final BCDFloat y = Y.As_BCDFloat ();

	 if ((this.Type == BCDFloat.Float_Type.NaN) || (y.Type == BCDFloat.Float_Type.NaN) || (
	    this.Is_Zero () && y
	       .Is_Zero ()) || (
	    (
	       (this.Type == BCDFloat.Float_Type.Positive_Infinity) ||
		  (this.Type == BCDFloat.Float_Type.Negative_Infinity)) &&
	       (
		  (y.Type == BCDFloat.Float_Type.Positive_Infinity) ||
		     (y.Type == BCDFloat.Float_Type.Negative_Infinity))))
	 {
	    Result.Type = BCDFloat.Float_Type.NaN;
	 }
	 else if (((this.Type == BCDFloat.Float_Type.Positive_Infinity) && (!y.Is_Negative ())) ||
	    ((this.Type == BCDFloat.Float_Type.Negative_Infinity) && (y.Is_Negative ())))
	 {
	    Result.Type = BCDFloat.Float_Type.Positive_Infinity;
	 }
	 else if (((this.Type == BCDFloat.Float_Type.Negative_Infinity) && (!y.Is_Negative ())) ||
	    ((this.Type == BCDFloat.Float_Type.Positive_Infinity) && (y.Is_Negative ())))
	 {
	    Result.Type = BCDFloat.Float_Type.Negative_Infinity;
	 }
	 else // noinspection StatementWithEmptyBody
	    if ((y.Type == BCDFloat.Float_Type.Positive_Infinity) || (y.Type == BCDFloat.Float_Type.Negative_Infinity))
	    {
	       // do nothing
	    }
	    else if (y.Is_Zero ())
	    {
	       if (this.Is_Negative ())
	       {
		  Result.Type = BCDFloat.Float_Type.Negative_Infinity;
	       }
	       else
	       {
		  Result.Type = BCDFloat.Float_Type.Positive_Infinity;
	       } // if
	    } // if
	    else
	    {
	       final int idealExp = this.Exponent - y.Exponent;
	       Result.Exponent = idealExp;

	       BCDInteger coeffU = new BCDInteger (this.Coefficient);
	       coeffU.Normalize ();
	       final BCDInteger coeffV = new BCDInteger (y.Coefficient);
	       coeffV.Normalize ();

	       if (coeffU.Num_Digits () < (Number.Max_Precision + 1 + coeffV.Num_Digits ()))
	       {
		  final int corr = (Number.Max_Precision + 1 + coeffV.Num_Digits ()) - coeffU.Num_Digits ();
		  coeffU = coeffU.Shift_Right (corr);
		  Result.Exponent -= corr;
	       } // if

	       Result.Coefficient = coeffU.$div (coeffV).As_BCDInteger ();
	       Result.Round_Max ();

	       if (Result.Exponent < idealExp)
	       {
		  Result.Normalize (idealExp);
	       } // if

	       if (Result.Exponent > idealExp)
	       {
		  Result.Coefficient = Result.Coefficient.Shift_Right (Result.Exponent - idealExp);
		  Result.Exponent -= Result.Exponent - idealExp;
	       } // if
	    } // if

	 Retval = Result;
      } // if
      return Retval;
   } // div

   /**
    * x - y
    *
    * @param Y
    *    value to subtract
    * @return x - y
    */
   @Contract (pure = true)
   @NotNull
   @Override
   public Number $minus (@NotNull final Number Y)
   {
      //BCDFloat.Logger.entering (TAG, "$minus", new Object[] {this, Y});

      final Number Retval;

      if (Y instanceof BCDFraction)
      {
	 if (this.Is_Integer ())
	 {
	    Retval = this.As_BCDFraction ().$minus (Y);
	 }
	 else
	 {
	    Retval = this.$minus (Y.As_BCDFloat ());
	 } // if
      }
      else
      {
	 final BCDFloat v = new BCDFloat (Y.As_BCDFloat ());

	 v.Coefficient = v.Coefficient.neg ().As_BCDInteger ();

	 if (v.Type == BCDFloat.Float_Type.Positive_Infinity)
	 {
	    v.Type = BCDFloat.Float_Type.Negative_Infinity;
	 }
	 else if (v.Type == BCDFloat.Float_Type.Negative_Infinity)
	 {
	    v.Type = BCDFloat.Float_Type.Positive_Infinity;
	 } // if

	 Retval = this.$plus (v);
      }

      //BCDFloat.Logger.exiting (TAG, "$minus", Retval);
      return Retval;
   } // sub

   /**
    * x + Y
    *
    * @param Y
    *    value to add
    * @return x + Y
    */
   @Contract (pure = true)
   @NotNull
   @Override
   public final Number $plus (@NotNull final Number Y)
   {
      // BCDFloat.Logger.entering ( BCDFloat.TAG, "$plus", new Object [] { this, Y });

      final Number Retval;
      final BCDFloat y = Y.As_BCDFloat ();

      if (Y instanceof BCDFraction)
      {
	 if (this.Is_Integer ())
	 {
	    Retval = this.As_BCDFraction ().$plus (Y);
	 }
	 else
	 {
	    Retval = this.$plus (Y.As_BCDFloat ());
	 } // if
      }
      else if (this.Is_Zero ())
      {
	 // zero + very small numbers might be handled incorrectly by the normal add.

	 if (y.Is_Zero ())
	 {
	    // this is done to handel the case where y is (-0) — (0) + (-0) should be (+0)

	    Retval = new BCDFloat (BCDFloat.Num_0);
	 }
	 else
	 {
	    Retval = new BCDFloat (y);
	 } // if
      }
      else
      {
	 final BCDFloat Result = new BCDFloat ();

	 if ((this.Type == BCDFloat.Float_Type.NaN) || (y.Type == BCDFloat.Float_Type.NaN) ||
	    (
	       (this.Type == BCDFloat.Float_Type.Positive_Infinity) &&
		  (y.Type == BCDFloat.Float_Type.Negative_Infinity)) ||
	    ((this.Type == BCDFloat.Float_Type.Negative_Infinity) && (y.Type == BCDFloat.Float_Type.Positive_Infinity)))
	 {
	    Result.Type = BCDFloat.Float_Type.NaN;
	 }
	 else if ((this.Type == BCDFloat.Float_Type.Positive_Infinity) ||
	    (y.Type == BCDFloat.Float_Type.Positive_Infinity))
	 {
	    Result.Type = BCDFloat.Float_Type.Positive_Infinity;
	 }
	 else if ((this.Type == BCDFloat.Float_Type.Negative_Infinity) ||
	    (y.Type == BCDFloat.Float_Type.Negative_Infinity))
	 {
	    Result.Type = BCDFloat.Float_Type.Negative_Infinity;
	 } // if
	 else
	 {
	    final int de = Math.abs (this.Exponent - y.Exponent);

	    if (de >= ((2 * Number.Max_Precision) + 2))
	    {
	       if (this.Exponent > y.Exponent)
	       {
		  Result.Coefficient = new BCDInteger (this.Coefficient);
		  Result.Exponent = this.Exponent;
	       }
	       else
	       {
		  Result.Coefficient = new BCDInteger (y.Coefficient);
		  Result.Exponent = y.Exponent;
	       } // if
	    }
	    else
	    {
	       BCDInteger coeff = this.Coefficient;
	       BCDInteger coeffV = y.Coefficient;
	       Result.Exponent = y.Exponent;
	       if (this.Exponent > y.Exponent)
	       {
		  coeff = this.Coefficient.Shift_Right (de);
	       } // if
	       if (y.Exponent > this.Exponent)
	       {
		  coeffV = y.Coefficient.Shift_Right (de);
		  Result.Exponent = this.Exponent;
	       } // if
	       Result.Coefficient = coeff.$plus (coeffV).As_BCDInteger ();
	       Result.Round_Max ();
	    } // if
	 } // if
	 Retval = Result;
      } // if

      // BCDFloat.Logger.exiting (BCDFloat.TAG, "$plus", Retval);
      return Retval;
   } // add

   /**
    * x × Y
    *
    * @param Y
    *    Multiplier
    * @return x × Y
    */
   @Contract (pure = true)
   @NotNull
   @Override
   public final Number $times (@NotNull final Number Y)
   {
      final Number Retval;

      if (Y instanceof BCDFraction)
      {
	 if (this.Is_Integer ())
	 {
	    Retval = this.As_BCDFraction ().$times (Y);
	 }
	 else
	 {
	    Retval = this.$times (Y.As_BCDFloat ());
	 } // if
      }
      else
      {
	 final BCDFloat Result = new BCDFloat ();
	 final BCDFloat y = Y.As_BCDFloat ();

	 if ((this.Type == BCDFloat.Float_Type.NaN) || (y.Type == BCDFloat.Float_Type.NaN))
	 {
	    Result.Type = BCDFloat.Float_Type.NaN;
	 }
	 else if ((
	    (this.Type == BCDFloat.Float_Type.Positive_Infinity) &&
	       (y.Type == BCDFloat.Float_Type.Positive_Infinity)) ||
	    ((this.Type == BCDFloat.Float_Type.Negative_Infinity) && (y.Type == BCDFloat.Float_Type.Negative_Infinity)))
	 {
	    Result.Type = BCDFloat.Float_Type.Positive_Infinity;
	 }
	 else if ((
	    (this.Type == BCDFloat.Float_Type.Positive_Infinity) &&
	       (y.Type == BCDFloat.Float_Type.Negative_Infinity)) ||
	    ((this.Type == BCDFloat.Float_Type.Negative_Infinity) && (y.Type == BCDFloat.Float_Type.Positive_Infinity)))
	 {
	    Result.Type = BCDFloat.Float_Type.Negative_Infinity;
	 }
	 else if (((this.Type == BCDFloat.Float_Type.Negative_Infinity) && (!y.Is_Negative ())) ||
	    ((this.Type == BCDFloat.Float_Type.Positive_Infinity) && (y.Is_Negative ())) ||
	    ((y.Type == BCDFloat.Float_Type.Negative_Infinity) && (!this.Is_Negative ())) ||
	    ((y.Type == BCDFloat.Float_Type.Positive_Infinity) && (this.Is_Negative ())))
	 {
	    Result.Type = BCDFloat.Float_Type.Negative_Infinity;
	 }
	 else if (((this.Type == BCDFloat.Float_Type.Positive_Infinity) && (!y.Is_Negative ())) ||
	    ((this.Type == BCDFloat.Float_Type.Negative_Infinity) && (y.Is_Negative ())) ||
	    ((y.Type == BCDFloat.Float_Type.Positive_Infinity) && (!this.Is_Negative ())) ||
	    ((y.Type == BCDFloat.Float_Type.Negative_Infinity) && (this.Is_Negative ())))
	 {
	    Result.Type = BCDFloat.Float_Type.Negative_Infinity;
	 }
	 else
	 {
	    Result.Coefficient = this.Coefficient.$times (y.Coefficient).As_BCDInteger ();
	    Result.Exponent = this.Exponent + y.Exponent;

	    Result.Round_Max ();
	 }

	 Retval = Result;
      } // if
      return Retval;
   } // mul

   /**
    * x<sup>Y</sup>
    *
    * @param Y
    *    power
    * @return x<sup>y</sup>
    * @throws BCDError
    *    calculation error
    */
   @Contract (pure = true)
   @NotNull
   @Override
   public final Number $times$times (@NotNull final Number Y)
      throws
      BCDError
   {
      // BCDFloat.Logger.entering (BCDFloat.TAG, "$times$times", this, Y);

      Number Retval;

      if (Y.Is_Zero ())
      {
	 if (this.Is_Zero ())
	 {
	    Retval = BCDFloat.NaN;
	 }
	 else
	 {
	    Retval = BCDFloat.Num_1;
	 } // if
      }
      else if (Y.equals (BCDFloat.Num_1))
      {
	 Retval = new BCDFloat (this);
      }
      else if (this.Is_Zero ())
      {
	 Retval = BCDFloat.Num_0;
      }
      else if (this.equals (BCDFloat.Num_1))
      {
	 Retval = BCDFloat.Num_1;
      }
      else
      {
	 if (Y.Is_Integer ())
	 {
	    final long y_long = Y.To_Long ();
	    boolean neg = false;

	    if (y_long < 0)
	    {
	       neg = true;
	    } // if

	    Retval = this;

	    final long y_neg = neg
	       ? -y_long
	       : y_long;
	    for (int i = 1; i < y_neg; i = i + 1)
	    {
	       Retval = Retval.$times (this);
	    }

	    if (neg)
	    {
	       Retval = BCDFloat.Num_1.$div (Retval);
	    } // if
	 }
	 else
	 {
	    if (this.To_Long () >= 0)
	    {
	       // exp(Y*log(x));
	       final Number ln_x = this.log_e ();
	       final Number ln_pow = Y.$times (ln_x);
	       Retval = ln_pow.exp_e ();
	    }
	    else
	    {
	       Retval = BCDFloat.NaN;
	    } // if
	 } // if
      } // if

      // BCDFloat.Logger.exiting (BCDFloat.TAG, "pow", Retval);
      return Retval;
   }// pow

   /**
    * <p>
    * Either convert to a BCDFloat or return this.
    * </p>
    *
    * @return value as java int
    */
   @SuppressWarnings ("ReturnOfThis")
   @NotNull
   @Override
   public BCDFloat As_BCDFloat ()
   {
      return this;
   } // As_BCDFloat

   /**
    * <p>
    * Either convert to a or return this.
    * </p>
    *
    * @return value as java int
    */
   @SuppressWarnings ("ReturnOfThis")
   @Override
   @NotNull
   public BCDFraction As_BCDFraction ()
   {
      net.sourceforge.uiq3.Assumed.isTrue (this.Is_Integer (), "Is_Integer");

      return this.As_BCDInteger ().As_BCDFraction ();
   } // As_BCDFraction

   /**
    * <p>
    * Either convert to a BCDInteger or return this.
    * </p>
    *
    * @return value as java int
    */
   @NotNull
   @Override
   public BCDInteger As_BCDInteger ()
   {
      // BCDFloat.Logger.entering (BCDFloat.TAG, "Integer");

      final BCDFloat Temp = new BCDFloat (this);

      Temp.Normalize ();

      // BCDFloat.Logger.log (java.util.logging.Level.FINE, "> Exponent  = " + Exponent);

      final int Int_Digits = Math.max ((Temp.Exponent + Temp.Coefficient.Num_Digits ()), 0);

      // BCDFloat.Logger.log (java.util.logging.Level.FINE, "> Int_Digits  = " + Int_Digits);
      // BCDFloat.Logger.log (java.util.logging.Level.FINE, "> Frac_Digits = " + Frac_Digits);

      final BCDInteger Retval;
      if (Int_Digits <= 0)
      {
	 Retval = BCDInteger.Num_0;
      }
      else
      {
	 Retval = new BCDInteger (Temp.To_Long ());
      } // if

      // BCDFloat.Logger.exiting (BCDFloat.TAG, "Integer");
      return Retval;
   } // As_BCDInteger

   /**
    * clone the float - but it might be easier to use the provided copy Constructor instead.
    *
    * @return a new BCDFloat from an existing one.
    * @see java.lang.Object#clone()
    */
   @NotNull
   @Override
   public Number Clone ()
   {
      return new BCDFloat (this);
   } // clone

   /**
    * <p>
    * coefficient - while a 18 digits number can also be represented as an long an 18digits multiplication need a
    * 36digits temporary and that is more then a long can do.
    * </p>
    *
    * @return Coefficient
    */
   @NotNull
   public BCDInteger Coefficient ()
   {
      return this.Coefficient;
   } // Coefficient

   /**
    * <p>
    * coefficient - while a 18 digits number can also be represented as an long an 18digits multiplication need a
    * 36digits temporary and that is more then a long can do.
    * </p>
    *
    * @return Coefficient
    * @throws BCDError
    *    when it's not a normal number
    */
   @Override
   public long Coefficient_As_Long ()
      throws
      BCDError
   {
      if (this.Type == BCDFloat.Float_Type.Finite)
      {
	 return this.Coefficient.To_Long ();
      }
      else
      {
	 throw new BCDError ("Not a finite number.");
      } // if
   } // Coefficient_As_Long

   /**
    * Compare values
    *
    * @param Value
    *    value to compare with
    * @return <dl>
    * <dt>Compare_Equal</dt>
    * <dd>Values are the same</dd>
    * <dt>Compare_Less</dt>
    * <dd>Left value less the right value</dd>
    * <dt>Compare_Greater</dt>
    * <dd>Left value greater the right value</dd>
    * </dl>
    * @throws BCDError
    *    numbers can not be compared.
    */
   @Override
   public final int Compare (@NotNull final Number Value)
      throws
      BCDError
   {
      // BCDFloat.Logger.entering (BCDFloat.TAG, "Compare", new Object [] {
      // this, Value });

      final BCDFloat Left = new BCDFloat (this);
      final BCDFloat Right = new BCDFloat (Value.As_BCDFloat ());

      if ((Left.Type != BCDFloat.Float_Type.Finite) || (Right.Type != BCDFloat.Float_Type.Finite))
      {
	 throw new BCDError ("Can compare only numbers.");
      } // if

      Left.Round_Effective ();
      Right.Round_Effective ();

      // BCDFloat.Logger.log (java.util.logging.Level.FINE, "  > Left              = " + Left);
      // BCDFloat.Logger.log (java.util.logging.Level.FINE, "  > Right             = " + Right);

      final int Left_Exponent = Left.Get_1_Normalized_Exponent ();
      final int Right_Exponent = Right.Get_1_Normalized_Exponent ();

      // BCDFloat.Logger.log (java.util.logging.Level.FINE, "  > Left Exponent     = " +
      // Left_Exponent);
      // BCDFloat.Logger.log (java.util.logging.Level.FINE, "  > Right Exponent    = " +
      // Right_Exponent);

      final int Retval;
      if (Left_Exponent > Right_Exponent)
      {
	 Retval = Number.Compare_Greater;
      }
      else if (Left_Exponent < Right_Exponent)
      {
	 Retval = Number.Compare_Less;
      }
      else
      {
	 final BCDInteger Left_Coefficient;
	 final BCDInteger Right_Coefficient;
	 final int Diff_Exponent = Left.Exponent () - Right.Exponent ();

	 // BCDFloat.Logger.log (java.util.logging.Level.FINE, "> Diff Exponent     = " +
	 // Diff_Exponent);

	 if (Diff_Exponent > 0)
	 {
	    Left_Coefficient = Left.Coefficient.Shift_Right (Diff_Exponent);
	    Right_Coefficient = Right.Coefficient;
	 }
	 else if (Diff_Exponent < 0)
	 {
	    Left_Coefficient = Left.Coefficient;
	    Right_Coefficient = Right.Coefficient.Shift_Right (-Diff_Exponent);
	 }
	 else
	 {
	    Left_Coefficient = Left.Coefficient;
	    Right_Coefficient = Right.Coefficient;
	 } // if

	 // BCDFloat.Logger.log (java.util.logging.Level.FINE, "  > Left Coefficient  = " +
	 // Left_Coefficient);
	 // BCDFloat.Logger.log (java.util.logging.Level.FINE, "  > Right Coefficient = " +
	 // Right_Coefficient);

	 Retval = Left_Coefficient.Compare (Right_Coefficient);
      } // if

      // BCDFloat.Logger.exiting (BCDFloat.TAG, "Compare", Retval);
      return Retval;
   } // Compare

   /**
    * number: sum(i = 0 ... Num_Digits-1; Digits[i]*10^i)
    */
   @Override
   public byte[] Digits ()
   {
      return this.Coefficient.Digits ();
   } // Digits

   /**
    * current exponent - note that the exponent is based on the internal representation where the radix point is right
    * most. For example π is 314159265358979324×10<sup>-17</sup>.
    *
    * @return Current exponent.
    * @throws BCDError
    *    when the value is not finite
    */
   @Override
   public final int Exponent ()
      throws
      BCDError
   {
      if (this.Type == BCDFloat.Float_Type.Finite)
      {
	 return this.Exponent;
      }
      else
      {
	 throw new BCDError ("Not a finite number.");
      } // if
   } // Exponent

   /**
    * Round to a given Precision after the decimal point.
    *
    * @param Decimal
    *    decimal digits to keep
    * @param Exponent
    *    max exponent
    * @return rounded value
    */
   @NotNull
   @Override
   public final BCDFloat Fix (final int Decimal, final int Exponent)
   {
      BCDFloat.Logger.entering (BCDFloat.TAG, "Fix", new Object[] {this, Decimal, Exponent});

      Required.isNatural (Decimal, "Decimal");
      Required.isPositive (Exponent, "Exponent");

      BCDFloat Retval = new BCDFloat (this);

      if (Retval.Type == BCDFloat.Float_Type.Finite)
      {
	 Retval.Normalize ();

	 final int Int_Digits = Math.max ((Retval.Exponent + Retval.Coefficient.Num_Digits ()), 0);
	 final int Frac_Digits = Retval.Coefficient.Num_Digits () > Int_Digits
	    ? Retval.Coefficient.Num_Digits () - Int_Digits
	    : 0;
	 final int Zero_Digits = (Int_Digits == 0) || (Frac_Digits < -Retval.Exponent)
	    ? -Retval.Exponent - Frac_Digits
	    : 0;
	 final int Precision = (Decimal + Int_Digits) - Zero_Digits;

	 BCDFloat.Logger.log (
	    java.util.logging.Level.FINE,
	    "\n> Int_Digits  = {0} " +
	       "\n> Frac_Digits = {1} " +
	       "\n> Zero_Digits = {2} " +
	       "\n> Precision   = {3} ",
	    new Object[] {Int_Digits, Frac_Digits, Zero_Digits, Precision});

	 if (Int_Digits == 0 && Zero_Digits > Decimal)
	 {
	    // handle the case where there are more 0 at the beginning of the fractions then there are
	    Retval = BCDFloat.Num_0;
	 }
	 else if (Precision < 1)
	 {
	    // handle the case where there numbers with only fractions are rounded to fraction 0 digits.
	    // If that happens the result can only be 0, 1 and -1.
	    //
	    final byte[] Digits = Retval.Coefficient.Digits ();

	    if (Digits.length == 0 || Digits[Digits.length -1 ] < 5)
	    {
	       Retval = BCDFloat.Num_0;
	    }
	    else
	    {
	       Retval.Round_To (0, Exponent);
	    } // if
	 }
	 else if ((Frac_Digits + Zero_Digits) > Decimal)
	 {
	    Retval.Round_To (Precision, Exponent);
	 } // if
      } // if

      Retval.Normalize ();

      BCDFloat.Logger.exiting (BCDFloat.TAG, "Fix", Retval);
      return Retval;
   }// Fix

   /**
    * Recompose a frac double from the given data.
    *
    * @return a double
    */
   @NotNull
   @Override
   public final BCDFloat Frac ()
   {
      // BCDFloat.Logger.entering (BCDFloat.TAG, "Compose_Frac");

      BCDFloat Retval = new BCDFloat (this);

      Retval.Normalize ();

      final int Num_Digits = Retval.Coefficient.Num_Digits ();
      final int Int_Digits = Math.max ((Retval.Exponent + Num_Digits), 0);
      final int Frac_Digits = Num_Digits > Int_Digits
	 ? Num_Digits - Int_Digits
	 : 0;

      if (Frac_Digits <= 0)
      {
	 Retval = BCDFloat.Num_0;
      }
      else if (Int_Digits > 0)
      {
	 Retval = Retval.$minus (Retval.Integer ()).As_BCDFloat ();
      } // if

      // BCDFloat.Logger.exiting (BCDFloat.TAG, "Compose_Frac");
      return Retval;
   } // Compose

   /**
    * current exponent so that the ratix point is 1 digit to right. For example π would be
    * 3.14159265358979324×10<sup>0</sup>.
    *
    * @return Current exponent.
    */
   public final int Get_1_Normalized_Exponent ()
   {
      final int Retval;

      if (this.Type == BCDFloat.Float_Type.Finite)
      {
	 Retval = (this.Exponent + this.Coefficient.Num_Digits ()) - 1;
      }
      else
      {
	 Retval = 0;
      } // if

      return Retval;
   } // Get_1_Normalized_Exponent

   /**
    * <p>
    * Get valuef or indirect addressing.
    * </p>
    *
    * @param Digit_Count
    *    Should be 1 … 2 depending if value is used to access memory or perform a goto.
    * @return int value for indirect addressing.
    */
   @Override
   public int Indirect_Value (final int Digit_Count)
   {
      // BCDFloat.Logger.entering (BCDFloat.TAG, "Indirect_Value");

      final int Retval;
      final Number Integer = this.Integer ();
      final byte[] Digits = Integer.Digits ();

      if (Digit_Count == 2)
      {
	 final int Exponent;

	 try
	 {
	    Exponent = Integer.Exponent ();
	 }
	 catch (@NotNull final BCDError bcdError)
	 {
	    throw new IllegalStateException ("Value should not contain a NAN value.", bcdError);
	 } // try

	 // BCDFloat.Logger.fine ("> Integer     = " + Integer);
	 // BCDFloat.Logger.fine ("> Digits      = " + java.util.Arrays.toString (Digits));

	 if (Digits.length > 1)
	 {
	    Retval = (Digits[Digits.length - 1] * 10) + Digits[Digits.length - 2];
	 }
	 else if (Exponent > 0)
	 {
	    Retval = Digits[Digits.length - 1] * 10;
	 }
	 else if (Digits.length > 0)
	 {
	    Retval = Digits[Digits.length - 1];
	 }
	 else
	 {
	    Retval = 0;
	 } // if
      }
      else if (Digit_Count == 1)
      {
	 if (Digits.length > 0)
	 {
	    Retval = Digits[Digits.length - 1];
	 }
	 else
	 {
	    Retval = 0;
	 } // if
      }
      else
      {
	 throw new java.lang.IllegalArgumentException ("Only 1 … 2 are allowed for Digit_Count");
      } // if

      // BCDFloat.Logger.exiting (BCDFloat.TAG, "Indirect_Value", Retval);
      return Retval;
   } // Indirect_Value

   /**
    * integer part of the BCDFloat
    * <p>
    * Do not mix up with To_Integer - This version returns another BCDFloat!
    * </p>
    *
    * @return integer part of this
    */
   @NotNull
   @Override
   public Number Integer ()
   {
      // BCDFloat.Logger.entering (BCDFloat.TAG, "Integer");

      BCDFloat Retval = new BCDFloat (this);

      Retval.Normalize ();
      Retval.Round_Max ();

      // BCDFloat.Logger.log (java.util.logging.Level.FINE, "> Exponent  = " + this.Exponent);

      final int Num_Digits = Retval.Coefficient.Num_Digits ();
      final int Int_Digits = Math.max ((Retval.Exponent + Num_Digits), 0);
      final int Frac_Digits = Num_Digits > Int_Digits
	 ? Num_Digits - Int_Digits
	 : 0;

      // BCDFloat.Logger.log (java.util.logging.Level.FINE, "> Int_Digits  = " + Int_Digits);
      // BCDFloat.Logger.log (java.util.logging.Level.FINE, "> Frac_Digits = " + Frac_Digits);

      if (Int_Digits <= 0)
      {
	 Retval = BCDFloat.Num_0;
      }
      else if (Frac_Digits > 0)
      {
	 Retval = new BCDFloat (Retval.To_Long ());
      } // if

      // BCDFloat.Logger.exiting (BCDFloat.TAG, "Integer", Retval);
      return Retval;
   } // Integer

   /**
    * current value is a normal number
    *
    * @return true when it is a normal number
    */
   @Override
   public final boolean Is_Finite ()
   {
      return this.Type == BCDFloat.Float_Type.Finite;
   } // Is_Finite

   /**
    * current value is Infinity (but not N/A).
    *
    * @return true when x is infinite
    */
   @Override
   public final boolean Is_Infinite ()
   {
      return (this.Type == BCDFloat.Float_Type.Negative_Infinity) ||
	 (this.Type == BCDFloat.Float_Type.Positive_Infinity);
   } // Is_Infinite

   /**
    * Current value is integer value.
    *
    * @return true when x is an integer number
    */
   @Override
   public boolean Is_Integer ()
   {
      final BCDFloat x = new BCDFloat (this);

      x.Normalize ();

      final int Num_Digits = x.Coefficient.Num_Digits ();
      final int Int_Digits = Math.max ((x.Exponent + Num_Digits), 0);
      final int Frac_Digits = Num_Digits > Int_Digits
	 ? Num_Digits - Int_Digits
	 : 0;

      return Frac_Digits == 0;
   } // Is_Integer

   /**
    * current value is not a number
    *
    * @return when is not a number
    */
   @Override
   public final boolean Is_NaN ()
   {
      return this.Type == BCDFloat.Float_Type.NaN;
   } // Is_NaN

   /**
    * Current value us negative
    *
    * @return true when x &lt;= -0
    */
   @Override
   public final boolean Is_Negative ()
   {
      return (this.Type == BCDFloat.Float_Type.Negative_Infinity) || (this.Coefficient.Is_Negative ());
   } // Is_Negative

   /**
    * value is 0
    *
    * @return true when x is 0
    */
   @Override
   public final boolean Is_Zero ()
   {
      return ((this.Type == BCDFloat.Float_Type.Finite) && (this.Coefficient.Is_Zero ()));
   } // Is_Zero

   /**
    * <p>
    * normalize the value
    * </p>
    */
   @Override
   public final void Normalize ()
   {
      if (this.Type == BCDFloat.Float_Type.Finite)
      {
	 this.Coefficient.Normalize ();
	 int l = this.Coefficient.Num_Digits ();

	 // noinspection MethodCallInLoopCondition
	 while ((l > 1) && (this.Coefficient.Digits ()[0] == 0))
	 {
	    this.Coefficient = this.Coefficient.Shift_Left (1);
	    this.Exponent = this.Exponent + 1;
	    l = l - 1;
	 } // while
	 if (l == 0)
	 {
	    this.Exponent = 0;
	 } // if
      } // if
      return;
   } // Normalize

   /**
    * @param maxE
    *    maximum exponent
    */
   private void Normalize (final int maxE)
   {
      if (this.Type == BCDFloat.Float_Type.Finite)
      {
	 this.Coefficient.Normalize ();
	 int l = this.Coefficient.Num_Digits ();

	 // noinspection MethodCallInLoopCondition
	 while ((l > 1) && (this.Coefficient.Digits ()[0] == 0) && (this.Exponent < maxE))
	 {
	    this.Coefficient = this.Coefficient.Shift_Left (1);
	    this.Exponent = this.Exponent + 1;
	    l = l - 1;
	 } // while

	 if (l == 0)
	 {
	    this.Exponent = 0;
	 } // if
      } // if
      return;
   } // Normalize

   /**
    * the current number of digits - not that when the number is not normalised there might be more 0's in the number
    * then strictly needed.
    *
    * @return number of digits
    * @throws BCDError
    *    Not a finite number.
    */
   @Override
   public int Num_Digits ()
      throws
      BCDError
   {
      final int Retval;

      if (this.Type == BCDFloat.Float_Type.Finite)
      {
	 this.Coefficient.Normalize ();
	 Retval = this.Coefficient.Num_Digits ();
      }
      else if (this.Type == BCDFloat.Float_Type.NaN)
      {
	 Retval = 0;
      }
      else
      {
	 throw new BCDError ("Not a finite number.");
      } // if

      return Retval;
   } // Get_Num_Digits

   /**
    * Convert Polar to Rectangle Coordinates
    *
    * @param θ
    *    angle
    * @return x = cos (w) × r
    */
   @Contract (pure = true)
   @NotNull
   @Override
   public final Number P_To_X (@NotNull final Number θ)
   {
      // BCDFloat.Logger.entering (BCDFloat.TAG, "P_To_X");

      final Number Retval = θ.cos ().$times (this);

      // BCDFloat.Logger.exiting (BCDFloat.TAG, "P_To_X");
      return Retval;
   } // P_To_X

   /**
    * Convert Polar to Rectangle Coordinates
    *
    * @param θ
    *    angle
    * @param Half_Circle
    *    180 for deg, 200 for gra, pi for rad
    * @return x = cos (w) × r
    */
   @Contract (pure = true)
   @NotNull
   @Override
   public final Number P_To_X (@NotNull final Number θ, @NotNull final Number Half_Circle)
   {
      // BCDFloat.Logger.entering (BCDFloat.TAG, "P_To_X");

      final Number Retval = θ.cos (Half_Circle).$times (this);

      // BCDFloat.Logger.exiting (BCDFloat.TAG, "P_To_X");
      return Retval;
   } // P_To_X

   /**
    * Convert Polar to Rectangle Coordinates
    *
    * @param θ
    *    angle
    * @return y = sin (θ) × r
    */
   @Contract (pure = true)
   @NotNull
   @Override
   public final Number P_To_Y (@NotNull final Number θ)
   {
      // BCDFloat.Logger.entering (BCDFloat.TAG, "P_To_Y");

      final Number Retval;

      if (this.Is_Negative () && θ.Is_Zero ())
      {
	 Retval = BCDFloat.NaN;
      }
      else
      {
	 Retval = θ.sin ().$times (this);
      } // if

      // BCDFloat.Logger.exiting (BCDFloat.TAG, "P_To_Y");
      return Retval;
   } // P_To_Y

   /**
    * Convert Polar to Rectangle Coordinates
    *
    * @param θ
    *    angle
    * @param Half_Circle
    *    180 for deg, 200 for gra, pi for rad
    * @return y = sin (θ) × r
    */
   @Contract (pure = true)
   @NotNull
   @Override
   public final Number P_To_Y (@NotNull final Number θ, @NotNull final Number Half_Circle)
   {
      // BCDFloat.Logger.entering (BCDFloat.TAG, "P_To_Y");

      final Number Retval;

      if (this.Is_Negative () && θ.Is_Zero ())
      {
	 Retval = BCDFloat.NaN;
      }
      else
      {
	 Retval = θ.sin (Half_Circle).$times (this);
      } // if

      // BCDFloat.Logger.exiting (BCDFloat.TAG, "P_To_Y");
      return Retval;
   } // P_To_Y

   /**
    * Convert Rectangle to Polar Coordinates
    *
    * @param y
    *    y position
    * @return r = √(x²+y²)
    */
   @Contract (pure = true)
   @NotNull
   @Override
   public final Number R_To_R (@NotNull final Number y)
   {
      // BCDFloat.Logger.entering (BCDFloat.TAG, "R_To_R");

      final Number x2_y2 = this.square ().$plus (y.square ());
      final Number Retval = x2_y2.square_root ();

      // BCDFloat.Logger.exiting (BCDFloat.TAG, "R_To_R");
      return Retval;
   } // R_To_R

   /**
    * Convert Rectangle to Polar Coordinates
    *
    * @param y
    *    y position
    * @return θ = arctan (y÷x)
    */
   @Contract (pure = true)
   @NotNull
   @Override
   public final Number R_To_θ (@NotNull final Number y)
   {
      // BCDFloat.Logger.entering (BCDFloat.TAG, "R_To_θ", y);

      Number Retval;

      if (this.Is_Zero () && y.Is_Zero ())
      {

	 Retval = BCDFloat.NaN;
      }
      else
      {
	 final Number Div = y.$div (this);
	 Retval = Div.arc_tan ();

	 if (this.Is_Zero () && y.Is_Negative ())
	 {
	    Retval = Retval.neg ();
	 }
	 else if (this.Is_Negative () && y.Is_Negative ())
	 {
	    Retval = BCDFloat.Num_π.$minus (Retval).neg ();
	 }
	 else if (this.Is_Negative ())
	 {
	    Retval = BCDFloat.Num_π.$plus (Retval);
	 } // if
      } // if

      // BCDFloat.Logger.exiting (BCDFloat.TAG, "R_To_θ");
      return Retval;
   } // R_To__θ

   /**
    * Convert Rectangle to Polar Coordinates
    *
    * @param y
    *    y position
    * @param Half_Circle
    *    180 for deg, 200 for gra, pi for rad
    * @return θ = arctan (y÷x)
    */
   @Contract (pure = true)
   @NotNull
   @Override
   public final Number R_To_θ (@NotNull final Number y, @NotNull final Number Half_Circle)
   {
      // BCDFloat.Logger.entering (BCDFloat.TAG, "R_To_θ");

      Number Retval;

      if (this.Is_Zero () && y.Is_Zero ())
      {

	 Retval = BCDFloat.NaN;
      }
      else
      {
	 final Number Div = y.$div (this);
	 Retval = Div.arc_tan (Half_Circle);

	 if (this.Is_Zero () && y.Is_Negative ())
	 {
	    Retval = Retval.neg ();
	 }
	 else if (this.Is_Negative () && y.Is_Negative ())
	 {
	    Retval = Half_Circle.$minus (Retval).neg ();
	 }
	 else if (this.Is_Negative ())
	 {
	    Retval = Half_Circle.$plus (Retval);
	 } // if
      } // if

      // BCDFloat.Logger.exiting (BCDFloat.TAG, "R_To_θ");
      return Retval;
   } // R_To_θ

   /**
    * Round to Effective and Max_Exponent.
    */
   @NotNull
   @Override
   public Number Round ()
   {
      return this.Round (Number.Effective_Precision, BCDFloat.Max_Exponent);
   } // Round_Effective

   /**
    * Round to a given Precision and Exponent.
    *
    * @param Precision
    *    Total precison
    * @param Exponent
    *    max exponent
    * @return rounded value
    */
   @NotNull
   @Override
   public final Number Round (final int Precision, final int Exponent)
   {
      // BCDFloat.Logger.entering (BCDFloat.TAG, "Round", new Object [] {
      // Precision, Exponent });

      Required.isPositive (Precision, "Precision");
      Required.isPositive (Exponent, "Exponent");

      final BCDFloat Retval = new BCDFloat (this);

      Retval.Round_To (Precision, Exponent);

      // BCDFloat.Logger.entering (BCDFloat.TAG, "Round", Retval);
      return Retval;
   } // round

   /**
    * Round to Effective and Max_Exponent.
    */
   private void Round_Effective ()
   {
      this.Round_To (Number.Effective_Precision, BCDFloat.Max_Exponent);

      return;
   } // Round_Effective

   /**
    * Round to Max_Precision and Max_Exponent.
    */
   private void Round_Max ()
   {
      this.Round_To (Number.Max_Precision, BCDFloat.Max_Exponent);
      return;
   } // Round_Max

   /**
    * Round to a given Precision and Exponent.
    *
    * @param Precision
    *    Total precision
    * @param Exponent
    *    max exponent
    */
   @SuppressWarnings ("ReuseOfLocalVariable")
   private void Round_To (final int Precision, final int Exponent)
   {
      //BCDFloat.Logger.entering (BCDFloat.TAG, "Round_To", new Object[] {this, Precision, Exponent});

      if (this.Type == BCDFloat.Float_Type.Finite)
      {
	 this.Coefficient.Normalize ();
	 int Length = this.Coefficient.Num_Digits ();
	 byte[] Digits = this.Coefficient.Digits ();

	 //BCDFloat.Logger.log (
	 //java.util.logging.Level.FINE,
	 //"Digits before trimming: {0} {1}",
	 //new Object[] {Length, java.util.Arrays.toString (Digits)});

	 if (Length > Precision)
	 {
	    final boolean Round_Up = Digits[Length - Precision - 1] >= 5;

	    this.Coefficient = this.Coefficient.Shift_Left (Length - Precision);
	    this.Exponent += Length - Precision;

	    if (Round_Up)
	    {
	       this.Coefficient = this.Coefficient.addUnsigned (1);
	    } // if
	 } // if

	 this.Coefficient.Normalize ();

	 Length = this.Coefficient.Num_Digits ();
	 Digits = this.Coefficient.Digits ();

	 //BCDFloat.Logger.log (
	 //java.util.logging.Level.FINE,
	 //"Digits before rounding: {0} {1}",
	 //new Object[] {Length, java.util.Arrays.toString (Digits)});

	 if (Length == Precision)
	 {
	    // 0.99999… is mathematically 1 and should be rounded as well.
	    // the simple 1 ÷ 3 × 3 - 1 won't give the correct result if don't
	    // CASIO calculators also round 0.99999…8 —
	    // the more complex is 1 ÷ 56 × 56 - 1 won't give the correct result if don't

	    Search_9:
	    {
	       // having to check if the first number (the number is stored least significant digit first) was 8
	       // needs some flags.
	       boolean first = true;
	       boolean was8 = false;

	       for (final byte digit : Digits)
	       {
		  final boolean first8 = first && (digit == 8);

		  if (first8)
		  {
		     was8 = true;
		  } // if

		  if ((digit != 9) && !first8)
		  {
		     // leave the loop when the first non 9 was found. unless it's the first digit then 8 is also
		     // a reason to continue.

		     break Search_9;
		  } // if

		  first = false;
	       } // for

	       if (was8)
	       {
		  // first digit was 8 and all digits are 9 => add 2

		  this.Coefficient = this.Coefficient.addUnsigned (2);
	       }
	       else
	       {
		  // all digits are 9 => add 1

		  this.Coefficient = this.Coefficient.addUnsigned (1);
	       } // if
	    } // Search_9
	 } // if

	 if (this.Exponent > Exponent)
	 {
	    if (this.Coefficient.Is_Negative ())
	    {
	       this.Type = BCDFloat.Float_Type.Negative_Infinity;
	    }
	    else
	    {
	       this.Type = BCDFloat.Float_Type.Positive_Infinity;
	    } // if
	 } // if

	 if (this.Exponent < -Exponent)
	 {
	    this.Coefficient = new BCDInteger ();
	    this.Exponent = 0;
	 } // if
      } // if

      this.Normalize ();

      //BCDFloat.Logger.exiting (BCDFloat.TAG, "Round_To", this);
      return;
   } // Round_To

   /**
    * convert to native/debug string representation
    *
    * @return (Coefficient, Exponent)
    */
   @NotNull
   @Override
   public String To_Debug_String ()
   {
      final StringBuilder Retval = new StringBuilder (100);

      Retval.append ('(');

      if (this.Type == BCDFloat.Float_Type.Finite)
      {
	 Retval.append (this.Coefficient.toString ());
	 Retval.append (',');
	 Retval.append (this.Exponent);
      }
      else
      {
	 Retval.append (this.Type.Image);
      }

      Retval.append (')');

      return Retval.toString ();
   } // To_Debug_String

   /**
    * Convert to integer
    * <p>
    * Do not mix up with Integer - This version returns a Java int!
    * </p>
    *
    * @return value as java int
    */
   @Override
   public int To_Integer ()
   {
      return (int) this.To_Long ();
   } // To_Integer

   /**
    * <p>
    * Convert to long integer
    * </p>
    * <p>
    * Do not mix up with Integer - This version returns a Java int!
    * </p>
    *
    * @return value as java int
    */
   @Override
   public long To_Long ()
   {
      final BCDInteger Retval;

      if (this.Exponent > 0)
      {
	 Retval = this.Coefficient.Shift_Right (this.Exponent);
      }
      else if (this.Exponent < 0)
      {
	 if (this.Coefficient.Num_Digits () > (-this.Exponent))
	 {
	    Retval = this.Coefficient.Shift_Left (-this.Exponent);
	 }
	 else
	 {
	    Retval = new BCDInteger (0);
	 } // if
      }
      else
      {
	 Retval = new BCDInteger (this.Coefficient);
      } // if
      return Retval.To_Long ();
   } // To_Long

   /**
    * |x|
    *
    * @return abs value
    */
   @Contract (pure = true)
   @NotNull
   @Override
   public final Number abs ()
   {
      final BCDFloat Retval = new BCDFloat (this);

      Retval.Coefficient = Retval.Coefficient.abs ().As_BCDInteger ();

      return Retval;
   } // abs

   /**
    * <p>
    * Logical and
    * </p>
    *
    * @return x ∧ Y
    */
   @Contract (pure = true)
   @NotNull
   @Override
   public Number and (@NotNull final Number y)
   {
      return this.As_BCDInteger ().and (y);
   }

   /**
    * arc cosine
    *
    * @return arc cosine
    */
   @Contract (pure = true)
   @NotNull
   @Override
   public final Number arc_cos ()
   {
      // BCDFloat.Logger.entering (BCDFloat.TAG, "arc_cos", this.To_Debug_String ());

      Number Retval;

      if (this.Is_Zero ())
      {
	 Retval = BCDFloat.Num_0_5_π;
      }
      else
      {
	 final BCDFloat x = new BCDFloat (this).neg ().As_BCDFloat ();

	 Retval = x.arc_sin ();
	 Retval = Retval.$plus (BCDFloat.Num_0_5_π);
      }

      // BCDFloat.Logger.exiting (BCDFloat.TAG, "arc_cos", Retval.To_Debug_String ());
      return Retval;
   } // arc_cos

   /**
    * arc cosine
    *
    * @param Half_Circle
    *    value for a half circle
    * @return arccos (x) ÷ π × Half_Circle
    */
   @Contract (pure = true)
   @NotNull
   @Override
   public final Number arc_cos (@NotNull final Number Half_Circle)
   {
      // BCDFloat.Logger.entering (BCDFloat.TAG, "arc_cos", Half_Circle);

      final Number Retval = this.arc_cos ().$div (BCDFloat.Num_π).$times (Half_Circle);

      // BCDFloat.Logger.exiting (BCDFloat.TAG, "arc_cos", Retval);
      return Retval;
   } // arc_cos

   /**
    * area hyperbolic cosine: log<sub>e</sub> (x + √(x-1) × √(x+1))
    *
    * @return Hyperbolic sine.
    */
   @Contract (pure = true)
   @NotNull
   @Override
   public final Number arc_cos_hyp ()
   {
      final Number x_m_1 = this.$minus (BCDFloat.Num_1);
      final Number x_p_1 = this.$plus (BCDFloat.Num_1);
      final Number root_x_m_1 = x_m_1.square_root ();
      final Number root_x_p_1 = x_p_1.square_root ();
      final Number mul = root_x_m_1.$times (root_x_p_1);
      final Number x_mul = this.$plus (mul);
      final Number Retval = x_mul.log_e ();

      return Retval;
   } // arc_cos_hyp

   /**
    * arc sine
    *
    * @return arc sine
    */
   @Contract (pure = true)
   @NotNull
   @Override
   public final Number arc_sin ()
   {
      // BCDFloat.Logger.entering (BCDFloat.TAG, "arc_sin", To_Debug_String ());

      BCDFloat x = new BCDFloat (this);
      final boolean xNeg;

      if (x.Coefficient.Is_Negative ())
      {
	 x = x.abs ().As_BCDFloat ();
	 xNeg = true;
      }
      else
      {
	 xNeg = false;
      } // if

      final BCDFloat xTest = x.$minus (BCDFloat.Num_1).As_BCDFloat ();
      BCDFloat Retval;

      if (xTest.Is_Zero ())
      {
	 Retval = net.sourceforge.uiq3.math.BCDFloat.Num_0_5_π;
      }
      else
      {
	 if (!xTest.Coefficient.Is_Negative ()) // x > 1
	 {
	    Retval = new BCDFloat (BCDFloat.NaN);
	 }
	 else
	 {
	    Retval = (x.$div (BCDFloat.Num_1.$minus (x.$times (x)).square_root ()).arc_tan ()).As_BCDFloat ();
	 } // if
      } // if

      if (xNeg)
      {
	 Retval = Retval.neg ().As_BCDFloat ();
      } // if

      // BCDFloat.Logger.exiting (BCDFloat.TAG, "arc_sin", Retval);
      return Retval;
   }

   /**
    * arc sine
    *
    * @param Half_Circle
    *    value for a half circle
    * @return arcsin (x) ÷ π × Half_Circle
    */
   @Contract (pure = true)
   @NotNull
   @Override
   public final Number arc_sin (@NotNull final Number Half_Circle)
   {
      // BCDFloat.Logger.entering (BCDFloat.TAG, "BCDFloat.arc_sin (" + To_Debug_String () + ","
      // + Half_Circle.To_Debug_String () + ")");

      final Number Retval = this.arc_sin ().$div (BCDFloat.Num_π).$times (Half_Circle);

      // BCDFloat.Logger.exiting (BCDFloat.TAG, "BCDFloat.arc_sin (" + Retval.To_Debug_String ()
      // + ")");
      return Retval;
   } // arc_sin

   /**
    * area hyperbolic sine: log<sub>e</sub> (x + √(x² + 1))
    *
    * @return Hyperbolic sine.
    */
   @Contract (pure = true)
   @NotNull
   @Override
   public final Number arc_sin_hyp ()
   {
      final Number x_x = this.$times (this);
      final Number x_x_1 = x_x.$plus (BCDFloat.Num_1);
      final Number root = x_x_1.square_root ();
      final Number x_root = this.$plus (root);
      final Number Retval = x_root.log_e ();

      return Retval;
   } // arc_sin_hyp

   /**
    * arc tangent
    *
    * @return arc tangent
    */
   @Contract (pure = true)
   @NotNull
   @Override
   public final Number arc_tan ()
   {
      // BCDFloat.Logger.entering (BCDFloat.TAG, "arc_tan", this);

      BCDFloat x = new BCDFloat (this);
      final boolean xNeg;

      if (x.Coefficient.Is_Negative ())
      {
	 x = x.abs ().As_BCDFloat ();
	 xNeg = true;
      }
      else
      {
	 xNeg = false;
      } // if

      boolean gtOne = false;

      if (!x.$minus (BCDFloat.Num_1).As_BCDFloat ().Coefficient.Is_Negative ()) // x > 1
      {
	 gtOne = true;
	 x = BCDFloat.Num_1.$div (x).As_BCDFloat ();
      } // if

      BCDFloat Retval = x;

      Iterate:
      for (int i = 1; i < Number.Max_Precision; i = i + 1)
      {
	 final Number atTan = Retval.tan ();
	 final BCDFloat atn = (Retval.$minus (atTan.$minus (x).$div (BCDFloat.Num_1.$plus (atTan.$times (atTan)))))
	    .As_BCDFloat ();

	 if (atn.$minus (Retval).Is_Zero () || (atn.Type != BCDFloat.Float_Type.Finite))
	 // stop if increment is zero
	 {
	    break Iterate;
	 } // if

	 Retval = atn;
      } // for

      if (gtOne)
      {
	 Retval = (BCDFloat.Num_0_5_π.$minus (Retval)).As_BCDFloat ();
      }

      if (xNeg)
      {
	 Retval = Retval.neg ().As_BCDFloat ();
      } // if

      // BCDFloat.Logger.exiting (BCDFloat.TAG, "arc_tan", Retval);
      return Retval;
   } // arc_tan

   /**
    * arc tangent
    *
    * @param Half_Circle
    *    value for a half circle
    * @return arctan (x) ÷ π × Half_Circle
    */
   @Contract (pure = true)
   @NotNull
   @Override
   public final Number arc_tan (@NotNull final Number Half_Circle)
   {
      // BCDFloat.Logger.entering (BCDFloat.TAG, "BCDFloat.arc_tan (" + To_Debug_String () + ","
      // + Half_Circle.To_Debug_String () + ")");

      final Number Retval = this.arc_tan ().$div (BCDFloat.Num_π).$times (Half_Circle);

      // BCDFloat.Logger.exiting (BCDFloat.TAG, "BCDFloat.arc_tan (" + Retval.To_Debug_String ()
      // + ")");

      return Retval;
   } // arc_tan

   /**
    * area hyperbolic tangent: log<sub>e</sub> ((1 + x) ÷ (1 - x)) / 2
    *
    * @return Hyperbolic sine.
    */
   @Contract (pure = true)
   @NotNull
   @Override
   public final Number arc_tan_hyp ()
   {
      final Number x_p_1 = BCDFloat.Num_1.$plus (this);
      final Number x_m_1 = BCDFloat.Num_1.$minus (this);
      final Number diff = x_p_1.$div (x_m_1);
      final Number ln_diff = diff.log_e ();
      final Number Retval = ln_diff.$div (BCDFloat.Num_2);

      return Retval;
   } // arc_tan_hyp

   /**
    * clone the float - but it might be easier to use the provided copy Constructor instead.
    *
    * @return a new BCDFloat from an existing one.
    * @see java.lang.Object#clone()
    */
   @SuppressWarnings ("MethodDoesntCallSuperMethod")
   @NotNull
   @Override
   public Object clone ()
      throws
      CloneNotSupportedException
   {
      return new BCDFloat (this);
   } // clone

   /**
    * @param rightValue
    *    value to compare with
    * @return <dl>
    * <dt>Compare_Equal</dt>
    * <dd>Values are the same</dd>
    * <dt>Compare_Less</dt>
    * <dd>Left value less the right value</dd>
    * <dt>Compare_Greater</dt>
    * <dd>Left value greater the right value</dd>
    * </dl>
    * @see java.lang.Comparable#compareTo(java.lang.Object)
    */
   @Override
   public int compareTo (@NotNull final Number rightValue)
   {
      try
      {
	 return this.Compare (rightValue);
      }
      catch (@NotNull final BCDError exception)
      {
	 BCDFloat.Logger.log (java.util.logging.Level.SEVERE, "LOG01040:Comparison failed", exception);
	 throw new java.lang.IllegalStateException ("LOG01040:Comparison failed", exception);
      }
   } // compareTo

   /**
    * cos (x)
    *
    * @return cos (x)
    */
   @Contract (pure = true)
   @NotNull
   @Override
   public final Number cos ()
   {
      // BCDFloat.Logger.entering (BCDFloat.TAG, "cos" + To_Debug_String () + ")");

      final Number Retval = this.$plus (BCDFloat.Num_0_5_π).sin ();

      // BCDFloat.Logger.exiting (BCDFloat.TAG, "cos" + Retval.To_Debug_String ()
      // + ")");
      return Retval;
   } // cos

   /**
    * cosine
    *
    * @param Half_Circle
    *    value for a half circle
    * @return sin (x × π ÷ Half_Circle)
    */
   @Contract (pure = true)
   @NotNull
   @Override
   public final Number cos (@NotNull final Number Half_Circle)
   {
      // BCDFloat.Logger.entering (BCDFloat.TAG, "BCDFloat.cos (" + To_Debug_String () + ","
      // + Half_Circle.To_Debug_String () + ")");

      final Number Retval = this.$times (BCDFloat.Num_π).$div (Half_Circle).cos ();

      // BCDFloat.Logger.exiting (BCDFloat.TAG, "BCDFloat.cos (" + Retval.To_Debug_String ()
      // + ")");
      return Retval;
   } // cos

   /**
    * Hyperbolic cosine: (Exponent^x + Exponent^-x)/2
    *
    * @return Hyperbolic cosine.
    */
   @Contract (pure = true)
   @NotNull
   @Override
   public final Number cos_hyp ()
   {
      final Number e_x = this.exp_e ();
      final Number e_m_x = BCDFloat.Num_0.$minus (this).exp_e ();
      final Number sub = e_x.$plus (e_m_x);
      final Number Retval = sub.$div (BCDFloat.Num_2);

      return Retval;
   } // cos_hyp

   /**
    * @param rightValue
    *    BCDFloat to compare with
    * @return true when the values are equal
    */
   @Override
   public boolean equals (@NotNull final Object rightValue)
   {
      final BCDFloat Right;

      if (rightValue instanceof String)
      {
	 Right = new BCDFloat ((String) rightValue);
      }
      else if (rightValue instanceof Number)
      {
	 Right = ((Number) rightValue).As_BCDFloat ();
      }
      else
      {
	 throw new java.lang.IllegalArgumentException ("Compare with BCDFloat, BCDInteger " + "and String only.");
      } // if

      try
      {
	 return this.Compare (Right) == Number.Compare_Equal;
      }
      catch (@NotNull final BCDError exception)
      {
	 BCDFloat.Logger.log (java.util.logging.Level.SEVERE, "LOG01050:", exception);
	 throw new java.lang.IllegalStateException ("LOG01050:", exception);
      }
   } // equals

   /**
    * Exponent^x using series expansion, usable around 0
    *
    * @return e<sup>x</sup>
    */
   @NotNull
   private BCDFloat expI ()
   {
      Number a = new BCDFloat (this);
      Number b = BCDFloat.Num_1;
      Number Retval = b.$plus (this); // 1+x

      // Taylor Series Expansion
      for (int i = 2; i < (10 * Number.Max_Precision); i = i + 1) // max. number of iterations
      {
	 a = a.$times (this);
	 b = b.$times (new BCDFloat (i));
	 final Number rn = Retval.$plus (a.$div (b));

	 if (rn.$minus (Retval).Is_Zero () || (!rn.Is_Finite ()))
	 // stop if increment is zero
	 {
	    Retval = rn;
	    break;
	 } // if

	 Retval = rn;
      } // for

      return Retval.As_BCDFloat ();
   }

   /**
    * 10<sup>x</sup>
    *
    * @return 10<sup>x</sup>
    * @throws BCDError
    *    calculation error
    */
   @Contract (pure = true)
   @NotNull
   @Override
   public final Number exp_10 ()
      throws
      BCDError
   {
      return BCDFloat.Num_10.$times$times (this);
   } // exp_10

   /**
    * e<sup>x</sup>
    *
    * @return e<sup>x</sup>
    */
   @Contract (pure = true)
   @NotNull
   @Override
   public final Number exp_e ()
   {
      // e^x based on expI using transformation
      // to interval [0, ^ln 10] using:
      // x - k * ln 10 = r
      // 10^k * Exponent^r = Exponent^x

      final BCDFloat ln10 = BCDFloat.Num_log_e_10;
      final long k = this.$div (ln10).To_Long ();
      BCDFloat Retval;

      if (k == 0)
      {
	 Retval = this.expI ();
      }
      else
      {
	 Retval = (this.$minus ((new BCDFloat (k)).$times (ln10))).As_BCDFloat ();
	 Retval = Retval.expI ();
	 Retval.Exponent += k;
      } // if

      return Retval;
   }

   /**
    * @return a has code
    */
   @Override
   public int hashCode ()
   {
      int Retval = 5;
      Retval = (53 * Retval) + (this.Coefficient.hashCode ());
      Retval = (53 * Retval) + this.Exponent;
      return Retval;
   } // hashCode

   /**
    * log<sub>e</sub> x using series expansion, usable around 1
    *
    * @return log<sub>e</sub>
    */
   @NotNull
   private Number lnI ()
   {
      final Number d = this.$minus (BCDFloat.Num_1); // 1-x
      Number a = d.Clone ();
      Number Retval = d.Clone ();

      // Taylor Series Expansion
      for (int i = 2; i < (10 * Number.Max_Precision); i = i + 1) // max. number of iterations
      {
	 a = a.$times (d);
	 final net.sourceforge.uiq3.math.Number b = new net.sourceforge.uiq3.math.BCDFloat (i);
	 final Number rn;
	 if ((i % 2) == 0)
	 {
	    rn = Retval.$minus (a.$div (b));
	 }
	 else
	 {
	    rn = Retval.$plus (a.$div (b));
	 } // if

	 if (rn.$minus (Retval).Is_Zero () || (!rn.Is_Finite ()))
	 // stop if increment is zero
	 {
	    Retval = rn;
	    break;
	 } // if

	 Retval = rn;
      } // for

      return Retval;
   } // lnI

   /**
    * log<sub>10</sub> (x)
    *
    * @return log<sub>10</sub> (x)
    */
   @Contract (pure = true)
   @NotNull
   @Override
   public final Number log_10 ()
   {
      return (this.log_e ().$div (BCDFloat.Num_log_e_10)).Clone ();
   } // log_10

   /**
    * log<sub>e</sub> (x)
    *
    * @return log<sub>e</sub> (x)
    */
   @Contract (pure = true)
   @NotNull
   @Override
   public final Number log_e ()
   {
      BCDFloat Retval = new BCDFloat ();

      if (this.Is_Zero ())
      {
	 Retval.Type = BCDFloat.Float_Type.Negative_Infinity;
      } // if
      else if (this.Coefficient.Is_Negative ())
      {
	 Retval.Type = BCDFloat.Float_Type.NaN;
      } // if
      else
      {
	 BCDFloat x = new BCDFloat (this);

	 boolean cInv = false;
	 if (!x.$minus (BCDFloat.Num_1).As_BCDFloat ().Coefficient.Is_Negative ())
	 // x > 1 use x' = 1/x, ln x' = - ln x
	 {
	    cInv = true;
	    x = (BCDFloat.Num_1.$div (x)).As_BCDFloat ();
	 } // if

	 // determine k (min, integer) such that Exponent^k*x = r > 1/Exponent
	 Retval = x;
	 Number ep = BCDFloat.Num_e;

	 // calculate max n (step) required with k = n�
	 int p = 1;
	 // noinspection MethodCallInLoopCondition
	 while (Retval.$minus (BCDFloat.Num_1_div_e).As_BCDFloat ().Coefficient.Is_Negative ())
	 {
	    p *= 2;
	    ep = ep.$times (ep);
	    Retval = x.$times (ep).As_BCDFloat ();
	 } // while

	 // using max step approximate k with bisection
	 // (decreasing number of multiplications for large k)
	 Retval = x;
	 int k = 0;
	 for (p /= 2; p >= 1; p /= 2)
	 {
	    // prepare step
	    ep = BCDFloat.Num_e;
	    int pt = 1;
	    while (pt != p)
	    {
	       pt *= 2;
	       ep = ep.$times (ep);
	    } // while

	    int kp = k;
	    BCDFloat rp = Retval;

	    // add step, rollback on overflow
	    // noinspection MethodCallInLoopCondition
	    while (Retval.$minus (BCDFloat.Num_1_div_e).As_BCDFloat ().Coefficient.Is_Negative ())
	    {
	       kp = k;
	       rp = Retval;

	       Retval = Retval.$times (ep).As_BCDFloat ();
	       k += p;
	    } // while
	    Retval = rp;
	    k = kp;
	 } // for

	 // ln x = ln (r * Exponent^(-k) ) = ln r - k
	 Retval = (Retval.lnI ().$minus (new BCDFloat (k))).As_BCDFloat ();

	 if (cInv) // x > 1
	 {
	    Retval.Coefficient = Retval.Coefficient.neg ().As_BCDInteger ();
	 } // if
      } // if
      return Retval;
   } // log_e

   /**
    * @return negative value
    */
   @Contract (pure = true)
   @NotNull
   @Override
   public final Number neg ()
   {
      final BCDFloat Retval = new BCDFloat (this.As_BCDFloat ());

      Retval.Coefficient = Retval.Coefficient.neg ().As_BCDInteger ();

      return Retval;
   } // neg

   /**
    * <p>
    * Logical not
    * </p>
    *
    * @return ¬ x
    */
   @Contract (pure = true)
   @NotNull
   @Override
   public Number not ()
   {
      return this.As_BCDInteger ().not ();
   }

   /**
    * <p>
    * Logical or
    * </p>
    *
    * @return x ∨ y
    */
   @Contract (pure = true)
   @NotNull
   @Override
   public Number or (@NotNull final Number y)
   {
      return this.As_BCDInteger ().or (y);
   }

   /**
    * <sup>r</sup>√x
    *
    * @param r
    *    root
    * @return e<sup>(log<sub>e</sub> x ÷ r)</sup>
    */
   @Contract (pure = true)
   @NotNull
   @Override
   public final Number root (@NotNull final Number r)
   {
      final Number log = this.log_e ();
      final Number div = log.$div (r);
      final Number Retval = div.exp_e ();

      return Retval;
   } // root

   /**
    * sine
    *
    * @return sin (x)
    */
   @Contract (pure = true)
   @NotNull
   @Override
   public final Number sin ()
   {
      // BCDFloat.Logger.entering (BCDFloat.TAG, "sin", this);

      BCDFloat x = this.Clone ().As_BCDFloat ();
      boolean xNeg = false;

      // sin x = - sin(-x)
      if (x.Coefficient.Is_Negative ())
      {
	 xNeg = true;
	 final net.sourceforge.uiq3.math.Number abs = x.Coefficient ().abs ();
	 x.Coefficient = abs.As_BCDInteger ();
      }

      if (x.compareTo (BCDFloat.Num_2_π) >= 0)
      {
	 final long k = x.$div (BCDFloat.Num_2_π).To_Long ();
	 x = (x.$minus (BCDFloat.Num_2_π.$times (new BCDFloat (k)))).As_BCDFloat ();
      } // if

      if (x.compareTo (BCDFloat.Num_π) > 0)
      {
	 xNeg = !xNeg;
	 x = (x.$minus (BCDFloat.Num_π)).As_BCDFloat ();
      } // if

      BCDFloat Retval = x.sinI ().As_BCDFloat ();

      if (Retval.Get_1_Normalized_Exponent () <= -(BCDFloat.Num_π.Digits ().length - 1))
      {
	 Retval = BCDFloat.Num_0;
      }
      else if (xNeg)
      {
	 Retval = Retval.neg ().As_BCDFloat ();
      } // if

      // BCDFloat.Logger.exiting (BCDFloat.TAG, "sin", Retval);
      return Retval;
   } // sin

   /**
    * sine
    *
    * @param Half_Circle
    *    value for a half circle
    * @return sin (x × π ÷ Half_Circle)
    */
   @Contract (pure = true)
   @NotNull
   @Override
   public final Number sin (@NotNull final Number Half_Circle)
   {
      // BCDFloat.Logger.entering (BCDFloat.TAG, "BCDFloat.sin (" + To_Debug_String () + ","
      // + Half_Circle.To_Debug_String () + ")");

      final Number Retval = this.$times (BCDFloat.Num_π).$div (Half_Circle).sin ();

      // BCDFloat.Logger.exiting (BCDFloat.TAG, "BCDFloat.sin (" + Retval.To_Debug_String ()
      // + ")");
      return Retval;
   } // sin

   /**
    * sin using series expansion, usable around 0 (-π .. π)
    *
    * @return sin (x)
    */
   @NotNull
   private Number sinI ()
   {
      Number x = this;
      final Number xsq = x.$times (x);
      Number f = BCDFloat.Num_1;
      Number Retval = x;

      for (int i = 1; i < (10 * Number.Max_Precision); i = i + 1)
      {
	 x = x.$times (xsq);
	 f = f.$times (new BCDFloat (2 * i)).$times (new BCDFloat ((2 * i) + 1));

	 final Number sn;
	 if ((i % 2) != 0)
	 {
	    sn = Retval.$minus (x.$div (f));
	 }
	 else
	 {
	    sn = Retval.$plus (x.$div (f));
	 }

	 if (sn.$minus (Retval).Is_Zero () || (!sn.Is_Finite ()))
	 // stop if increment is zero
	 {
	    Retval = sn;
	    break;
	 }

	 Retval = sn;
      }

      return Retval;
   } // sinI

   /**
    * hyperbolic sine: (e<sup>x</sup> - e<sup>-x</sup>) ÷ 2
    *
    * @return Hyperbolic sine.
    */
   @Contract (pure = true)
   @NotNull
   @Override
   public final Number sin_hyp ()
   {
      final Number e_x = this.exp_e ();
      final Number e_m_x = BCDFloat.Num_0.$minus (this).exp_e ();
      final Number sub = e_x.$minus (e_m_x);
      final Number Retval = sub.$div (BCDFloat.Num_2);

      return Retval;
   } // sin_hyp

   /**
    * x<sup>2</sup>
    *
    * @return x<sup>2</sup>
    */
   @Contract (pure = true)
   @NotNull
   @Override
   public final Number square ()
   {
      return this.$times (this);
   } // square

   /**
    * <sup>2</sup>√x
    *
    * @return <sup>2</sup>√x
    */
   @Contract (pure = true)
   @NotNull
   @Override
   public final Number square_root ()
   {
      // start value: input value with half of effective exponent
      BCDFloat Retval = new BCDFloat ();

      if (!this.Is_Zero ())
      {
	 if (this.Coefficient.Is_Negative ())
	 {
	    Retval.Type = BCDFloat.Float_Type.NaN;
	 }
	 else
	 {
	    Retval.Coefficient = new BCDInteger (this.Coefficient);
	    Retval.Exponent = (this.Exponent - this.Coefficient.Num_Digits ()) / 2;

	    final BCDFloat oneHalf = BCDFloat.Num_0_5;

	    // max. number of iterations
	    Newton_Iteration:
	    for (int i = 0; i < (10 * Number.Max_Precision); i = i + 1)
	    {
	       final Number rn = Retval.$plus (this.$div (Retval)).$times (oneHalf);

	       // stop if increment is zero
	       if (rn.$minus (Retval).Is_Zero () || (!rn.Is_Finite ()))
	       {
		  Retval = rn.As_BCDFloat ();
		  break Newton_Iteration;
	       } // if
	       Retval = rn.As_BCDFloat ();
	    } // for

	    Retval.Normalize ((this.Exponent + 1) / 2);
	 } // if
      } // if
      return Retval;
   } // square_root

   /**
    * tan x
    *
    * @return tan (x × π ÷ Half_Circle)
    */
   @Contract (pure = true)
   @NotNull
   @Override
   public final Number tan ()
   {
      // BCDFloat.Logger.entering (BCDFloat.TAG, "BCDFloat.tan (" + To_Debug_String () + ")");

      final Number Sin = this.sin ();
      final Number Cos = this.cos ();
      final Number Retval = Sin.$div (Cos);

      // BCDFloat.Logger.exiting (BCDFloat.TAG, "BCDFloat.tan (" + Retval.To_Debug_String ()
      // + ")");

      return Retval;
   } // tan

   /**
    * tangent
    *
    * @param Half_Circle
    *    value for a half circle
    * @return tangent
    */
   @Contract (pure = true)
   @NotNull
   @Override
   public final Number tan (@NotNull final Number Half_Circle)
   {
      // BCDFloat.Logger.entering (BCDFloat.TAG, "BCDFloat.tan (" + To_Debug_String () + ","
      // + Half_Circle.To_Debug_String () + ")");

      final Number Retval = this.$times (BCDFloat.Num_π).$div (Half_Circle).tan ();

      // BCDFloat.Logger.exiting (BCDFloat.TAG, "BCDFloat.tan (" + Retval.To_Debug_String () +
      // ")");

      return Retval;
   } // tan

   /**
    * Hyperbolic tangent: (e<sup>2x</sup> - 1) ÷ (e<sup>2x</sup> + 1)
    *
    * @return Hyperbolic tangent.
    */
   @NotNull
   @Override
   public final Number tan_hyp ()
   {
      final Number x_2 = this.$times (BCDFloat.Num_2);
      final Number exp_x_2 = x_2.exp_e ();
      final Number exp_x_2_m_1 = exp_x_2.$minus (BCDFloat.Num_1);
      final Number exp_x_2_p_1 = exp_x_2.$plus (BCDFloat.Num_1);
      final Number Retval = exp_x_2_m_1.$div (exp_x_2_p_1);

      return Retval;
   } // tan_hyp

   /**
    * convert to scientific string representation
    *
    * @see java.lang.Object#toString()
    */
   @NotNull
   @Override
   public String toString ()
   {
      final StringBuilder Retval = new StringBuilder (100);

      if (this.Type != BCDFloat.Float_Type.Finite)
      {
	 Retval.append (this.Type.Image);
      }
      else
      {
	 StringBuilder s = new StringBuilder (this.Coefficient.toString ());
	 String sign = "";

	 if (this.Coefficient.Is_Negative ())
	 {
	    s = new StringBuilder (s.substring (1));
	    sign = "-";
	 } // if

	 final int adjExp = (this.Exponent + s.length ()) - 1;

	 if ((this.Exponent <= 0) && (adjExp >= (-6)))
	 {
	    // no E-notation
	    if (this.Exponent == 0)
	    {
	       // no decimal point
	       Retval.append (sign);
	       Retval.append (s);
	    }
	    else
	    {
	       // decimal point
	       int l = s.length ();

	       while (l < (-this.Exponent + 1))
	       {
		  s.insert (0, "0");
		  l = l + 1;
	       } // while

	       Retval.append (sign);
	       Retval.append (s.substring (0, l + this.Exponent));
	       Retval.append ('.');
	       Retval.append (s.substring (l + this.Exponent));
	    }
	 }
	 else
	 {
	    // E-notation
	    final int l = s.length ();

	    if (l > 1)
	    {
	       // decimal point
	       s = new StringBuilder (s.substring (0, 1) + "." + s.substring (1));
	    }
	    Retval.append (sign);
	    Retval.append (s);
	    Retval.append ("E");
	    Retval.append (adjExp);
	 } // if
      } // if
      return Retval.toString ();
   } // toString

   /**
    * <p>
    * Logical xor
    * </p>
    *
    * @return x ⊻ y
    */
   @Contract (pure = true)
   @NotNull
   @Override
   public Number xor (@NotNull final Number y)
   {
      return this.As_BCDInteger ().xor (y);
   }

   static
   {
      TAG = BCDFloat.class.getName ();
      Logger = java.util.logging.Logger.getLogger (BCDFloat.TAG);
      Num_π = new BCDFloat ("3.14159265358979324");
      Num_log_e_10 = new BCDFloat ("2.30258509299404567");
      Num_e = new BCDFloat ("2.71828182845904526");
      Num_60 = new BCDFloat (6L, 1);
      Num_3600 = new BCDFloat (36L, 2);
      Num_200 = new BCDFloat (2L, 2);
      Num_2_π = new BCDFloat ("6.28318530717958648");
      Num_2 = new BCDFloat (2L);
      Num_180 = new BCDFloat (18L, 1);
      Num_100 = new BCDFloat (1L, 2);
      Num_1000 = new BCDFloat (1L, 3);
      Num_10 = new BCDFloat (1L, 1);
      Num_1_div_e = new BCDFloat ("0.367879441171442322");
      Num_1 = new BCDFloat (1L);
      Num_Minus_1 = new BCDFloat (-1L);
      Num_0_5_π = new BCDFloat ("1.57079632679489662");
      Num_0_5 = new BCDFloat ("5e-1");
      Num_0 = new BCDFloat (0L);
      NaN = new BCDFloat (BCDFloat.Float_Type.NaN.Image);
   } // static
} // BCDFloat

// vim: set nowrap tabstop=8 shiftwidth=3 softtabstop=3 expandtab textwidth=96 :
// vim: set fileencoding=utf-8 filetype=java foldmethod=syntax spell spelllang=en_gb :
