// @formatter:off
/*
 * ************************************************************ {{{1 **********
 * Copyright © 2005 … 2014  "Martin Krischik" «krischik@users.sourceforge.net» (extensions for FX-602P)
 * Copyright © 2005  Kai G. Schwebke, upn.schwebke.com
 * **************************************************************************
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see «http://www.gnu.org/licenses»..
 * **************************************************************************
 * $Author: krischik $
 * $Revision: 8151 $
 * $Date: 2022-05-01 22:26:51 +0200 (So, 01. Mai 2022) $
 * $Id: BCDInteger.java 8151 2022-05-01 20:26:51Z krischik $
 * $HeadURL: svn+ssh://krischik@svn.code.sf.net/p/uiq3/code/trunk/Java/Calculator/Library/src/main/java/net/sourceforge/uiq3/math/BCDInteger.java $
 * ************************************************************ }}}1 **********
 */
//  @formatter:on

package net.sourceforge.uiq3.math;

import net.sourceforge.uiq3.Required;

import org.jetbrains.annotations.Contract;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

/**
 * Arbitrary Max_Precision BCD Integer Arithmetic
 */
@SuppressWarnings ({
   "AssertionCanBeIf",
   "NonAsciiCharacters"})
public final class BCDInteger
   implements
   Number
{
   private static final String INTEGER_VALUES_CAN_NOT_USE_LOGARITHMIC_FUNCTIONS =
      "Integer Values can not use logarithmic functions";
   private static final String INTEGER_VALUES_CAN_NOT_USE_TRIGONOMETRIC_FUNCTIONS =
      "Integer Values can not use trigonometric functions";
   /**
    * <p>
    * Class logger instance.
    * </p>
    */
   @SuppressWarnings ("unused")
   private static final java.util.logging.Logger Logger;
   /**
    * <p>
    * No digits
    * </p>
    */
   private static final byte[] Null_Digits = new byte[0];
   /**
    * <p>
    * Class logger tag.
    * </p>
    */
   private static final String TAG;
   /**
    * 0
    */
   public static final BCDInteger Num_0 = new BCDInteger (0L);
   /**
    * 1
    */
   public static final BCDInteger Num_1 = new BCDInteger (1L);
   /**
    * 10
    */
   public static final BCDInteger Num_10 = new BCDInteger (10L);
   /**
    * 2
    */
   public static final BCDInteger Num_2 = new BCDInteger (2L);
   /**
    * number: sum(i = 0 ... Num_Digits-1; Digits[i]*10^i)
    */
   private byte[] Digits;
   /**
    * remainder of last long division
    */
   @SuppressWarnings ("FieldNotUsedInToString")
   @Nullable
   private BCDInteger Last_Remainder;
   /**
    * remainder of last short division
    */
   @SuppressWarnings ("FieldNotUsedInToString")
   private byte Last_Remainder_Short;
   /**
    * sign
    */
   private boolean Negative;
   /**
    * number of digits
    */
   private int Num_Digits;

   /**
    * new Number, val = 0, length = 0
    */
   public BCDInteger ()
   {
      super ();
      this.Num_Digits = 0;
      this.Digits = BCDInteger.Null_Digits;
      this.Negative = false;

      return;
   } // BCDInteger

   /**
    * new Number from existing number
    *
    * @param Value
    *    value to copy
    */
   public BCDInteger (@NotNull final BCDInteger Value)
   {
      super ();

      Required.notNull (Value);

      this.Num_Digits = Value.Num_Digits;
      this.Digits = new byte[this.Num_Digits];
      this.Negative = Value.Negative;

      System.arraycopy (
	 /* src     => */Value.Digits,
	 /* srcPos  => */0,
	 /* dest    => */this.Digits,
	 /* destPos => */0,
	 /* length  => */this.Num_Digits);
      return;
   } // BCDInteger

   /**
    * new Number from attributes.
    */
   public BCDInteger (final byte @NotNull [] Digits, final boolean Negative)
   {
      super ();
      this.Num_Digits = Digits.length;
      this.Digits = new byte[this.Num_Digits];
      this.Negative = Negative;

      System.arraycopy (
	 /* src     => */Digits,
	 /* srcPos  => */0,
	 /* dest    => */this.Digits,
	 /* destPos => */0,
	 /* length  => */this.Num_Digits);
      return;
   } // BCDInteger

   /**
    * new Number, val and length from string
    *
    * @param s
    *    string to extract number from
    */
   public BCDInteger (@NotNull final CharSequence s)
   {
      super ();
      final int s_Length = s.length ();
      int Temp_Num_Digits = s_Length;

      this.Negative = s.charAt (0) == '-';

      int i = 0;
      // Numbers formatted for the serial port might have preceding spaces
      while ((i < s_Length) && (
	 (s.charAt (i) == ' ') || (s.charAt (i) == '+') || (s.charAt (i) == '-') || (
	    s.charAt (
	       i) == '0')))
      {
	 i = i + 1;
	 Temp_Num_Digits = Temp_Num_Digits - 1;
      }

      if (Temp_Num_Digits <= 0)
      {
	 this.Num_Digits = 0;
	 this.Digits = BCDInteger.Null_Digits;
      }
      else
      {
	 int j = Temp_Num_Digits - 1;
	 this.Num_Digits = Temp_Num_Digits;
	 this.Digits = new byte[this.Num_Digits];

	 // noinspection MethodCallInLoopCondition
	 while (i < s_Length)
	 {
	    this.Digits[j] = (byte) Character.digit (s.charAt (i), 10);
	    j = j - 1;
	    i = i + 1;
	 } // while
      } // if

      return;
   } // BCDInteger

   /**
    * new Number, val = 0, length = n
    *
    * @param n
    *    digits for new number
    */
   public BCDInteger (final int n)
   {
      super ();
      this.Num_Digits = n;
      this.Digits = new byte[n];
      this.Negative = false;

      return;
   } // BCDInteger

   /**
    * new Number, val = 0, length = n
    *
    * @param value
    *    to init the integer from
    */
   public BCDInteger (final long value)
   {
      super ();

      long Temp_Value;

      if (value < 0)
      {
	 this.Negative = true;
	 Temp_Value = -value;
      }
      else
      {
	 this.Negative = false;
	 Temp_Value = value;
      } // if

      final byte[] Temp_Digits = new byte[net.sourceforge.uiq3.math.Number.Max_Precision];
      int Current_Digit = 0;
      while (Temp_Value != 0)
      {
	 Temp_Digits[Current_Digit] = (byte) (Temp_Value % 10);
	 Temp_Value = Temp_Value / 10;
	 Current_Digit = Current_Digit + 1;
      }

      this.Num_Digits = Current_Digit;
      this.Digits = new byte[this.Num_Digits];

      System.arraycopy (
	 /* src     => */Temp_Digits,
	 /* srcPos  => */0,
	 /* dest    => */this.Digits,
	 /* destPos => */0,
	 /* length  => */this.Num_Digits);
      return;
   } // BCDInteger

   /**
    * signed division
    *
    * @param y
    *    right value
    * @return x ÷ y
    */
   @Contract (pure = true)
   @SuppressWarnings ({
      "ChainOfInstanceofChecks",
      "ChainedMethodCall"})
   @NotNull
   @Override
   public Number $div (@NotNull final Number y)
   {
      final Number Retval;

      if (y instanceof BCDFraction)
      {
	 Retval = this.As_BCDFraction ().$div (y);
      }
      else if (y instanceof BCDFloat)
      {
	 Retval = this.As_BCDFloat ().$div (y);
      }
      else
      {
	 final BCDInteger v = y.As_BCDInteger ();

	 v.Normalize ();

	 BCDInteger Result;

	 if (v.Num_Digits > 1)
	 {
	    Result = this.divUnsigned (v);
	 }
	 else
	 {
	    Result = this.divUnsigned (v.Digits[0]);
	    this.Last_Remainder = new BCDInteger (1);
	    this.Last_Remainder.Digits[0] = this.Last_Remainder_Short;
	 }

	 if (this.Negative != v.Negative)
	 {
	    Result = new BCDInteger (Result.Digits, true);
	 }

	 Retval = Result;
      } // if
      return Retval;
   } // div

   /**
    * signed subtraction
    *
    * @param y
    *    right value
    * @return x - y
    */
   @Contract (pure = true)
   @SuppressWarnings ({
      "ChainOfInstanceofChecks",
      "ChainedMethodCall"})
   @NotNull
   @Override
   public Number $minus (@NotNull final Number y)
   {
      final Number Retval;

      if (y instanceof BCDFraction)
      {
	 Retval = this.As_BCDFraction ().$minus (y);
      }
      else if (y instanceof BCDFloat)
      {
	 Retval = this.As_BCDFloat ().$minus (y);
      }
      else
      {
	 final Number Neg_Y = y.neg ();
	 Retval = this.$plus (Neg_Y);
      } // if

      return Retval;
   }

   /**
    * signed addition
    *
    * @param y
    *    right value
    * @return x + y
    */
   @Contract (pure = true)
   @SuppressWarnings ({
      "ChainOfInstanceofChecks",
      "ChainedMethodCall"})
   @NotNull
   @Override
   public Number $plus (@NotNull final Number y)
   {
      final Number Retval;

      if (y instanceof BCDFraction)
      {
	 Retval = this.As_BCDFraction ().$plus (y);
      }
      else if (y instanceof BCDFloat)
      {
	 Retval = this.As_BCDFloat ().$plus (y);
      }
      else
      {
	 final BCDInteger a = y.As_BCDInteger ();
	 BCDInteger Result;

	 if (this.Negative == a.Negative)
	 {
	    Result = this.addUnsigned (a);
	 }
	 else
	 {
	    final int c = this.Compare_Unsigned (a);

	    if (c > 0)
	    {
	       Result = this.subUnsigned (a);
	    }
	    else
	    {
	       Result = a.subUnsigned (this);
	    } // if

	    if (((c > 0) && (this.Negative)) || ((c < 0) && (a.Negative)))
	    {
	       Result = new BCDInteger (Result.Digits, true);
	    } // if
	 } // if
	 Retval = Result;
      } // if
      return Retval;
   }

   /**
    * signed multiplication
    *
    * @param y
    *    Multipicator
    * @return x × y
    */
   @Contract (pure = true)
   @NotNull
   @SuppressWarnings ({
      "ChainOfInstanceofChecks",
      "ChainedMethodCall"})
   @Override
   public Number $times (@NotNull final Number y)
   {
      final Number Retval;

      if (y instanceof BCDFraction)
      {
	 Retval = this.As_BCDFraction ().$times (y);
      }
      else if (y instanceof BCDFloat)
      {
	 Retval = this.As_BCDFloat ().$times (y);
      }
      else
      {
	 BCDInteger Result = this.mulUnsigned (y.As_BCDInteger ());

	 if (this.Negative != y.Is_Negative ())
	 {
	    Result = new BCDInteger (Result.Digits, true);
	 } // if

	 Retval = Result;
      } // if
      return Retval;
   } // mul

   /**
    * x<sup>y</sup>
    *
    * @param y
    *    right value
    * @return x<sup>y</sup>
    * @throws BCDError
    *    calculation error
    */
   @Contract (pure = true)
   @NotNull
   @Override
   public Number $times$times (@NotNull final Number y)
      throws
      BCDError
   {
      Number Retval;
      final long y_long = y.To_Long ();

      if (y_long < 0)
      {
	 Retval = BCDInteger.Num_0;
      }
      else
      {
	 Retval = this;

	 for (int i = 1; i < y_long; i = i + 1)
	 {
	    Retval = Retval.$times (this);
	 } // for
      } // if
      return Retval;
   }

   /**
    * <p>
    * Either convert to a BCDFloat or return this.
    * </p>
    *
    * @return value as java int
    */
   @NotNull
   @Override
   public BCDFloat As_BCDFloat ()
   {
      return new BCDFloat (this);
   }

   /**
    * <p>
    * Either convert to a or return this.
    * </p>
    *
    * @return value as java int
    */
   @Override
   @NotNull
   public BCDFraction As_BCDFraction ()
   {
      return new BCDFraction (this);
   }

   /**
    * <p>
    * Either convert to a BCDInteger or return this.
    * </p>
    *
    * @return value as java int
    */
   @SuppressWarnings ("ReturnOfThis")
   @NotNull
   @Override
   public BCDInteger As_BCDInteger ()
   {
      return this;
   } // As_BCDInteger

   /**
    * clone the integer - but it might be easier to use the provided copy Constructor instead.
    *
    * @return a new BCDFloat from an existing one.
    * @see Object#clone()
    */
   @NotNull
   @Override
   public Number Clone ()
   {
      return new BCDInteger (this);
   } // Clone

   /**
    * <p>
    * coefficient - while a 18 digits number can also be represented as an long an 18digits multiplication need a
    * 36digits temporary and that is more then a long can do.
    * </p>
    *
    * @return Coefficient
    * @throws BCDError
    *    when it's not a normal number
    */
   @Override
   public long Coefficient_As_Long ()
      throws
      BCDError
   {
      return this.To_Long ();
   } // Coefficient_As_Long

   /**
    * signed compare
    *
    * @param Value
    *    value to compare with
    * @return <dl>
    * <dt>Compare_Equal</dt>
    * <dd>Values are the same</dd>
    * <dt>Compare_Less</dt>
    * <dd>Left value less the right value</dd>
    * <dt>Compare_Greater</dt>
    * <dd>Left value greater the right value</dd>
    * </dl>
    */
   @Override
   public int Compare (@NotNull final Number Value)
   {
      // BCDInteger.Logger.entering (BCDInteger.TAG, "Compare", Value);

      final int Retval;
      final BCDInteger Right = Value.As_BCDInteger ();

      if ((!this.Negative) && (!Right.Negative))
      {
	 Retval = this.Compare_Unsigned (Right);
      }
      else if (this.Negative && Right.Negative)
      {
	 Retval = -this.Compare_Unsigned (Right);
      }
      else if (this.Negative)
      {
	 Retval = Number.Compare_Less;
      }
      else
      {
	 Retval = Number.Compare_Greater;
      } // if

      // BCDInteger.Logger.exiting (BCDInteger.TAG, "Compare", Retval);
      return Retval;
   } // Compare

   /**
    * unsigned (absolute) compare
    *
    * @param Value
    *    value to compare with
    * @return <dl>
    * <dt>Compare_Equal</dt>
    * <dd>Values are the same</dd>
    * <dt>Compare_Less</dt>
    * <dd>Left value less the right value</dd>
    * <dt>Compare_Greater</dt>
    * <dd>Left value greater the right value</dd>
    * </dl>
    */
   int Compare_Unsigned (@NotNull final BCDInteger Value)
   {
      // BCDInteger.Logger.entering (XXXX.TAG, "BCDInteger.Compare_Unsigned (" + Value + ")");
      final int Retval;

      // the compare looks all a bit strange but it is designed to work on
      // non Normalize() values. And you you can't Normalize() - dif and mul will
      // then fall over.
      final int cmpDigits = Math.max (this.Num_Digits, Value.Num_Digits);

      // BCDInteger.Logger.log (java.util.logging.Level.FINE, "> Cmp Digit       = " +
      // cmpDigits);

      Compare_Loop:
      {
	 for (int i = cmpDigits - 1; i >= 0; --i)
	 {
	    final int Left_Digit;

	    if (i < this.Num_Digits)
	    {
	       Left_Digit = this.Digits[i];
	    }
	    else
	    {
	       Left_Digit = 0;
	    } // if

	    final int Right_Digit;
	    if (i < Value.Num_Digits)
	    {
	       Right_Digit = Value.Digits[i];
	    }
	    else
	    {
	       Right_Digit = 0;
	    } // if

	    // BCDInteger.Logger.log (java.util.logging.Level.FINE, "> Left Digit      = " +
	    // Left_Digit);
	    // BCDInteger.Logger.log (java.util.logging.Level.FINE, "> Right Digit     = " +
	    // Right_Digit);

	    if (Left_Digit > Right_Digit)
	    {
	       Retval = Number.Compare_Greater;
	       break Compare_Loop;
	    }
	    else if (Left_Digit < Right_Digit)
	    {
	       Retval = Number.Compare_Less;
	       break Compare_Loop;
	    } // if
	 } // for
	 Retval = Number.Compare_Equal;
      } // Compare_Loop

      // BCDInteger.Logger.exiting (XXXX.TAG, "BCDInteger.Compare_Unsigned (" + Retval + ")");
      return Retval;
   }// Compare_Unsigned

   /**
    * number: sum(i = 0 ... Num_Digits-1; Digits[i]*10^i)
    */
   @Override
   public byte[] Digits ()
   {
      return this.Digits.clone ();
   } // Digits

   /**
    * current exponent - note that the exponent is based on the internal representation where the radix point is right
    * most. For example π is 314159265358979324×10<sup>-17</sup>.
    *
    * @return allways 0
    * @throws BCDError
    *    when the value is not finite
    */
   @Override
   public int Exponent ()
      throws
      BCDError
   {
      return 0;
   }

   /**
    * Round to a given Precision after the decimal point.
    *
    * @param Decimal
    *    decimal digits to keep
    * @param Exponent
    *    max exponent
    * @return rounded value
    */
   @NotNull
   @Override
   public Number Fix (final int Decimal, final int Exponent)
   {
      return new BCDInteger (this);
   }

   /**
    * Recompose a frac double from the given data.
    *
    * @return 0
    */
   @NotNull
   @Override
   public Number Frac ()
   {
      return BCDInteger.Num_0;
   }

   /**
    * <p>
    * Get valuef or indirect addressing.
    * </p>
    *
    * @param Digit_Count
    *    Should be 1 … 2 depending if value is used to access memory or perform a goto.
    * @return int value for indirect addressing.
    */
   @Override
   public int Indirect_Value (final int Digit_Count)
   {
      final int Retval;
      final byte[] Digits = this.Digits ();

      if (Digit_Count == 2)
      {
	 // BCDInteger.Logger.fine ("> Digits      = " + java.util.Arrays.toString (Digits));

	 if (Digits.length >= 2)
	 {
	    Retval = (Digits[Digits.length - 1] * 10) + Digits[Digits.length - 2];
	 }
	 else if (Digits.length >= 1)
	 {
	    Retval = Digits[Digits.length - 1];
	 } // if
	 else
	 {
	    Retval = 0;
	 } // if
      }
      else if (Digit_Count == 1)
      {
	 if (Digits.length > 0)
	 {
	    Retval = Digits[Digits.length - 1];
	 }
	 else
	 {
	    Retval = 0;
	 } // if
      }
      else
      {
	 throw new java.lang.IllegalArgumentException ("Only 1 … 2 are allowed for Digit_Count");
      } // if

      return Retval;
   } // Indirect_Value

   /**
    * integer part of the BCDFloat
    * </p>
    * Do not mix up with To_Integer - This version returns another Number!
    * </p>
    *
    * @return a clone
    */
   @NotNull
   @Override
   public Number Integer ()
   {
      return new BCDInteger (this);
   } // Integer

   /**
    * current value is a normal number
    *
    * @return true as all intergers are finite
    */
   @Override
   public boolean Is_Finite ()
   {
      return true;
   }

   /**
    * current value is Infinity.
    *
    * @return false since all integer are finite
    */
   @Override
   public boolean Is_Infinite ()
   {
      return false;
   }

   /**
    * Current value is integer value.
    *
    * @return true since all integers are – well – integers
    */
   @Override
   public boolean Is_Integer ()
   {
      return true;
   }

   /**
    * current value is not a number
    *
    * @return false since all integers are numbers
    */
   @Override
   public boolean Is_NaN ()
   {
      return false;
   }

   /**
    * Current value us negative
    *
    * @return true when x <= -0
    */
   @Override
   public boolean Is_Negative ()
   {
      return this.Negative;
   }

   /**
    * value is 0
    *
    * @return true when x is 0
    */
   @Override
   public final boolean Is_Zero ()
   {
      final BCDInteger Value = new BCDInteger (this);

      Value.Normalize ();

      return (Value.Num_Digits == 0);
   } // Is_Zero

   @Nullable
   BCDInteger Last_Remainder ()
   {
      return this.Last_Remainder;
   }

   /**
    * Normalize (remove leading zero digits)
    */
   @Override
   public void Normalize ()
   {
      // BCDInteger.Logger.entering (BCDInteger.TAG, "Normalize", this);

      int sh = this.Num_Digits - 1;

      while ((sh >= 0) && (this.Digits[sh] == 0))
      {
	 --sh;
      } // while

      if (sh < (this.Num_Digits - 1))
      {
	 if (sh == (-1))
	 {
	    this.Num_Digits = 0;
	    this.Digits = BCDInteger.Null_Digits;
	 }
	 else
	 {
	    final byte[] nd = new byte[sh + 1];

	    System.arraycopy (this.Digits, 0, nd, 0, sh + 1);

	    this.Digits = nd;
	    this.Num_Digits = sh + 1;
	 } // if
      } // if

      // BCDInteger.Logger.exiting (BCDInteger.TAG, "Normalize", this);
      return;
   } // Normalize

   /**
    * returns number of digits
    *
    * @return Num_Digits
    */
   @Override
   public int Num_Digits ()
   {
      return this.Num_Digits;
   } // Get_Num_Digits

   /**
    * Convert Polar to Rectangle Coordinates
    *
    * @param θ
    *    angle to convert
    * @param Half_Circle
    *    180 for deg, 200 for gra, pi for rad
    * @return x = cos (θ) × r
    */
   @Contract (pure = true)
   @NotNull
   @Override
   public Number P_To_X (
      @NotNull final net.sourceforge.uiq3.math.Number θ,
      @NotNull final net.sourceforge.uiq3.math.Number Half_Circle)
   {
      throw new UnsupportedOperationException (BCDInteger.INTEGER_VALUES_CAN_NOT_USE_TRIGONOMETRIC_FUNCTIONS);
   }

   /**
    * Convert Polar to Rectangle Coordinates
    *
    * @param θ
    *    angle
    * @return x = cos (θ) × r
    */
   @Contract (pure = true)
   @NotNull
   @Override
   public Number P_To_X (@NotNull final Number θ)
   {
      throw new UnsupportedOperationException (BCDInteger.INTEGER_VALUES_CAN_NOT_USE_TRIGONOMETRIC_FUNCTIONS);
   }

   /**
    * Convert Polar to Rectangle Coordinates
    *
    * @param θ
    *    angle
    * @return y = sin (θ) × r
    */
   @Contract (pure = true)
   @NotNull
   @Override
   public Number P_To_Y (@NotNull final Number θ)
   {
      throw new UnsupportedOperationException (BCDInteger.INTEGER_VALUES_CAN_NOT_USE_TRIGONOMETRIC_FUNCTIONS);
   }

   /**
    * Convert Polar to Rectangle Coordinates
    *
    * @param θ
    *    angle
    * @param Half_Circle
    *    180 for deg, 200 for gra, pi for rad
    * @return y = sin (θ) × r
    */
   @Contract (pure = true)
   @NotNull
   @Override
   public Number P_To_Y (@NotNull final Number θ, @NotNull final Number Half_Circle)
   {
      throw new UnsupportedOperationException (BCDInteger.INTEGER_VALUES_CAN_NOT_USE_TRIGONOMETRIC_FUNCTIONS);
   }

   /**
    * Convert Rectangle to Polar Coordinates
    *
    * @param y
    *    y position
    * @return r = √(x²+y²)
    */
   @Contract (pure = true)
   @NotNull
   @Override
   public Number R_To_R (@NotNull final Number y)
   {
      throw new UnsupportedOperationException (BCDInteger.INTEGER_VALUES_CAN_NOT_USE_TRIGONOMETRIC_FUNCTIONS);
   }

   /**
    * Convert Rectangle to Polar Coordinates
    *
    * @param y
    *    y position
    * @return θ = arctan (y÷x)
    */
   @Contract (pure = true)
   @NotNull
   @Override
   public Number R_To_θ (@NotNull final Number y)
   {
      throw new UnsupportedOperationException (BCDInteger.INTEGER_VALUES_CAN_NOT_USE_TRIGONOMETRIC_FUNCTIONS);
   }

   /**
    * Convert Rectangle to Polar Coordinates
    *
    * @param y
    *    y position
    * @param Half_Circle
    *    180 for deg, 200 for gra, pi for rad
    * @return θ = arctan (y÷x)
    */
   @Contract (pure = true)
   @NotNull
   @Override
   public Number R_To_θ (@NotNull final Number y, @NotNull final Number Half_Circle)
   {
      throw new UnsupportedOperationException (BCDInteger.INTEGER_VALUES_CAN_NOT_USE_TRIGONOMETRIC_FUNCTIONS);
   }

   /**
    * Round to effective Precision and Exponent.
    *
    * @return rounded value
    */
   @NotNull
   @Override
   public Number Round ()
   {
      return new BCDInteger (this);
   }

   /**
    * Round to a given Precision and Exponent.
    *
    * @param Precision
    *    Total precison
    * @param Exponent
    *    max exponent
    * @return rounded value
    */
   @NotNull
   @Override
   public Number Round (final int Precision, final int Exponent)
   {
      return new BCDInteger (this);
   }

   /**
    * shift left by n digits (= divide by 10<sup>n</sup>)
    *
    * @param n
    *    digits to shift
    * @return x ÷ 10<sup>n</sup>
    */
   @NotNull
   BCDInteger Shift_Left (final int n)
   {
      final int newDigits = this.Num_Digits - n;
      final BCDInteger Retval = new BCDInteger (newDigits);
      Retval.Negative = this.Negative;

      System.arraycopy (this.Digits, n, Retval.Digits, 0, newDigits);

      return Retval;
   }

   /**
    * shift right by n digits (= multiply by 10<sup>n</sup>)
    *
    * @param n
    *    digits to shift
    * @return x × 10<sup>n</sup>
    */
   @NotNull
   BCDInteger Shift_Right (final int n)
   {
      final int newDigits = this.Num_Digits + n;
      final BCDInteger Retval = new BCDInteger (newDigits);
      Retval.Negative = this.Negative;

      System.arraycopy (this.Digits, 0, Retval.Digits, n, newDigits - n);

      return Retval;
   } // Shift_Right

   /**
    * convert to native/debug string representation
    *
    * @return (Coefficient, Exponent)
    */
   @NotNull
   @Override
   public String To_Debug_String ()
   {
      return this.toString ();
   } // To_Debug_String

   /**
    * <p>
    * Convert to integer
    * </p>
    * <p>
    * Do not mix up with Integer – This version returns a Java int! Actualy it just returns
    * <code>(int)To_Long ()</code>
    * </p>
    *
    * @return value as java int
    */
   @Override
   public int To_Integer ()
   {
      return (int) this.To_Long ();
   } // To_Integer

   /**
    * @return return mantissa as long integer.
    */
   @Override
   public long To_Long ()
   {
      long Retval = 0;

      if (this.Num_Digits > 0)
      {
	 for (int i = this.Num_Digits - 1; i >= 0; --i)
	 {
	    Retval = (Retval * 10) + this.Digits[i];
	 } // for
      } // if
      if (this.Negative)
      {
	 Retval = -Retval;
      } // if

      return Retval;
   } // To_Long

   /**
    * |x|
    *
    * @return abs value
    */
   @Contract (pure = true)
   @NotNull
   @Override
   public Number abs ()
   {
      return new net.sourceforge.uiq3.math.BCDInteger (this.Digits, false);
   } // abs

   /**
    * unsigned (absolute) addition
    *
    * @param a
    *    right value
    * @return abs (x) + abs (y)
    */
   @NotNull
   private BCDInteger addUnsigned (@NotNull final BCDInteger a)
   {
      final int newDigits = Math.max (this.Num_Digits, a.Num_Digits) + 1;
      final BCDInteger newInt = new BCDInteger (newDigits);

      for (int i = 0; i < (newDigits - 1); i = i + 1)
      {
	 if (i < this.Num_Digits)
	 {
	    newInt.Digits[i] += this.Digits[i];
	 } // if
	 if (i < a.Num_Digits)
	 {
	    newInt.Digits[i] += a.Digits[i];
	 } // if
	 if (newInt.Digits[i] >= 10)
	 {
	    newInt.Digits[i] -= 10;
	    newInt.Digits[i + 1] = (byte) (newInt.Digits[i + 1] + 1);
	 } // if
      } // for

      newInt.Negative = this.Negative;

      return newInt;
   } // addUnsigned

   /**
    * unsigned (absolute) addition of one digit
    *
    * @param y
    *    right value
    * @return abs (x) - abs (y)
    */
   @SuppressWarnings ("SameParameterValue")
   @NotNull
   BCDInteger addUnsigned (final int y)
   {
      final int newDigits = this.Num_Digits + 1;
      final BCDInteger newInt = new BCDInteger (newDigits);

      assert (0 <= y) && (y <= 10) : "only one digit is added.";

      newInt.Digits[0] = (byte) y;

      for (int i = 0; i < (newDigits - 1); i = i + 1)
      {
	 if (i < this.Num_Digits)
	 {
	    newInt.Digits[i] += this.Digits[i];
	 } // if
	 if (newInt.Digits[i] >= 10)
	 {
	    newInt.Digits[i] -= 10;
	    newInt.Digits[i + 1] = (byte) (newInt.Digits[i + 1] + 1);
	 } // if
      } // for

      newInt.Negative = this.Negative;

      return newInt;
   } // addUnsigned

   /**
    * <p>
    * Logical and
    * </p>
    *
    * @return x ∧ y
    */
   @Contract (pure = true)
   @NotNull
   @Override
   public Number and (@NotNull final Number y)
   {
      final long Left = this.To_Long ();
      final long Right = y.To_Long ();
      final long Retval = Left & Right;

      return new BCDInteger (Retval);
   } // and

   /**
    * arc cosine
    *
    * @return arc cosine
    */
   @Contract (pure = true)
   @NotNull
   @Override
   public Number arc_cos ()
   {
      throw new UnsupportedOperationException (BCDInteger.INTEGER_VALUES_CAN_NOT_USE_TRIGONOMETRIC_FUNCTIONS);
   }

   /**
    * arc cosine
    *
    * @param Half_Circle
    *    value for a half circle
    * @return arccos (x) ÷ π × Half_Circle
    */
   @Contract (pure = true)
   @NotNull
   @Override
   public Number arc_cos (@NotNull final Number Half_Circle)
   {
      throw new UnsupportedOperationException (BCDInteger.INTEGER_VALUES_CAN_NOT_USE_TRIGONOMETRIC_FUNCTIONS);
   }

   /**
    * area hyperbolic cosine: log<sub>e</sub> (x + √(x-1) × √(x+1))
    *
    * @return Hyperbolic sine.
    */
   @Contract (pure = true)
   @NotNull
   @Override
   public Number arc_cos_hyp ()
   {
      throw new UnsupportedOperationException (BCDInteger.INTEGER_VALUES_CAN_NOT_USE_TRIGONOMETRIC_FUNCTIONS);
   }

   /**
    * arc sine
    *
    * @return arc sine
    */
   @Contract (pure = true)
   @NotNull
   @Override
   public Number arc_sin ()
   {
      throw new UnsupportedOperationException (BCDInteger.INTEGER_VALUES_CAN_NOT_USE_TRIGONOMETRIC_FUNCTIONS);
   }

   /**
    * arc sine
    *
    * @param Half_Circle
    *    value for a half circle
    * @return arcsin (x) ÷ π × Half_Circle
    */
   @Contract (pure = true)
   @NotNull
   @Override
   public Number arc_sin (@NotNull final Number Half_Circle)
   {
      throw new UnsupportedOperationException (BCDInteger.INTEGER_VALUES_CAN_NOT_USE_TRIGONOMETRIC_FUNCTIONS);
   }

   /**
    * area hyperbolic sine: log<sub>e</sub> (x + √(x² + 1))
    *
    * @return Hyperbolic sine.
    */
   @Contract (pure = true)
   @NotNull
   @Override
   public Number arc_sin_hyp ()
   {
      throw new UnsupportedOperationException (BCDInteger.INTEGER_VALUES_CAN_NOT_USE_TRIGONOMETRIC_FUNCTIONS);
   }

   /**
    * arc tangent
    *
    * @return arc tangent
    */
   @Contract (pure = true)
   @NotNull
   @Override
   public Number arc_tan ()
   {
      throw new UnsupportedOperationException (BCDInteger.INTEGER_VALUES_CAN_NOT_USE_TRIGONOMETRIC_FUNCTIONS);
   }

   /**
    * arc tangent
    *
    * @param Half_Circle
    *    value for a half circle
    * @return arctan (x) ÷ π × Half_Circle
    */
   @Contract (pure = true)
   @NotNull
   @Override
   public Number arc_tan (@NotNull final Number Half_Circle)
   {
      throw new UnsupportedOperationException (BCDInteger.INTEGER_VALUES_CAN_NOT_USE_TRIGONOMETRIC_FUNCTIONS);
   }

   /**
    * area hyperbolic tangent: log<sub>e</sub> ((1 + x) ÷ (1 - x)) / 2
    *
    * @return Hyperbolic sine.
    */
   @Contract (pure = true)
   @NotNull
   @Override
   public Number arc_tan_hyp ()
   {
      throw new UnsupportedOperationException (BCDInteger.INTEGER_VALUES_CAN_NOT_USE_TRIGONOMETRIC_FUNCTIONS);
   }

   /**
    * clone the float - but it might be easier to use the provided copy Constructor instead.
    *
    * @return a new BCDInteger from an existing one.
    * @see java.lang.Object#clone()
    */
   @SuppressWarnings ("MethodDoesntCallSuperMethod")
   @NotNull
   @Override
   public Object clone ()
      throws
      CloneNotSupportedException
   {
      return new BCDInteger (this);
   } // clone

   /**
    * @param rightValue
    *    value to compare with
    * @return <dl>
    * <dt>Compare_Equal</dt>
    * <dd>Values are the same</dd>
    * <dt>Compare_Less</dt>
    * <dd>Left value less the right value</dd>
    * <dt>Compare_Greater</dt>
    * <dd>Left value greater the right value</dd>
    * </dl>
    * @see java.lang.Comparable#compareTo(java.lang.Object)
    */
   @Override
   public int compareTo (@NotNull final Number rightValue)
   {
      if (!(rightValue instanceof BCDInteger))
      {
	 throw new java.lang.IllegalArgumentException ("Compare with other BCDIntegers only.");
      }

      return this.Compare (rightValue);
   } // compareTo

   /**
    * cos (x)
    *
    * @return cos (x)
    */
   @Contract (pure = true)
   @NotNull
   @Override
   public Number cos ()
   {
      throw new UnsupportedOperationException (BCDInteger.INTEGER_VALUES_CAN_NOT_USE_TRIGONOMETRIC_FUNCTIONS);
   }

   /**
    * cosine
    *
    * @param Half_Circle
    *    value for a half circle
    * @return sin (x × π ÷ Half_Circle)
    */
   @Contract (pure = true)
   @NotNull
   @Override
   public Number cos (@NotNull final Number Half_Circle)
   {
      throw new UnsupportedOperationException (BCDInteger.INTEGER_VALUES_CAN_NOT_USE_TRIGONOMETRIC_FUNCTIONS);
   }

   /**
    * Hyperbolic cosine: (Exponent^x + Exponent^-x)/2
    *
    * @return Hyperbolic cosine.
    */
   @Contract (pure = true)
   @NotNull
   @Override
   public Number cos_hyp ()
   {
      throw new UnsupportedOperationException (BCDInteger.INTEGER_VALUES_CAN_NOT_USE_TRIGONOMETRIC_FUNCTIONS);
   }

   /**
    * <code>
    * predicate: Num_Digits > v.numDigits, v = v.Normalize(), v.numDigits > 1
    * </code>
    *
    * @param Right
    *    right value
    * @return abs(x) ÷ abs(y)
    */
   @NotNull
   private BCDInteger divUnsigned (@NotNull final BCDInteger Right)
   {
      BCDInteger y = new BCDInteger (Right);
      final int n = y.Num_Digits;
      final int m = this.Num_Digits - n;

      final BCDInteger q = new BCDInteger (m + 1);

      final byte d = (byte) (10 / (y.Digits[n - 1] + 1));
      final BCDInteger u = this.mulUnsigned (d);
      y = y.mulUnsigned (d);

      for (int j = m; j >= 0; --j)
      {
	 byte qc = (byte) (((u.Digits[j + n] * 10) + u.Digits[(j + n) - 1]) / y.Digits[n - 1]);
	 byte rc = (byte) (((u.Digits[j + n] * 10) + u.Digits[(j + n) - 1]) % y.Digits[n - 1]);
	 if ((qc == 10) || ((qc * y.Digits[n - 2]) > ((10 * rc) + u.Digits[(j + n) - 2])))
	 {
	    --qc;
	    rc += y.Digits[n - 1];

	    if (rc < 10)
	    {
	       if ((qc == 10) || ((qc * y.Digits[n - 2]) > ((10 * rc) + u.Digits[(j + n) - 2])))
	       {
		  --qc;
		  // noinspection UnusedAssignment
		  rc += y.Digits[n - 1];
	       }
	    }
	 }

	 Number ut = u.mid (j, n + 1);
	 ut = ut.$minus (y.mulUnsigned (qc));

	 q.Digits[j] = qc;
	 if (ut.Is_Negative ())
	 {
	    --q.Digits[j];
	    ut = ut.$plus (y);
	 }

	 for (int k = 0; k <= n; k = k + 1)
	 {
	    u.Digits[k + j] = ut.Digits ()[k];
	 }
      }

      this.Last_Remainder = u.divUnsigned (d);

      return q;
   }

   /**
    * <code>
    * predicate: Num_Digits > v.numDigits, v = v.Normalize(), v.numDigits > 1
    * </code>
    *
    * @param y
    *    right value
    * @return abs(x) ÷ abs(y)
    */
   @NotNull
   private BCDInteger divUnsigned (final int y)
   {
      final BCDInteger u = this.mulUnsigned (1);
      final BCDInteger q = new BCDInteger (this.Num_Digits);

      for (int i = this.Num_Digits - 1; i >= 0; --i)
      {
	 final int ut = (u.Digits[i + 1] * 10) + u.Digits[i];
	 q.Digits[i] = (byte) (ut / y);
	 u.Digits[i + 1] -= (byte) ((q.Digits[i] * y) / 10);
	 u.Digits[i] -= (byte) ((q.Digits[i] * y) % 10);
	 if (u.Digits[i] < 0)
	 {
	    u.Digits[i] += 10;
	    --u.Digits[i + 1];
	 } // if
      } // for

      this.Last_Remainder_Short = u.Digits[0];

      return q;
   } // divUnsigned

   /**
    * @param rightValue
    *    right value
    * @return true when the values are equal
    */
   @Override
   public boolean equals (@NotNull final Object rightValue)
   {
      final BCDInteger Right;

      if (rightValue instanceof CharSequence)
      {
	 Right = new BCDInteger ((CharSequence) rightValue);
      }
      else if (rightValue instanceof BCDInteger)
      {
	 Right = ((BCDInteger) rightValue);
      }
      else
      {
	 throw new java.lang.IllegalArgumentException ("Compare with other BCDInteger and String only.");
      } // if

      return this.Compare (Right) == Number.Compare_Equal;
   } // equals

   /**
    * 10<sup>x</sup>
    *
    * @return 10<sup>x</sup>
    * @throws BCDError
    *    calculation error
    */
   @Contract (pure = true)
   @NotNull
   @Override
   public Number exp_10 ()
      throws
      BCDError
   {
      return BCDInteger.Num_10.$times$times (this);
   }

   /**
    * e<sup>x</sup>
    *
    * @return e<sup>x</sup>
    */
   @Contract (pure = true)
   @NotNull
   @Override
   public Number exp_e ()
   {
      throw new UnsupportedOperationException (BCDInteger.INTEGER_VALUES_CAN_NOT_USE_LOGARITHMIC_FUNCTIONS);
   } // exp_e

   /**
    * <>calculate the greatest common denominator using the Euclidean algorithm</>
    *
    * @param y
    *    second value
    * @return gcd
    */
   @NotNull
   @SuppressWarnings ({
      "ReturnOfThis",
      "MethodWithMultipleReturnPoints"})
   public BCDInteger gcd (@NotNull final BCDInteger y)
   {
      // BCDInteger.Logger.log (java.util.logging.Level.FINER, "gcd ({0},{1})", new Object [] {
      // this, y });

      if (y.Is_Zero ())
      {
	 return this;
      }
      else
      {
	 // noinspection ResultOfMethodCallIgnored
	 this.$div (y);

	 assert this.Last_Remainder != null : "Last_Remainder is set after division";

	 this.Last_Remainder.Normalize ();

	 return y.gcd (this.Last_Remainder);
      } // if
   } // gcd

   @Override
   public int hashCode ()
   {
      int hash = 7;

      for (final byte digit : this.Digits)
      {
	 hash = (11 * hash) + digit;
      } // for

      hash = (11 * hash);

      if (this.Negative)
      {
	 hash = hash + 1;
      }
      return hash;
   }

   /**
    * log<sub>10</sub> (x)
    *
    * @return log<sub>10</sub> (x)
    */
   @Contract (pure = true)
   @NotNull
   @Override
   public Number log_10 ()
   {
      throw new UnsupportedOperationException (BCDInteger.INTEGER_VALUES_CAN_NOT_USE_LOGARITHMIC_FUNCTIONS);
   }

   /**
    * log<sub>e</sub> (x)
    *
    * @return log<sub>e</sub> (x)
    */
   @Contract (pure = true)
   @NotNull
   @Override
   public Number log_e ()
   {
      throw new UnsupportedOperationException (BCDInteger.INTEGER_VALUES_CAN_NOT_USE_LOGARITHMIC_FUNCTIONS);
   }

   /**
    * get part of number
    *
    * @param start
    *    extract from
    * @param len
    *    extract length
    * @return digits from start to start + len
    */
   @NotNull
   private BCDInteger mid (final int start, final int len)
   {
      final BCDInteger Retval = new BCDInteger (len);

      System.arraycopy (this.Digits, start, Retval.Digits, 0, (start + len) - start);

      return Retval;
   } // mid

   /**
    * unsigned (absolute) multiplication
    *
    * @param y
    *    right value
    * @return abs (x) × abs (y)
    */
   @NotNull
   private BCDInteger mulUnsigned (@NotNull final BCDInteger y)
   {
      final int newDigits = this.Num_Digits + y.Num_Digits;
      BCDInteger newInt = new BCDInteger (newDigits);

      for (int i = 0; i < y.Num_Digits; i = i + 1)
      {
	 final BCDInteger Integer = this.Shift_Right (i);

	 newInt = newInt.addUnsigned (Integer.mulUnsigned (y.Digits[i]));
      } // for

      return newInt;
   } // mulUnsigned

   /**
    * unsigned (absolute) multiplication by one digit
    *
    * @param y
    *    right value
    * @return abs (x) × abs (y)
    */
   @NotNull
   private BCDInteger mulUnsigned (final int y)
   {
      final int newDigits = this.Num_Digits + 1;
      final BCDInteger Retval = new BCDInteger (newDigits);

      for (int i = 0; i < this.Num_Digits; i = i + 1)
      {
	 final int t = Retval.Digits[i] + (y * this.Digits[i]);
	 Retval.Digits[i] = (byte) (t % 10);
	 Retval.Digits[i + 1] = (byte) (t / 10);
      } // for

      return Retval;
   } // mulUnsigned

   /**
    * @return negative value
    */
   @Contract (pure = true)
   @NotNull
   @Override
   public Number neg ()
   {
      final BCDInteger Retval = new BCDInteger (this);

      Retval.Negative = !Retval.Negative;

      return Retval;
   } // neg

   /**
    * <p>
    * Logical not
    * </p>
    *
    * @return ¬ x
    */
   @Contract (pure = true)
   @NotNull
   @Override
   public Number not ()
   {
      final long Left = this.To_Long ();
      final long Retval = ~Left;

      return new BCDInteger (Retval);
   } // not

   /**
    * <p>
    * Logical or
    * </p>
    *
    * @return x ∨ y
    */
   @Contract (pure = true)
   @NotNull
   @Override
   public Number or (@NotNull final Number y)
   {
      final long Left = this.To_Long ();
      final long Right = y.To_Long ();
      final long Retval = Left | Right;

      return new BCDInteger (Retval);
   } // or

   /**
    * <sup>r</sup>√x
    *
    * @param r
    *    root
    * @return e<sup>(log<sub>e</sub> x ÷ r)</sup>
    */
   @Contract (pure = true)
   @NotNull
   @Override
   public Number root (@NotNull final Number r)
   {
      throw new UnsupportedOperationException (BCDInteger.INTEGER_VALUES_CAN_NOT_USE_LOGARITHMIC_FUNCTIONS);
   }

   /**
    * sine
    *
    * @return sin (x)
    */
   @Contract (pure = true)
   @NotNull
   @Override
   public Number sin ()
   {
      throw new UnsupportedOperationException (BCDInteger.INTEGER_VALUES_CAN_NOT_USE_TRIGONOMETRIC_FUNCTIONS);
   }

   /**
    * sine
    *
    * @param Half_Circle
    *    value for a half circle
    * @return sin (x × π ÷ Half_Circle)
    */
   @Contract (pure = true)
   @NotNull
   @Override
   public Number sin (@NotNull final Number Half_Circle)
   {
      throw new UnsupportedOperationException (BCDInteger.INTEGER_VALUES_CAN_NOT_USE_TRIGONOMETRIC_FUNCTIONS);
   }

   /**
    * hyperbolic sine: (e<sup>x</sup> - e<sup>-x</sup>) ÷ 2
    *
    * @return Hyperbolic sine.
    */
   @Contract (pure = true)
   @NotNull
   @Override
   public Number sin_hyp ()
   {
      throw new UnsupportedOperationException (BCDInteger.INTEGER_VALUES_CAN_NOT_USE_TRIGONOMETRIC_FUNCTIONS);
   }

   /**
    * x²
    *
    * @return x²
    */
   @Contract (pure = true)
   @NotNull
   @Override
   public Number square ()
   {
      return this.$times (this);
   } // square

   /**
    * ²√x
    *
    * @return ²√x
    */
   @Contract (pure = true)
   @NotNull
   @Override
   public Number square_root ()
   {
      throw new UnsupportedOperationException (BCDInteger.INTEGER_VALUES_CAN_NOT_USE_LOGARITHMIC_FUNCTIONS);
   }

   /**
    * <p>
    * unsigned (absolute) subtraction
    * </p>
    * <code>
    * predicate: this > a (or this < a if Num_Digits == a.numDigits and underflow in most significant digit is allowed)
    * </code>
    *
    * @param y
    *    right value
    * @return abs (x) - abs (y)
    */
   @NotNull
   private BCDInteger subUnsigned (@NotNull final BCDInteger y)
   {
      final int newDigits = this.Num_Digits;
      final BCDInteger newInt = new BCDInteger (newDigits);

      for (int i = 0; i < this.Num_Digits; i = i + 1)
      {
	 newInt.Digits[i] += this.Digits[i];
	 if (i < y.Num_Digits)
	 {
	    newInt.Digits[i] -= y.Digits[i];
	 } // if
	 if ((newInt.Digits[i] < 0) && (i < (this.Num_Digits - 1)))
	 {
	    newInt.Digits[i] = (byte) (newInt.Digits[i] + 10);
	    --newInt.Digits[i + 1];
	 } // if
      } // for

      return newInt;
   } // subUnsigned

   /**
    * tan x
    *
    * @return tan (x × π ÷ Half_Circle)
    */
   @Contract (pure = true)
   @NotNull
   @Override
   public Number tan ()
   {
      throw new UnsupportedOperationException (BCDInteger.INTEGER_VALUES_CAN_NOT_USE_TRIGONOMETRIC_FUNCTIONS);
   }

   /**
    * tangent
    *
    * @param Half_Circle
    *    value for a half circle
    * @return tangent
    */
   @Contract (pure = true)
   @NotNull
   @Override
   public Number tan (@NotNull final Number Half_Circle)
   {
      throw new UnsupportedOperationException (BCDInteger.INTEGER_VALUES_CAN_NOT_USE_TRIGONOMETRIC_FUNCTIONS);
   }

   /**
    * Hyperbolic tangent: (e<sup>2x</sup> - 1) ÷ (e<sup>2x</sup> + 1)
    *
    * @return Hyperbolic tangent.
    */
   @NotNull
   @Override
   public Number tan_hyp ()
   {
      throw new UnsupportedOperationException (BCDInteger.INTEGER_VALUES_CAN_NOT_USE_TRIGONOMETRIC_FUNCTIONS);
   }

   /**
    * return string representation of number
    *
    * @see java.lang.Object#toString()
    */
   @NotNull
   @Override
   public String toString ()
   {
      final StringBuilder Retval = new StringBuilder (100);

      if (this.Negative)
      {
	 Retval.append ('-');
      } // if
      if (this.Num_Digits == 0)
      {
	 Retval.append ('0');
      }
      else
      {
	 boolean leadingZeros = true;

	 for (int i = this.Num_Digits - 1; i >= 0; --i)
	 {
	    if ((this.Digits[i] != 0) || (i == 0))
	    {
	       leadingZeros = false;
	    } // if
	    if (!leadingZeros)
	    {
	       Retval.append (Integer.toString (this.Digits[i], 10));
	    } // if
	 } // for
      } // if

      return Retval.toString ();
   } // toString

   /**
    * <p>
    * Logical xor
    * </p>
    *
    * @return x ⊻ y
    */
   @Contract (pure = true)
   @NotNull
   @Override
   public Number xor (@NotNull final Number y)
   {
      final long Left = this.To_Long ();
      final long Right = y.To_Long ();
      final long Retval = Left ^ Right;

      return new BCDInteger (Retval);
   } // xor

   static
   {
      TAG = BCDInteger.class.getName ();
      Logger = java.util.logging.Logger.getLogger (BCDInteger.TAG);
   } // static
} // BCDInteger

// vim: set nowrap tabstop=8 shiftwidth=3 softtabstop=3 expandtab textwidth=96 :
// vim: set fileencoding=utf-8 filetype=java foldmethod=syntax spell spelllang=en_gb :
