// @formatter:off
/*
 * ************************************************************ {{{1 **********
 * Copyright © 2005 … 2014  "Martin Krischik" «krischik@users.sourceforge.net» (extensions for FX-602P)
 * Copyright © 2005  Kai G. Schwebke, upn.schwebke.com
 * **************************************************************************
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see «http://www.gnu.org/licenses»..
 * **************************************************************************
 * $Author: krischik $
 * $Revision: 6989 $
 * $Date: 2017-11-17 11:58:19 +0100 (Fr, 17. Nov 2017) $
 * $Id: Number.java 6989 2017-11-17 10:58:19Z krischik $
 * $HeadURL: svn+ssh://krischik@svn.code.sf.net/p/uiq3/code/trunk/Java/Calculator/Library/src/main/java/net/sourceforge/uiq3/math/Number.java $
 * ************************************************************ }}}1 **********
 */
//  @formatter:on

package net.sourceforge.uiq3.math;

import org.jetbrains.annotations.Contract;
import org.jetbrains.annotations.NotNull;

/**
 * An arithmetic number.
 */
@SuppressWarnings ("NonAsciiCharacters")
public interface Number
   extends
   Cloneable,
   Comparable<Number>
{
   /**
    * Compare result.
    * <dl>
    * <dt>Compare_Equal</dt>
    * <dd>Values are the same</dd>
    * <dt>Compare_Less</dt>
    * <dd>Left value less the right value</dd>
    * <dt>Compare_Greater</dt>
    * <dd>Left value greater the right value</dd>
    * </dl>
    */
   int Compare_Equal = 0;
   /**
    * Compare result.
    * <dl>
    * <dt>Compare_Equal</dt>
    * <dd>Values are the same</dd>
    * <dt>Compare_Less</dt>
    * <dd>Left value less the right value</dd>
    * <dt>Compare_Greater</dt>
    * <dd>Left value greater the right value</dd>
    * </dl>
    */
   int Compare_Greater = 1;
   /**
    * Compare result.
    * <dl>
    * <dt>Compare_Equal</dt>
    * <dd>Values are the same</dd>
    * <dt>Compare_Less</dt>
    * <dd>Left value less the right value</dd>
    * <dt>Compare_Greater</dt>
    * <dd>Left value greater the right value</dd>
    * </dl>
    */
   int Compare_Less = -1;
   /**
    * Effectivly needed precision. Both calculators only need 12 digits.
    */
   int Effective_Precision = 12;
   /**
    * Max supported precision. Fixed at 18 digits because:
    * <ol>
    * <li>18 digits is more the enough for simulated calculators - most of which have only 12
    * digits precision</li>
    * <li>18 digits mantissa can also be represented as a long.</li>
    * <ol>
    */
   int Max_Precision = 18;

   /**
    * x ÷ y
    *
    * @param y
    *    divisor
    * @return x ÷ y
    */
   @NotNull
   @Contract (pure = true)
   Number $div (@NotNull final Number y);

   /**
    * x ÷ y
    *
    * @param y
    *    divisor
    * @return x ÷ y
    */
   @NotNull
   @Contract (pure = true)
   default Number div (@NotNull final Number y)
   {
      return $div (y);
   }

   /**
    * x - y
    *
    * @param y
    *    value to subtract
    * @return x - y
    */
   @NotNull
   @Contract (pure = true)
   Number $minus (@NotNull final Number y);

   /**
    * x - y
    *
    * @param y
    *    value to subtract
    * @return x - y
    */
   @NotNull
   @Contract (pure = true)
   default Number minus (@NotNull final Number y)
   {
      return $minus (y);
   }

   /**
    * x + y
    *
    * @param y
    *    value to add
    * @return x + y
    */
   @NotNull
   @Contract (pure = true)
   Number $plus (@NotNull final Number y);

   /**
    * x + y
    *
    * @param y
    *    value to add
    * @return x + y
    */
   @NotNull
   @Contract (pure = true)
   default Number plus (@NotNull final Number y)
   {
      return $plus (y);
   }

   /**
    * x × y
    *
    * @param y
    *    Multiplier
    * @return x × y
    */
   @NotNull
   @Contract (pure = true)
   Number $times (@NotNull final Number y);

   /**
    * x × y
    *
    * @param y
    *    Multiplier
    * @return x × y
    */
   @NotNull
   @Contract (pure = true)
   default Number times (@NotNull final Number y)
   {
      return $times (y);
   }

   /**
    * x<sup>y</sup>
    *
    * @param y
    *    power
    * @return x<sup>y</sup>
    * @throws BCDError
    *    calculation error
    */
   @NotNull
   @Contract (pure = true)
   Number $times$times (@NotNull final Number y)
      throws
      BCDError;

   /**
    * x<sup>y</sup>
    *
    * @param y
    *    power
    * @return x<sup>y</sup>
    * @throws BCDError
    *    calculation error
    */
   @NotNull
   @Contract (pure = true)
   default Number pow (@NotNull final Number y)
      throws
      BCDError
   {
      return $times$times (y);
   }

   /**
    * |x|
    *
    * @return abs value
    */
   @NotNull
   @Contract (pure = true)
   Number abs ();

   /**
    * Logical and
    *
    * @return x ∧ y
    */
   @NotNull
   @Contract (pure = true)
   Number and (@NotNull final Number y);

   /**
    * arc cosine
    *
    * @return arc cosine
    */
   @NotNull
   @Contract (pure = true)
   Number arc_cos ();

   /**
    * arc cosine
    *
    * @param Half_Circle
    *    value for a half circle
    * @return arccos (x) ÷ π × Half_Circle
    */
   @NotNull
   @Contract (pure = true)
   Number arc_cos (@NotNull final Number Half_Circle);

   /**
    * area hyperbolic cosine: log<sub>e</sub> (x + √(x-1) × √(x+1))
    *
    * @return Hyperbolic sine.
    */
   @NotNull
   @Contract (pure = true)
   Number arc_cos_hyp ();

   /**
    * arc sine
    *
    * @return arc sine
    */
   @NotNull
   @Contract (pure = true)
   Number arc_sin ();

   /**
    * arc sine
    *
    * @param Half_Circle
    *    value for a half circle
    * @return arcsin (x) ÷ π × Half_Circle
    */
   @NotNull
   @Contract (pure = true)
   Number arc_sin (@NotNull final Number Half_Circle);

   /**
    * area hyperbolic sine: log<sub>e</sub> (x + √(x² + 1))
    *
    * @return Hyperbolic sine.
    */
   @NotNull
   @Contract (pure = true)
   Number arc_sin_hyp ();

   /**
    * arc tangent
    *
    * @return arc tangent
    */
   @NotNull
   @Contract (pure = true)
   Number arc_tan ();

   /**
    * arc tangent
    *
    * @param Half_Circle
    *    value for a half circle
    * @return arctan (x) ÷ π × Half_Circle
    */
   @NotNull
   @Contract (pure = true)
   Number arc_tan (@NotNull final Number Half_Circle);

   /**
    * area hyperbolic tangent: log<sub>e</sub> ((1 + x) ÷ (1 - x)) / 2
    *
    * @return Hyperbolic sine.
    */
   @NotNull
   @Contract (pure = true)
   Number arc_tan_hyp ();

   /**
    * Either convert to a BCDFloat or return this.
    *
    * @return value as java int
    */
   @NotNull
   BCDFloat As_BCDFloat ();

   /**
    * Either convert to a or return this.
    *
    * @return value as java int
    */
   @NotNull
   BCDFraction As_BCDFraction ();

   /**
    * Either convert to a BCDInteger or return this.
    *
    * @return value as java int
    */
   @NotNull
   BCDInteger As_BCDInteger ();

   /**
    * clone the float - but it might be easier to use the provided copy Constructor instead.
    *
    * @return a new BCDFloat from an existing one.
    * @see Object#clone()
    */
   @NotNull
   Number Clone ();

   /**
    * coefficient - while a 18 digits number can also be represented as an long an 18digits multiplication need a
    * 36digits temporary and that is more then a long can do.
    *
    * @return Coefficient
    * @throws BCDError
    *    when it's not a normal number
    */
   long Coefficient_As_Long ()
      throws
      BCDError;

   /**
    * Compare values
    *
    * @param Value
    *    Value to compare with
    * @return <dl>
    * <dt>Compare_Equal</dt>
    * <dd>Values are the same</dd>
    * <dt>Compare_Less</dt>
    * <dd>Left value less the right value</dd>
    * <dt>Compare_Greater</dt>
    * <dd>Left value greater the right value</dd>
    * </dl>
    * @throws BCDError
    *    numbers can not be compared.
    */
   int Compare (@NotNull final Number Value)
      throws
      BCDError;

   /**
    * @param rightValue
    *    value to compare with
    * @return <dl>
    * <dt>Compare_Equal</dt>
    * <dd>Values are the same</dd>
    * <dt>Compare_Less</dt>
    * <dd>Left value less the right value</dd>
    * <dt>Compare_Greater</dt>
    * <dd>Left value greater the right value</dd>
    * </dl>
    * @see Comparable#compareTo(Object)
    */
   @Override
   int compareTo (@NotNull final Number rightValue);

   /**
    * cos (x)
    *
    * @return cos (x)
    */
   @NotNull
   @Contract (pure = true)
   Number cos ();

   /**
    * cosine
    *
    * @param Half_Circle
    *    value for a half circle
    * @return sin (x × π ÷ Half_Circle)
    */
   @NotNull
   @Contract (pure = true)
   Number cos (@NotNull final Number Half_Circle);

   /**
    * Hyperbolic cosine: (Exponent^x + Exponent^-x)/2
    *
    * @return Hyperbolic cosine.
    */
   @NotNull
   @Contract (pure = true)
   Number cos_hyp ();

   /**
    * number: sum(i = 0 ... Num_Digits-1; Digits[i]*10^i)
    */
   byte[] Digits ();

   /**
    * @param rightValue
    *    BCDFloat to compare with
    * @return true when the values are equal
    */
   @Override
   boolean equals (@NotNull final Object rightValue);

   /**
    * 10<sup>x</sup>
    *
    * @return 10<sup>x</sup>
    * @throws BCDError
    *    calculation error
    */
   @NotNull
   @Contract (pure = true)
   Number exp_10 ()
      throws
      BCDError;

   /**
    * e<sup>x</sup>
    *
    * @return e<sup>x</sup>
    */
   @NotNull
   @Contract (pure = true)
   Number exp_e ();

   /**
    * current exponent - note that the exponent is based on the internal representation where the radix point is right
    * most. For example π is 314159265358979324×10<sup>-17</sup>.
    *
    * @return Current exponent.
    * @throws BCDError
    *    when the value is not finite
    */
   int Exponent ()
      throws
      BCDError;

   /**
    * Round to a given Precision after the decimal point.
    *
    * @param Decimal
    *    decimal digits to keep
    * @param Exponent
    *    max exponent
    * @return rounded value
    */
   @NotNull
   Number Fix (final int Decimal, final int Exponent);

   /**
    * Recompose a frac double from the given data.
    *
    * @return a double
    */
   @NotNull
   Number Frac ();

   /**
    * Get valuef or indirect addressing.
    *
    * @param Digit_Count
    *    Should be 1 … 2 depending if value is used to access memory or perform a goto.
    * @return int value for indirect addressing.
    */
   int Indirect_Value (int Digit_Count);

   /**
    * integer part of the BCDFloat
    * <p>
    * Do not mix up with To_Integer - This version returns another BCDFloat!
    *
    * @return integer part of this
    */
   @NotNull
   Number Integer ();

   /**
    * current value is a normal number
    *
    * @return true when it is a normal number
    */
   boolean Is_Finite ();

   /**
    * current value is Infinity (but not N/A).
    *
    * @return true when x is infinite
    */
   @SuppressWarnings ("unused")
   boolean Is_Infinite ();

   /**
    * Current value is integer value.
    *
    * @return true when x is an integer number
    */
   boolean Is_Integer ();

   /**
    * current value is not a number
    *
    * @return when is not a number
    */
   boolean Is_NaN ();

   /**
    * Current value us negative
    *
    * @return true when x <= -0
    */
   boolean Is_Negative ();

   /**
    * value is 0
    *
    * @return true when x is 0
    */
   boolean Is_Zero ();

   /**
    * log<sub>10</sub> (x)
    *
    * @return log<sub>10</sub> (x)
    */
   @NotNull
   @Contract (pure = true)
   Number log_10 ();

   /**
    * log<sub>e</sub> (x)
    *
    * @return log<sub>e</sub> (x)
    */
   @NotNull
   @Contract (pure = true)
   Number log_e ();

   /**
    * @return negative value
    */
   @NotNull
   @Contract (pure = true)
   Number neg ();

   /**
    * normalize the value
    * <p>
    * depending on type: remove leading zeros and / or trailing zeros
    */
   void Normalize ();

   /**
    * Logical not
    *
    * @return ¬ x
    */
   @NotNull
   @Contract (pure = true)
   Number not ();

   /**
    * the current number of digits - not that when the number is not normalised there might be more 0's in the number
    * then strictly needed.
    *
    * @return number of digits
    * @throws BCDError
    *    not a finite number
    */
   int Num_Digits ()
      throws
      BCDError;

   /**
    * Logical or
    *
    * @param y
    *    right value
    * @return x ∨ y
    */
   @NotNull
   @Contract (pure = true)
   Number or (@NotNull final Number y);

   /**
    * Convert Polar to Rectangle Coordinates
    *
    * @param θ
    *    angle
    * @return x = cos (θ) × r
    */
   @NotNull
   @Contract (pure = true)
   Number P_To_X (@NotNull final Number θ);

   /**
    * Convert Polar to Rectangle Coordinates
    *
    * @param θ
    *    angle
    * @param Half_Circle
    *    180 for deg, 200 for gra, pi for rad
    * @return x = cos (θ) × r
    */
   @NotNull
   @Contract (pure = true)
   Number P_To_X (@NotNull final Number θ, @NotNull final Number Half_Circle);

   /**
    * Convert Polar to Rectangle Coordinates
    *
    * @param θ
    *    angle
    * @return y = sin (θ) × r
    */
   @NotNull
   @Contract (pure = true)
   Number P_To_Y (@NotNull final Number θ);

   /**
    * Convert Polar to Rectangle Coordinates
    *
    * @param θ
    *    angle
    * @param Half_Circle
    *    180 for deg, 200 for gra, pi for rad
    * @return y = sin (θ) × r
    */
   @NotNull
   @Contract (pure = true)
   Number P_To_Y (@NotNull final Number θ, @NotNull final Number Half_Circle);

   /**
    * Convert Rectangle to Polar Coordinates
    *
    * @param y
    *    y position
    * @return r = √(x²+y²)
    */
   @NotNull
   @Contract (pure = true)
   Number R_To_R (@NotNull final Number y);

   /**
    * Convert Rectangle to Polar Coordinates
    *
    * @param y
    *    y position
    * @return θ = arctan (y÷x)
    */
   @NotNull
   @Contract (pure = true)
   Number R_To_θ (@NotNull final Number y);

   /**
    * Convert Rectangle to Polar Coordinates
    *
    * @param y
    *    y position
    * @param Half_Circle
    *    180 for deg, 200 for gra, pi for rad
    * @return θ = arctan (y÷x)
    */
   @NotNull
   @Contract (pure = true)
   Number R_To_θ (@NotNull final Number y, @NotNull final Number Half_Circle);

   /**
    * <sup>r</sup>√x
    *
    * @param r
    *    root
    * @return e<sup>(log<sub>e</sub> x ÷ r)</sup>
    */
   @NotNull
   @Contract (pure = true)
   Number root (@NotNull final Number r);

   /**
    * Round to effective Precision and Exponent.
    *
    * @return rounded value
    */
   @NotNull
   @Contract (pure = true)
   Number Round ();

   /**
    * Round to a given Precision and Exponent.
    *
    * @param Precision
    *    Total precison
    * @param Exponent
    *    max exponent
    * @return rounded value
    */
   @NotNull
   @Contract (pure = true)
   Number Round (int Precision, int Exponent);

   /**
    * sine
    *
    * @return sin (x)
    */
   @NotNull
   @Contract (pure = true)
   Number sin ();

   /**
    * sine
    *
    * @param Half_Circle
    *    value for a half circle
    * @return sin (x × π ÷ Half_Circle)
    */
   @NotNull
   @Contract (pure = true)
   Number sin (@NotNull final Number Half_Circle);

   /**
    * hyperbolic sine: (e<sup>x</sup> - e<sup>-x</sup>) ÷ 2
    *
    * @return Hyperbolic sine.
    */
   @NotNull
   @Contract (pure = true)
   Number sin_hyp ();

   /**
    * x<sup>2</sup>
    *
    * @return x<sup>2</sup>
    */
   @NotNull
   @Contract (pure = true)
   Number square ();

   /**
    * <sup>2</sup>√x
    *
    * @return <sup>2</sup>√x
    */
   @NotNull
   @Contract (pure = true)
   Number square_root ();

   /**
    * tan x
    *
    * @return tan (x × π ÷ Half_Circle)
    */
   @NotNull
   @Contract (pure = true)
   Number tan ();

   /**
    * tangent
    *
    * @param Half_Circle
    *    value for a half circle
    * @return tangent
    */
   @NotNull
   @Contract (pure = true)
   Number tan (@NotNull final Number Half_Circle);

   /**
    * Hyperbolic tangent: (e<sup>2x</sup> - 1) ÷ (e<sup>2x</sup> + 1)
    *
    * @return Hyperbolic tangent.
    */
   @NotNull
   @Contract (pure = true)
   Number tan_hyp ();

   /**
    * convert to native/debug string representation
    *
    * @return format depends on type.
    */
   @NotNull
   String To_Debug_String ();

   /**
    * Convert to integer
    * <p>
    * Do not mix up with Integer - This version returns a Java int!
    *
    * @return value as java int
    */
   int To_Integer ();

   /**
    * Convert to long integer
    * <p>
    * Do not mix up with Integer - This version returns a Java int!
    *
    * @return value as java int
    */
   long To_Long ();

   /**
    * display as human readable string. Same format as used by string conversion constructor
    *
    * @return string representation
    * @see java.lang.Object#toString()
    */
   @Override
   @NotNull
   String toString ();

   /**
    * Logical xor
    *
    * @return x ⊻ y
    */
   @NotNull
   @Contract (pure = true)
   Number xor (@NotNull final Number y);
} // Number

// vim: set nowrap tabstop=8 shiftwidth=3 softtabstop=3 expandtab textwidth=96 :
// vim: set fileencoding=utf-8 filetype=java foldmethod=syntax spell spelllang=en_gb :
