/*
 * ************************************************************ {{{1 **********
 * Copyright © 2005 … 2023  "Martin Krischik" «krischik@users.sourceforge.net»
 * **************************************************************************
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see «http://www.gnu.org/licenses»..
 * ************************************************************ }}}1 **********
 */

package net.sourceforge.uiq3.test

/**
 * <p>logging tag</p>
 */
private const val TAG = "net.sourceforge.uiq3.Calculator_Utilities"

/**
 * log instance.
 */
private val logger = java.util.logging.Logger.getLogger(TAG)

/**
 * Press the given GUI buttons with a small pause after each button.
 *
 * @param buttons list of buttons to press.
 */
fun Press_Buttons(buttons: List<net.sourceforge.uiq3.ui.IButton?>) =
   buttons.forEach { button ->
      logger.fine ("Press Button " + button?.buttonText  )

      button?.doClick();
      Thread.sleep(Second(0.01f))
   }

/**
 * Wait until a certain text is shown.
 *
 * @param Form form supplying the keystrokes
 */
fun Wait_For_Display(
   Form: net.sourceforge.uiq3.ui.ICalculatorCanvas,
   Index: Int,
   Expected: String,
   Timeout: Long = Second(10.0f)): Unit
{
   logger.entering(TAG, "Wait_For_Program_Finished", arrayOf(Timeout))

   Async_Assert_True(
      Message = { "“" + Form.getJavaText(Index) + "” was displayed but expected was “" + Expected + "”" },
      Condition = { Form.getJavaText(Index) == Expected },
      Timeout = Timeout,
      Wait = Timeout / 10)

   logger.exiting(TAG, "Wait_For_Program_Finished")
} // Wait_For_Display

/**
 * Wait until a certain text is shown.
 *
 * @param Form form supplying the keystrokes
 */
fun Wait_For_Printout(
   Form: net.sourceforge.uiq3.printer.ICanvas,
   Expected: String,
   Timeout: Long = Second(10.0f)): Unit
{
   logger.entering(TAG, "Wait_For_Program_Finished", arrayOf(Timeout))

   Async_Assert_True(
      Message = { "“" + Form.printout+ "” was printed but expected to contain “" + Expected + "”" },
      Condition = { Form.printout.contains(Expected) },
      Timeout = Timeout,
      Wait = Timeout / 10)

   logger.exiting(TAG, "Wait_For_Program_Finished")
} // Wait_For_Program_Finished


/**
 * Wait until all keystrokes from form have been read — And Then an extra 50ms.
 *
 * @param Calculator calculator running the program
 */
fun Wait_For_Program_Finished(
   Calculator: net.sourceforge.uiq3.calculator.ICalculator,
   Timeout: Long = Second(10.0f)): Unit
{
   logger.entering(TAG, "Wait_For_Program_Finished", arrayOf(Timeout))

   Async_Assert_True(
      Message = { "Program did not finish" },
      Condition = { Calculator.isWaiting },
      Timeout = Timeout,
      Wait = Timeout / 10)

   Thread.sleep(Timeout / 10)

   logger.exiting(TAG, "Wait_For_Program_Finished")
} // Wait_For_Program_Finished

/**
 * Wait until all keystrokes from form have been read — And Then an extra 50ms.
 *
 * @param Form form supplying the keystrokes
 */
fun Wait_For_Execution_Finished(
   Form: net.sourceforge.uiq3.ui.ICalculatorCanvas,
   Timeout: Long = Second(10.0f)): Unit
{
   logger.entering(TAG, "Wait_For_Execution_Finished", arrayOf(Timeout))

   Async_Assert_False(
      Message = { "Keystrokes where not consumed" },
      Condition = { Form.isReadKeyAvailable },
      Timeout = Timeout,
      Wait = Timeout / 10)

   logger.exiting(TAG, "Wait_For_Execution_Finished")
} // Wait_For_Execution_Finished

/**
 * Wait until all keystrokes from form have been read — And Then an extra 50ms.
 *
 * @param Form form supplying the keystrokes
 */
fun Wait_For_Execution_Finished(
   Form: net.sourceforge.uiq3.ui.ICalculatorCanvas,
   Calculator: net.sourceforge.uiq3.calculator.ICalculator,
   Timeout: Long): Unit
{
   logger.entering(TAG, "Wait_For_Execution_Finished", arrayOf(Timeout))

   val Wait = Timeout / 10
   Async_Assert_False(
      Message = { "Keystrokes where not consumed" },
      Condition = { Form.isReadKeyAvailable || !Calculator.isWaiting },
      Timeout = Timeout,
      Wait = Wait)

   Thread.sleep(Wait)

   logger.exiting(TAG, "Wait_For_Execution_Finished")
} // Wait_For_Execution_Finished

/**
 * Wait until a certain text is shown.
 *
 * @param Form form supplying the keystrokes
 */
fun Async_Assert_Display(
   Form: net.sourceforge.uiq3.ui.ICalculatorCanvas,
   Index: Int,
   Expected_Text: String,
   Timeout: Long = Second(10.0f)): Unit
{
   logger.entering(TAG, "Async_Assert_Display", arrayOf(Index, Expected_Text, Timeout))

   Async_Assert_True(
      Message = {
	 "“" + Form.getJavaText(Index) + "” was displayed but expected was “" + Expected_Text + "”"
      },
      Condition = { Form.getJavaText(Index) == Expected_Text },
      Timeout = Timeout,
      Wait = Timeout / 10)

   logger.exiting(TAG, "Async_Assert_Display")
} // Async_Assert_Display

/**
 * wait for the display to show two specific lines
 *
 * @param Expected_Text   expected content of line 0
 * @param Expected_Number expected content of line 1
 */
fun Async_Assert_Display(
   Form: net.sourceforge.uiq3.ui.ICalculatorCanvas,
   Expected_Text: String,
   Expected_Number: String,
   Timeout: Long): Unit
{
   logger.entering(TAG, "Async_Assert_Display", arrayOf(Expected_Text, Expected_Number, Timeout))

   Async_Assert_True(
      Message =
      {
	 val Actual_Text = Form.getJavaText(0)
	 val Actual_Numeric = Form.getJavaText(1)

	 "„" + Actual_Text + "“ and „" + Actual_Numeric + "“ was displayed instead of „" +
	    Expected_Text + "“ and „" + Expected_Number + "“."
      },
      Condition =
      {
	 val Actual_Text = Form.getJavaText(0)
	 val Actual_Numeric = Form.getJavaText(1)

	 logger.log(
	    java.util.logging.Level.FINE,
	    "\n> Actual_Text   = „{1}“" + "\n> Actual_Number  = „{2}“",
	    arrayOf(Actual_Text, Actual_Numeric))

	 (Actual_Text == Expected_Text) && (Actual_Numeric == Expected_Number)
      },
      Timeout = Timeout,
      Wait = Timeout / 10)

   logger.exiting(TAG, "Async_Assert_Display")
} // Async_Assert_Display

/**
 * wait for the display to show two specific lines
 *
 * @param Expected_Text   expected content of line 0
 * @param Expected_Number expected content of line 1
 */
fun Async_Assert_Display(
   Form: net.sourceforge.uiq3.ui.ICalculatorCanvas,
   Expected_Text: String,
   Expected_Number: String,
   Expected_Steps: String,
   Timeout: Long): Unit
{
   logger.entering(TAG, "Async_Assert_Display", arrayOf(Expected_Text, Expected_Number, Timeout))

   Async_Assert_True(
      Message =
      {
	 val Actual_Text = Form.getJavaText(0)
	 val Actual_Numeric = Form.getJavaText(1)
	 val Actual_Steps = Form.getJavaText(2)

	 "„" + Actual_Text + "“, „" + Actual_Numeric + "“ and „" + Actual_Steps + "“ was displayed instead of „" +
	    Expected_Text + "“, „" + Expected_Number + "“ and „" + Expected_Steps + "“."
      },
      Condition =
      {
	 val Actual_Text = Form.getJavaText(0)
	 val Actual_Numeric = Form.getJavaText(1)
	 val Actual_Steps = Form.getJavaText(2)

	 logger.log(
	    java.util.logging.Level.FINE,
	    "\n> Actual_Text   = „{1}“" + "\n> Actual_Number  = „{2}“" + "\n> Actual_Steps  = „{3}“",
	    arrayOf(Actual_Text, Actual_Numeric, Actual_Steps))

	 (Actual_Text == Expected_Text) && (Actual_Numeric == Expected_Number) && (Actual_Steps == Expected_Steps)
      },
      Timeout = Timeout,
      Wait = Timeout / 10)

   logger.exiting(TAG, "Async_Assert_Display")
} // Async_Assert_Display

/**
 * Wait until a certain text is shown.
 *
 * @param Calculator calculator running the program
 */
fun Async_Assert_Printer(
   Printer: net.sourceforge.uiq3.printer.IPrinter,
   Expected_Text: String,
   Timeout: Long = Second(10.0f)): Unit
{
   logger.entering(TAG, "Async_Assert_Printer", arrayOf(Expected_Text, Timeout))

   Async_Assert_True(
      Message = { "Printout “" + Printer.printout + "” did not contain “" + Expected_Text + "”" },
      Condition = { Printer.printout.contains(Expected_Text) },
      Timeout = Timeout,
      Wait = Timeout / 10)

   logger.exiting(TAG, "Async_Assert_Printer")
} // Async_Assert_Printer

// vim: set nowrap tabstop=8 shiftwidth=3 softtabstop=3 noexpandtab textwidth=96 :
// vim: set fileencoding=utf-8 filetype=kotlin foldmethod=syntax spell spelllang=en_gb :
