/*
 * ************************************************************ {{{1 **********
 * Copyright © 2020 … 2020  "Martin Krischik" «krischik@users.sourceforge.net»
 * **************************************************************************
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see «http://www.gnu.org/licenses»..
 * **************************************************************************
 * $Author: krischik $
 * $Revision: 7953 $
 * $Date: 2022-01-19 07:59:28 +0100 (Mi, 19. Jan 2022) $
 * $Id: Utilities.kt 7953 2022-01-19 06:59:28Z krischik $
 * $HeadURL: svn+ssh://krischik@svn.code.sf.net/p/uiq3/code/trunk/Java/Calculator/Test/src/main/java/net.sourceforge.uiq3/test/Utilities.kt $
 * ************************************************************ }}}1 **********
 */

package net.sourceforge.uiq3.test;

import org.junit.jupiter.api.Assertions.fail
import java.util.logging.Level

typealias Statements = () -> Unit

/**
 * <p>logging tag</p>
 */
private val TAG = "net.sourceforge.uiq3.test.Utilities"

/**
 * <p>logger</p>
 */
private val Logger = java.util.logging.Logger.getLogger(TAG)

/**
 * initialize the logger framework. Do not use on Android as you won't have logging.properties there.
 */
public fun Init_Logger(debug: Boolean = false)
{
   Logger.entering(TAG, "Init_Logger")

   java.io.File("target/surefire-reports").mkdirs()

   val manager = java.util.logging.LogManager.getLogManager()
   val properties = {}::class.java.getResourceAsStream("/logging.properties")

   // Read log properties so that the class which we test can write to a log
   // file
   if (properties != null)
   {
      manager.readConfiguration(properties)
   }

   if(debug)
   {
      Logger.log(java.util.logging.Level.ALL, "> Test logging level ALL")
      Logger.log(java.util.logging.Level.CONFIG, "> Test logging level CONFIG")
      Logger.log(java.util.logging.Level.FINE, "> Test logging level FINE")
      Logger.log(java.util.logging.Level.FINER, "> Test logging level FINER")
      Logger.log(java.util.logging.Level.FINEST, "> Test logging level FINEST")
      Logger.log(java.util.logging.Level.INFO, "> Test logging level INFO")
      Logger.log(java.util.logging.Level.OFF, "> Test logging level OFF")
      Logger.log(java.util.logging.Level.SEVERE, "> Test logging level SEVERE")
      Logger.log(java.util.logging.Level.WARNING, "> Test logging level WARNING")
   }
   Logger.exiting(TAG, "Init_Logger")
   return
} // Init_Logger

/**
 * Beginn eines feature blocks.
 * @param Text text der das feature beschreibt
 * @param statements befehle die ausgeführt werden sollen
 * @return nix
 */
public fun Feature(Text: String, statements: Statements)
{
   Logger.log(java.util.logging.Level.INFO, "Feature {0}", Text)
   statements()
}

/**
 * begin eines screnarios inerhalb eines features.
 * @param Text text der das scenatio beschreibt
 * @param statements befehle die ausgeführt werden sollen
 * @return nix
 */
public fun Scenario(Text: String, statements: Statements)
{
   Logger.log(java.util.logging.Level.INFO, "Scenario {0}", Text)
   statements()
}

public fun Feature(Text: String): Unit = Logger.log(java.util.logging.Level.INFO, "Feature {0}", Text)
public fun Scenario(Text: String): Unit = Logger.log(java.util.logging.Level.INFO, "Scenario {0}", Text)
public fun Info(Text: String): Unit = Logger.log(java.util.logging.Level.INFO, "Info: {0}", Text)
public fun Given(Text: String): Unit = Logger.log(java.util.logging.Level.INFO, "Given {0}", Text)
public fun When(Text: String): Unit = Logger.log(java.util.logging.Level.INFO, "When {0}", Text)
public fun Then(Text: String): Unit = Logger.log(java.util.logging.Level.INFO, "Then {0}", Text)
public fun And(Text: String): Unit = Logger.log(java.util.logging.Level.INFO, "And {0}", Text)

/**
 * make sleep command more readable
 *
 * @param Second time in seconds
 * @return time in milli seconds
 */
fun Second(Second: Float): Long = (Second * 1000).toLong()

/**
 * make sleep command more readable
 *
 * @param Minutes time in minutes
 * @return time in milli seconds
 */
fun Minutes(Minutes: Float): Long = (Minutes * 1000 * 60).toLong()

/**
 * checks if the condition becomes true within the given time frame
 *
 * @param Message
 * Message to show when the assertion fails
 * @param Condition
 * Condition to check. Should be pure (free of side effects as it is called mutliple times
 * @param Timeout
 * time after which the test will fail (milli sec)
 * @param Wait
 * Wait time between tests
 */
public tailrec fun Async_Assert_True(
   Message: () -> String = { "Not true after timeout" },
   Condition: () -> Boolean,
   Timeout: Long = Second(10.0f),
   Wait: Long = Second(0.5f)
)
{
   Logger.log(
      Level.FINE,
      "+ Async_Assert_True" +
	 "\n> Message   = {0}" +
	 "\n> Condition = {1}" +
	 "\n> Timeout   = {2}" +
	 "\n> Wait      = {3}",
      arrayOf<Any>(Message, Condition, Timeout, Wait)
   )

   Thread.sleep(Wait)

   if (Condition())
   {
      return
   }
   else if (Timeout < 0)
   {
      fail(Message())
   }
   else
   {
      Async_Assert_True(Message, Condition, Timeout = Timeout - Wait, Wait)
   } // if
} // Async_Assert_True

/**
 *
 * checks if the condition becomes false within the given time frame
 *
 * @param Message
 * Message to show when the assertion fails
 * @param Condition
 * Condition to check. Should be pure (free of side effects as it is called mutliple times
 * @param Timeout
 * time after which the test will fail (milli sec)
 * @param Wait
 * Wait time between tests
 */
public fun Async_Assert_False(
   Message: () -> String = { "Not true after timeout" },
   Condition: () -> Boolean,
   Timeout: Long = Second(10.0f),
   Wait: Long = Second(0.5f)
) = Async_Assert_True(Message, { !Condition() }, Timeout, Wait)

// ********************************************************* {{{1 **********
// vim: set textwidth=120 nowrap tabstop=8 shiftwidth=4 softtabstop=4 expandtab : 
// vim: set filetype=kotlin fileencoding=utf-8 fileformat=unix foldmethod=indent :
// vim: set spell spelllang=en_gb : 
