/*
 * ************************************************************ {{{1 **********
 * Copyright © 2005 … 2024  "Martin Krischik" «krischik@users.sourceforge.net»
 * **************************************************************************
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see «http://www.gnu.org/licenses»..
 * ************************************************************ }}}1 **********
 */

package net.sourceforge.uiq3.hp45;

import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import org.jetbrains.annotations.TestOnly;

/**
 * Main class for an HP45 calculator.
 *
 * @author "Martin Krischik" «krischik@users.sourceforge.net»
 */
public final class Main
   extends net.sourceforge.uiq3.calculator.Main
{
   private static final int Half_A_Second = 500;
   /**
    * class logger instance
    */
   @NotNull
   private static final java.util.logging.Logger logger;
   /**
    * reference to protect instance from garbage collector.
    */
   @Nullable
   private static Main self;
   /**
    * class logger tag.
    */
   @NotNull
   private static final String TAG;
   private static final java.util.concurrent.atomic.AtomicBoolean LAF_Initialized;

   static
   {
      TAG = Main.class.getName ();
      logger = java.util.logging.Logger.getLogger (TAG);
      Main.self = null;
      LAF_Initialized = new java.util.concurrent.atomic.AtomicBoolean (false);
   }

   /**
    * main method.
    *
    * @param args
    *    not used
    */
   public static void main (final String[] args)
   {
      final Package Main_Package = Main.class.getPackage ();

      net.sourceforge.uiq3.calculator.Main.Setup_Logging (
         /* App_Class      => */Main.class,
         /* App_Name       => */Main_Package.getName (),
         /* Home_Directory => */Properties.INSTANCE.getHome_Directory ());

      Main.logger.entering (Main.TAG, "main", args);

      Main.initializeLAF ();
      Main.self = new Main ();
      Main.self.startApp ();

      if (args.length >= 1)
      {
         Main.self.loadFile (new java.io.File (args[0]));
      } // if

      Main.logger.exiting (Main.TAG, "main");
      return;
   } // main

   /**
    * Calculator_Canvas displaying the calculator
    */
   @Nullable
   private Calculator_Canvas canvas;
   /**
    * Main window frame
    */
   @Nullable
   private Calculator_Frame frame;
   /**
    * The calculator engine with debug capability.
    */
   @Nullable
   private Desktop_Calculator calculator;

   /**
    * init main
    */
   Main ()
   {
      super ();
      Main.logger.entering (Main.TAG, "Main");

      this.canvas = null;
      this.frame = null;
      this.calculator = null;

      Main.logger.exiting (Main.TAG, "Main");
      return;
   } // Main

   @TestOnly @Nullable
   Desktop_Calculator
   getCalculator ()
   {
      return calculator;
   }
   
   @TestOnly @Nullable
   Calculator_Canvas
   getCanvas ()
   {
      return canvas;
   }

   @TestOnly @Nullable Calculator_Frame
   getFrame ()
   {
      return frame;
   }

   /**
    * To make widget classes compatible with GUI designer they must be instantiatable from there default constructor.
    * This means that each widget must initialize the LAF in there default constructor.
    */
   public static void initializeLAF ()
   {
      Main.logger.entering (Main.TAG, "initializeLAF");

      if (LAF_Initialized.compareAndSet (false, true))
      {
         final Properties properties = Properties.INSTANCE;

	 Main.registerFonts ();
	 Main.registerLAF ();
         Main.initializeLAF (properties, "RPN-45");

         properties.addPropertyChangeListener (
            event ->
            {
               final String propertyName = event.getPropertyName ();

               if (propertyName.equals (net.sourceforge.uiq3.calculator.IProperties.LOOK_AND_FEEL_NAME_ID) ||
                  propertyName.equals (net.sourceforge.uiq3.calculator.IProperties.LOOK_AND_FEEL_SIZE_ID)
               )
               {
                  Main.initializeLAF (properties, "RPN-45");

                  final java.beans.PropertyChangeSupport propertyChangeSupport = Main.getPropertyChangeSupport ();

                  propertyChangeSupport.firePropertyChange (LOOK_AND_FEEL_CHANGED, null, null);
               }
            });
      } // if

      Main.logger.exiting (Main.TAG, "initializeLAF");
      return;
   } // Initialize_LAF

   @SuppressWarnings ("HardcodedFileSeparator")
   public static void
   registerFonts ()
   {
      logger.entering(TAG, "registerFonts");
      
      Main.Companion.registerFonts ();
      
      final java.awt.GraphicsEnvironment Graphics_Environment =
	 java.awt.GraphicsEnvironment.getLocalGraphicsEnvironment();
      final java.awt.Font font = net.sourceforge.uiq3.calculator.Utils.createFont ("/LCD14.otf");

      if (font != null)
      {
	 Graphics_Environment.registerFont(font);
      }

      logger.exiting(TAG, "registerFonts");
      return;
   } // registerFonts



   /**
    * General info dialog; fed to the OSXAdapter as the method to call when "About OSXAdapter" is selected from the
    * application menu
    *
    * @see net.sourceforge.uiq3.calculator.Main#about()
    */
   @Override
   public void about ()
   {
      Main.logger.entering (Main.TAG, "about");

      if (this.frame != null)
      {
         this.frame.displayAboutBox ();
      }

      Main.logger.exiting (Main.TAG, "about");
      return;
   } // about

   /**
    * @return A new calculator engine
    * @see net.sourceforge.uiq3.calculator.Main#Create_Calculator()
    */
   @Override protected net.sourceforge.uiq3.calculator.ICalculator
   Create_Calculator ()
      throws
      java.io.IOException
   {
      Main.logger.entering (Main.TAG, "Create_Calculator");

      this.calculator = new Desktop_Calculator ();
      this.calculator.Save_Energy (false);
      this.calculator.Read_Listing_File (Calculator.HP45_LST);
      this.calculator.Init_Opcodes ();

      if (this.frame != null)
      {
         this.frame.setCalculator (this.calculator);
         this.frame.restoreStatus ();
         this.frame.repaint ();
      } // if

      if (this.canvas != null)
      {
         this.canvas.repaint ();
         this.canvas.transferFocus ();
      } // if

      Main.logger.exiting (Main.TAG, "Create_Calculator", this.calculator);
      return this.calculator;
   } // Create_Calculator

   /**
    * @return A new Calculator Swing_Canvas.
    * @see net.sourceforge.uiq3.calculator.Main#createCanvas()
    */
   @Override
   protected net.sourceforge.uiq3.ui.ICalculatorCanvas createCanvas ()
      throws
      java.io.IOException
   {
      Main.logger.entering (Main.TAG, "Create_Canvas");

      this.frame = new Calculator_Frame (this);
      this.frame.setName ("Calculator_Frame"); // NOI18N
      this.canvas = this.frame.getCalculatorCanvas ();
      this.frame.setVisible (true);

      Main.logger.exiting (Main.TAG, "Create_Canvas", this.canvas);
      return this.canvas;
   } // Create_Canvas

   /**
    * @return true when paused
    */
   @Override
   public boolean isRunning ()
   {
      return this.calculator != null && this.calculator.isRunning ();
   } // Is_Running

   /**
    * load a file
    *
    * @param Filename
    *    filename to load
    */
   @Override
   public void
   loadFile (final java.io.File Filename)
   {
      Main.logger.entering (Main.TAG, "loadFile", Filename);

      if (this.isRunning ())
      {
         Main.logger.log (java.util.logging.Level.INFO, "LOG00320:Load file : “{0}”", Filename);

         if (this.calculator != null)
         {
            this.calculator.Load (Filename);
         }
      }
      else
      {
         Main.logger.log (
            java.util.logging.Level.WARNING,
            "LOG00330:Calculator not running, file “{0}” not loaded",
            Filename);
      } // if

      Main.logger.exiting (Main.TAG, "loadFile");
      return;
   } // loadFile

   /**
    * Signals the MIDlet to enter the Paused state.
    */
   @Override  public void
   pauseApp ()
   {
      Main.logger.entering (Main.TAG, "pauseApp");

      if (calculator != null)
      {
         calculator.Save ();
      }

      super.pauseApp ();

      while ((this.calculator != null) && !this.calculator.Is_Shutdown ())
      {
         try
         {
            Main.logger.log (java.util.logging.Level.FINEST, "> waiting");

            Thread.sleep (Main.Half_A_Second);
         }
         catch (final InterruptedException Exception)
         {
            Main.logger.log (java.util.logging.Level.FINE, "Thread was interrupted", Exception);
         } // try
      } // if

      Main.logger.exiting (Main.TAG, "pauseApp");
      return;
   } // pauseApp

   /**
    * General preferences dialog; fed to the OSXAdapter as the method to call when "About OSXAdapter" is selected from
    * the application menu
    *
    * @see net.sourceforge.uiq3.calculator.Main#preferences()
    */
   @Override
   public void preferences ()
   {
      Main.logger.entering (Main.TAG, "preferences");

      if (this.frame != null)
      {
         this.frame.displayPreferencesBox ();
      }

      Main.logger.exiting (Main.TAG, "preferences");
      return;
   } // preferences

   /**
    * General quit handler; fed to the OSXAdapter as the method to call when a system quit event occurs A quit event is
    * triggered by Cmd-Q, selecting Quit from the application or Dock menu, or logging out
    *
    * @return true for quit
    */
   @Override
   public boolean quit ()
   {
      Main.logger.entering (Main.TAG, "quit");

      Main.logger.log (java.util.logging.Level.INFO, "LOG03020: Quit Application");

      this.pauseApp ();

      if (this.frame != null)
      {
         this.frame.saveStatus ();
         this.frame.dispose ();
      }

      Properties.INSTANCE.saveProperties ();
      System.exit (0);

      Main.logger.exiting (Main.TAG, "quit");
      return true;
   } // quit

   @SuppressWarnings ("HardcodedLineSeparator")
   @TestOnly @NotNull @Override
   public String
   toString ()
   {
      return "Main" +
         "{\nCanvas=" +
         this.canvas +
         ",\nCalculator_Frame=" +
         this.frame +
         ",\nRPN45=" +
         this.calculator +
         '}';
   }
} // Main

// vim: set nowrap tabstop=8 shiftwidth=3 softtabstop=3 expandtab :
// vim: set textwidth=0 filetype=java foldmethod=syntax nospell :
