/*
 * ************************************************************ {{{1 **********
 * Copyright © 2005 … 2024 "Martin Krischik" «krischik@users.sourceforge.net»
 * Copyright 2000          David G. Hicks    (Java port)
 * Copyright 1995          Eric L. Smith     (Original C/X Windows)
 * **************************************************************************
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see «http://www.gnu.org/licenses»..
 * ************************************************************ }}}1 **********
 */

package net.sourceforge.uiq3.hp45;

import org.jetbrains.annotations.NonNls;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import org.jetbrains.annotations.TestOnly;

import net.sourceforge.uiq3.Asserted;

/**
 * HP-45 Calculator Engine
 *
 * @author "Martin Krischik" «krischik@users.sourceforge.net»
 */
@SuppressWarnings ({"UnnecessaryLabelOnBreakStatement", "HardcodedFileSeparator"})
public abstract class Calculator
   implements
   net.sourceforge.uiq3.calculator.ICalculator
{
   /**
    * location of the +/- sign of the exponent inside a register or memory.
    */
   private static final int EXPONENT_SIGN = 2;
   /**
    * ⁰ SUPERSCRIPT ZERO Unicode: U+2070, UTF-8: E2 81 B0
    */
   private static final char[] EXPONENT_CHAR = {
      '⁰',
      '¹',
      '²',
      '³',
      '⁴',
      '⁵',
      '⁶',
      '⁷',
      '⁸',
      '⁹'
   };
   /**
    * <p>
    * Key event processing status
    * </p>
    * <dl>
    * <dt>Key_Flag_Idle</dt>
    * <dd>Calculator is idle and ready to receive the next button from the key buffer</dd>
    * <dt>Key_Flag_Pressed</dt>
    * <dd>Calculator is processing the pressed event</dd>
    * <dt>Key_Flag_Wait</dt>
    * <dd>Waiting for the calcuator to process the event</dd>
    * <dt>Key_Flag_Depressed</dt>
    * <dd>Calculator is processing the de-pressed event</dd>
    * </dl>
    */
   private static final int Key_Flag_Depressed = 2;
   /**
    * <p>
    * Key event processing status
    * </p>
    * <dl>
    * <dt>Key_Flag_Idle</dt>
    * <dd>Calculator is idle and ready to receive the next button from the key buffer</dd>
    * <dt>Key_Flag_Pressed</dt>
    * <dd>Calculator is processing the pressed event</dd>
    * <dt>Key_Flag_Wait</dt>
    * <dd>Waiting for the calcuator to process the event</dd>
    * <dt>Key_Flag_Depressed</dt>
    * <dd>Calculator is processing the de-pressed event</dd>
    * </dl>
    */
   private static final int Key_Flag_Idle = 0;
   /**
    * <p>
    * Key event processing status
    * </p>
    * <dl>
    * <dt>Key_Flag_Idle</dt>
    * <dd>Calculator is idle and ready to receive the next button from the key buffer</dd>
    * <dt>Key_Flag_Pressed</dt>
    * <dd>Calculator is processing the pressed event</dd>
    * <dt>Key_Flag_Wait</dt>
    * <dd>Waiting for the calcuator to process the event</dd>
    * <dt>Key_Flag_Depressed</dt>
    * <dd>Calculator is processing the de-pressed event</dd>
    * </dl>
    */
   private static final int Key_Flag_Pressed = 1;
   /**
    * <p>
    * Key event processing status
    * </p>
    * <dl>
    * <dt>Key_Flag_Idle</dt>
    * <dd>Calculator is idle and ready to receive the next button from the key buffer</dd>
    * <dt>Key_Flag_Pressed</dt>
    * <dd>Calculator is processing the pressed event</dd>
    * <dt>Key_Flag_Wait</dt>
    * <dd>Waiting for the calcuator to process the event</dd>
    * <dt>Key_Flag_Depressed</dt>
    * <dd>Calculator is processing the de-pressed event</dd>
    * </dl>
    */
   private static final int Key_Flag_Wait = 3;
   /**
    * <p>
    * Class logger instance.
    * </p>
    */
   private static final java.util.logging.Logger Logger;
   @NonNls
   private static final String Off_Text = "Off (press CLx)";
   /**
    * <p>
    * Class logger tag.
    * </p>
    */
   private static final String TAG;
   @SuppressWarnings ("HardcodedFileSeparator")
   @NonNls
   static final String HP45_LST = "/rpn-45.lst";
   /**
    * ROM Groups
    */
   static final int Max_Group = 2;
   /**
    * RAM Size
    */
   static final int Max_Ram = 100;
   /**
    * ROMs per group
    */
   static final int Max_Rom = 8;
   /**
    * <p>
    * maximum amount of opcodes
    * </p>
    */
   static final int Opcode_Size = 1024;
   /**
    * Size of each ROM
    */
   static final int Rom_Size = 256;
   /**
    * s register size
    */
   static final int S_Size = 12;
   /**
    * Size of one register.
    */
   static final int Word_Size = 14;
   /**
    * names of all registers
    */
   static final String[] REGISTER_NAMES;

   static
   {
      REGISTER_NAMES = new String[] {
	 "a     :",
	 "b     :",
	 "c / X :",
	 "d / Y :",
	 "e / Z :",
	 "f / T :",
	 "m     :"
      };
      TAG = Calculator.class.getName ();
      Logger = java.util.logging.Logger.getLogger (Calculator.TAG);
   } // static

   static String
   registerToString (final byte @NotNull [] register)
   {
      final StringBuilder retval = new StringBuilder (20);

      for (int i = Calculator.Word_Size - 1; i >= 0; i--)
      {
	 if (i == Calculator.Word_Size - 3)
	 {
	    retval.append (".");
	 }
	 if (i == EXPONENT_SIGN || i == Calculator.Word_Size - 1)
	 {
	    retval.append (" ");
	 } // if

	 retval.append (Integer.toString (register[i]));
      } // for

      return retval.toString ();
   }

   @SuppressWarnings ("ConstantValueVariableUse") static String
   memoryToString (final byte @NotNull [] register)
   {
      final StringBuilder retval = new StringBuilder (20);

      for (int i = Calculator.Word_Size - 1; i > EXPONENT_SIGN; i--)
      {
	 if (i == Calculator.Word_Size - 3)
	 {
	    retval.append (".");
	 }
	 if (i == Calculator.Word_Size - 1)
	 {
	    if (register[i] >= 8)
	    {
	       retval.append (" -");
	    }
	    else
	    {
	       retval.append ("  ");
	    } // if
	 }
	 else
	 {
	    retval.append (Integer.toString (register[i]));
	 } // if
      } // for

      final int exponent;

      if (register[EXPONENT_SIGN] >= 8)
      {
	 retval.append (" ⁻");
	 exponent = 100 - (register[1] * 10 + register[0]);
      }
      else
      {
	 retval.append ("  ");
	 exponent = register[1] * 10 + register[0];
      }

      Asserted.range (exponent, 0, 100, "exponent");

      retval.append (EXPONENT_CHAR[exponent / 10]);
      retval.append (EXPONENT_CHAR[exponent % 10]);

      return retval.toString ();
   }

   /**
    * register a, general purpose
    */
   final byte[] a = new byte[Calculator.Word_Size];
   /**
    * register b, general purpose
    */
   final byte[] b = new byte[Calculator.Word_Size];
   /**
    * register c, X
    */
   final byte[] c = new byte[Calculator.Word_Size];
   /**
    * register d, Y
    */
   final byte[] d = new byte[Calculator.Word_Size];
   /**
    * register e, Z
    */
   final byte[] e = new byte[Calculator.Word_Size];
   /**
    * register f, T
    */
   final byte[] f = new byte[Calculator.Word_Size];
   /**
    * register m, scratch pad.
    */
   final byte[] m = new byte[Calculator.Word_Size];
   /**
    * Calculator RAM.
    */
   final byte[][] ram = new byte[Calculator.Max_Ram][Calculator.Word_Size];
   /**
    * register s
    */
   final byte[] s = new byte[Calculator.S_Size];
   /**
    * All Rom modules
    */
   final short[][][] ucode = new short[Calculator.Max_Group][Calculator.Max_Rom][Calculator.Rom_Size];
   /**
    * Calculator is running - needed for power saving which was not there in the original
    */
   private boolean Calculator_Running = false;
   /**
    * Calculator is not running - needed for end program. If you are wondering why there is a Calculator_Running and and
    * a Calculator_Shutdown - there is that little time between the calculator stop running and beeing full shutdown
    */
   private volatile boolean Calculator_Shutdown;
   /**
    * currently displayed text.
    */
   @NotNull
   private String Display_Text = "";
   public static final String DISPLAY_TEXT_CHANGED = "Display_Text_Changed";
   /**
    * Group address read from file
    */
   private int Group_Read;
   /**
    * <p>
    * Key event processing status
    * </p>
    * <dl>
    * <dt>Key_Flag_Idle</dt>
    * <dd>Calculator is idle and ready to receive the next button from the key buffer</dd>
    * <dt>Key_Flag_Pressed</dt>
    * <dd>Calculator is processing the pressed event</dd>
    * <dt>Key_Flag_Wait</dt>
    * <dd>Waiting for the calculator to process the event</dd>
    * <dt>Key_Flag_Depressed</dt>
    * <dd>Calculator is processing the de-pressed event</dd>
    * </dl>
    */
   private int Key_Flag = Calculator.Key_Flag_Idle;
   /**
    * System time when last Key was clicked.
    */
   private long Key_Time;
   /**
    * Opcode read from file
    */
   private int Opcode_Read;
   /**
    * address read from file
    */
   private int Pointer_Read;
   /**
    * Rom address read from file
    */
   private int Rom_Read;
   /**
    * clear register operations
    */
   @Nullable
   private net.sourceforge.uiq3.calculator.IOperation do_op_clear_reg = null;
   /**
    * clear s register operation
    */
   @Nullable
   private net.sourceforge.uiq3.calculator.IOperation do_op_clear_s = null;
   /**
    * array of all operator functions. not null after reset
    */
   @NotNull
   private net.sourceforge.uiq3.calculator.IOperation[] op_fcn = null;
   /**
    * Display enable - used to switch off the display so that intermediate result are not shown to the user.
    */
   boolean Display_Enable = false;
   /**
    * Canvas class where we display our result or get our keycodes
    */
   @Nullable
   net.sourceforge.uiq3.ui.ICalculatorCanvas form = null;
   /**
    * Number of virtual CPU cycles until next IO run.
    */
   int IO_Count = 0;
   /**
    * previous carry
    */
   byte Previous_Carry;
   /**
    *
    */
   int Program_Counter = 0;
   /**
    *
    */
   int Program_Group = 0;

   // #ifdef DEBUG
   // # /**
   // # * used to store complete five-digit octal address of instruction. Used for
   // # * debugging only.
   // # */
   // # int Previous_Program_Counter;
   // #endif
   /**
    *
    */
   int Program_Rom = 0;
   /**
    *
    */
   int Return_Program_Counter;
   /**
    * carry
    */
   byte carry;
   /**
    *
    */
   int del_grp = 0;
   /**
    *
    */
   int del_rom = 0;
   /**
    *
    */
   int p = 0;
   /**
    * ram address
    */
   int ram_addr;
   /**
    * Handle property changes. We don't use the full feature with oldValue and new newValue as that would be overkill.
    */
   private final java.beans.PropertyChangeSupport propertyChangeSupport;

   Calculator ()
   {
      // Calculator.Logger.entering (Calculator.TAG, "Calculator");

      this.propertyChangeSupport = new java.beans.PropertyChangeSupport (this);

      // Calculator.Logger.exiting (Calculator.TAG, "Calculator");
      return;
   } // Calculator

   public void addPropertyChangeListener (final java.beans.PropertyChangeListener listener)
   {
      this.propertyChangeSupport.addPropertyChangeListener (listener);
   }

   public void removePropertyChangeListener (final java.beans.PropertyChangeListener listener)
   {
      this.propertyChangeSupport.removePropertyChangeListener (listener);
   }

   /**
    * Current display mode
    * <dl>
    * <dt>Display_Alpha</dt>
    * <dd>Displays the content of the Alpha register.</dd>
    * <dt>Display_L</dt>
    * <dd>Displays the content of the L0 aka X register.</dd>
    * <dt>Display_Formmated</dt>
    * <dd>Displays the content of the L0 aka X register inside an alpha
    * </dl>
    *
    * @return current display mode
    */
   @Override @NotNull
   public Display_Type
   Display_Mode ()
   {
      throw new UnsupportedOperationException (Constants.Not_Needed);
   } // Get_Display_Mode

   /**
    * Current display mode
    * <dl>
    * <dt>Display_Alpha</dt>
    * <dd>Displays the content of the Alpha register.</dd>
    * <dt>Display_L</dt>
    * <dd>Displays the content of the L0 aka X register.</dd>
    * <dt>Display_Formatted</dt>
    * <dd>Displays the content of the L0 aka X register inside an alpha
    * </dl>
    *
    * @param Display_Mode
    *    new display mode
    */
   @Override public void
   Display_Mode (@NotNull final Display_Type Display_Mode)
   {
      throw new UnsupportedOperationException (Constants.Not_Needed);
   } // Display_Mode

   /**
    * returns the currently displayed text
    *
    * @return Display_Text Test to display
    */
   @NotNull String
   getDisplayText ()
   {
      return this.Display_Text;
   } // Display_Text

   /**
    * changes the currently displayed text
    *
    * @param Display_Text
    *    Test to display
    */
   void
   setDisplayText (@NotNull final String Display_Text)
   {
      final String old = this.Display_Text;
      this.Display_Text = Display_Text;

      this.propertyChangeSupport.firePropertyChange (DISPLAY_TEXT_CHANGED, old, Display_Text);

      if (this.form != null)
      {
	 this.form.setJavaText (Display_Text);
      } // if
      return;
   } // Display_Text

   /**
    * Set Canvas to display result read keyboard
    *
    * @param value
    *    form in which the calculator is shown.
    */
   @SuppressWarnings ("ObjectEquality")
   public void
   setForm (@Nullable final net.sourceforge.uiq3.ui.ICalculatorCanvas value)
   {
      // Calculator.Logger.entering (Calculator.TAG, "value", value);

      if (this.form != value)
      {
	 this.form = value;

	 if (this.form != null)
	 {
	    this.form.setJavaText (this.Display_Text);
	 }
      } // if

      // Calculator.Logger.exiting (Calculator.TAG, "value");
      return;
   } // setForm

   /**
    * get the last key read from the keyboard buffer
    *
    * @return key code.
    */
   int Get_Read_Key ()
   {
      Calculator.Logger.entering (Calculator.TAG, "Get_Read_Key");

      final int Retval;

      if (this.form == null)
      {
	 Retval = net.sourceforge.uiq3.calculator.IKey.No_Key;
      }
      else
      {
	 Calculator.Logger.log (java.util.logging.Level.FINE, "  > Key_Flag        = " + this.Key_Flag);
	 Retval = this.form.getReadKey ();
      } // if

      Calculator.Logger.exiting (Calculator.TAG, "Get_Read_Key", Retval);
      return Retval;
   } // Get_Read_Key

   /**
    * Display result and read keyboard
    */
   @SuppressWarnings ("MagicNumber")
   private void Handle_IO ()
   {
      // Calculator.Logger.entering (Calculator.TAG, "Handle_IO");

      final StringBuilder Display_String = new StringBuilder (20);

      if (this.Display_Enable)
      {
	 // "a" contains the number to display and "b" contains format information:
	 // if "b" is 9 then show a space
	 // if "b" is 2 append an additional dot
	 // position 2 and 13 of "a" contain the sign.
	 // the of "a" are the actual digits.
	 for (int i = Calculator.Word_Size - 1; i >= 0; i--)
	 {
	    if (this.b[i] >= 8)
	    {
	       Display_String.append (" ");
	    }
	    else if (i == 2)
	    {
	       if (this.a[i] >= 8)
	       {
		  Display_String.append (" -");
	       }
	       else
	       {
		  Display_String.append ("  ");
	       } // if
	    }
	    else if (i == 13)
	    {
	       if (this.a[i] >= 8)
	       {
		  Display_String.append ("-");
	       }
	       else
	       {
		  Display_String.append (" ");
	       } // if
	    }
	    else
	    {
	       Display_String.append (Integer.toString (this.a[i]));
	    } // if
	    if (this.b[i] == 2)
	    {
	       Display_String.append (".");
	    } // if
	 } // for
      }
      else
      {
	 Display_String.append (" ");
      }// if

      this.setDisplayText (Display_String.toString ());

      switch (this.Key_Flag)
      {
	 case Key_Flag_Idle:
	 {
	    if ((this.form != null) && this.form.isReadKeyAvailable ())
	    {
	       this.Key_Flag = Calculator.Key_Flag_Pressed;
	       this.Key_Time = System.nanoTime ();
	    } // if
	    break;
	 }
	 case Key_Flag_Pressed:
	 {
	    break;
	 }
	 case Key_Flag_Depressed:
	 {
	    this.Key_Flag = Calculator.Key_Flag_Idle;
	    break;
	 }
	 case Key_Flag_Wait:
	 {
	    if (this.s[0] == 0)
	    {
	       // last key has been fully processed.
	       this.Key_Flag = Calculator.Key_Flag_Depressed;
	    }
	    break;
	 }
	 default:
	 {
	    this.Key_Flag = Calculator.Key_Flag_Idle;
	    Calculator.Logger.log (java.util.logging.Level.SEVERE, "LOG02510: Wong Key mode");
	    break;
	 }
      } // switch
      if (System.nanoTime () > (this.Key_Time + net.sourceforge.uiq3.Utils.Minutes_3))
      {
	 // after 3 minutes without key click - switch off
	 this.stop ();
      }
      else
      {
	 // Calculator.Logger.log (java.util.logging.Level.FINE, "  > Key_Flag        = " +
	 // this.Key_Flag);
	 // Calculator.Logger.log (java.util.logging.Level.FINE, "  > s [0]           = " +
	 // this.s [0]);
	 // after 1 second without key click - reduce virtual CPU speed.
	 this.Save_Energy (System.nanoTime () > (this.Key_Time + net.sourceforge.uiq3.Utils.Seconds_1));
      } // if

      // Calculator.Logger.exiting (Calculator.TAG, "Handle_IO");
      return;
   } // Handle_IO

   /**
    * Initialise Opcodes
    */
   @SuppressWarnings ("MagicNumber")
   synchronized void Init_Opcodes ()
   {
      this.setDisplayText ("Init Operations…");

      // noinspection ConstantConditions
      if (this.op_fcn == null)
      {
	 this.op_fcn = new net.sourceforge.uiq3.calculator.IOperation[Calculator.Opcode_Size];
	 this.do_op_clear_s = new op_clear_s (this);
	 this.do_op_clear_reg = new op_clear_reg (this);

	 for (int i = 0; i < Calculator.Opcode_Size; i += 4)
	 {
	    this.op_fcn[i] = new bad_op (this);
	    this.op_fcn[i + 1] = new op_jsb (this);
	    this.op_fcn[i + 2] = new op_arith (this);
	    this.op_fcn[i + 3] = new op_goto (this);
	 } // for

	 this.op_fcn[0x000] = new op_nop (this);
	 this.op_fcn[0x01c] = new op_dec_p (this);
	 this.op_fcn[0x028] = new op_display_toggle (this);
	 this.op_fcn[0x030] = new op_return (this);
	 this.op_fcn[0x034] = this.do_op_clear_s;
	 this.op_fcn[0x03c] = new op_inc_p (this);
	 this.op_fcn[0x0d0] = new op_keys_to_rom_addr (this);
	 this.op_fcn[0x0a8] = new op_c_exch_m (this);
	 this.op_fcn[0x128] = new op_c_to_stack (this);
	 this.op_fcn[0x1a8] = new op_stack_to_a (this);
	 this.op_fcn[0x200] = new op_rom_addr_to_buf (this);
	 this.op_fcn[0x228] = new op_display_off (this);
	 this.op_fcn[0x270] = new op_c_to_addr (this);
	 this.op_fcn[0x2a8] = new op_m_to_c (this);
	 this.op_fcn[0x2f0] = new op_c_to_data (this);
	 this.op_fcn[0x2f8] = new op_data_to_c (this);
	 this.op_fcn[0x328] = new op_down_rotate (this);
	 this.op_fcn[0x3a8] = this.do_op_clear_reg;

	 this.op_fcn[0x234] = new op_del_sel_grp (this);
	 this.op_fcn[0x2b4] = new op_del_sel_grp (this);

	 for (int i = 0; i < 1024; i += 128)
	 {
	    this.op_fcn[i | 0x010] = new op_sel_rom (this);
	    this.op_fcn[i | 0x074] = new op_del_sel_rom (this);
	 } // for

	 for (int i = 0; i < 1024; i += 64)
	 {
	    this.op_fcn[i | 0x018] = new op_load_constant (this);
	    this.op_fcn[i | 0x004] = new op_set_s (this);
	    this.op_fcn[i | 0x024] = new op_clr_s (this);
	    this.op_fcn[i | 0x014] = new op_test_s (this);
	    this.op_fcn[i | 0x00c] = new op_set_p (this);
	    this.op_fcn[i | 0x02c] = new op_test_p (this);
	 } // for
      } // if
      return;
   } // Init_Opcodes

   /**
    * @return true when calculator is running.
    */
   @Override
   public synchronized boolean isRunning ()
   {
      return this.Calculator_Running;
   } // Is_Running

   /**
    * Calculator is not running - needed for end program. If you are wondering why there is a Calculator_Running and and
    * a Calculator_Shutdown - there is that little time between the calculator stop running and being fully shutdown.
    *
    * @return when calculators is shut down
    */
   @TestOnly
   public boolean Is_Shutdown ()
   {
      return this.Calculator_Shutdown;
   } // Is_Shutdown

   /**
    * is waiting in interactive mode for user input.
    *
    * @return true when in interactive mode.
    */
   @Override
   public boolean isWaiting ()
   {
      return true;
   }

   /**
    * <p>locate the last step to display the free steps or for debugging.</p>
    *
    * @return last step
    */
   @Override
   public int Last_Step ()
   {
      return 0;
   }

   /**
    * @param Data_In
    *    file to read from
    * @throws java.io.IOException
    *    and I/O error while reading the file
    */
   @SuppressWarnings ("ResultOfMethodCallIgnored")
   void
   Load (@NotNull final java.io.DataInputStream Data_In)
      throws
      java.io.IOException
   {
      // Calculator.Logger.entering (Calculator.TAG, "Load");

      if (!Constants.DEMO)
      {
	 this.Display_Enable = Data_In.readBoolean ();
	 this.Key_Flag = Data_In.readInt ();
	 this.Program_Counter = Data_In.readInt ();
	 this.Program_Rom = Data_In.readInt ();
	 this.Program_Group = Data_In.readInt ();
	 this.Return_Program_Counter = Data_In.readInt ();

	 this.del_rom = Data_In.readInt ();
	 this.del_grp = Data_In.readInt ();
	 Data_In.read (this.a);
	 Data_In.read (this.b);
	 Data_In.read (this.c);
	 Data_In.read (this.d);
	 Data_In.read (this.e);
	 Data_In.read (this.f);
	 Data_In.read (this.m);
	 Data_In.read (this.s);
	 this.ram_addr = Data_In.readInt ();

	 for (int i = 0; i < Calculator.Max_Ram; i++)
	 {
	    Data_In.read (this.ram[i]);
	 } // for

	 this.p = Data_In.readInt ();
	 this.carry = Data_In.readByte ();
	 this.Previous_Carry = Data_In.readByte ();
	 this.IO_Count = Data_In.readInt ();
      } // if

      // Calculator.Logger.exiting (Calculator.TAG, "Load");
      return;
   } // Load

   /**
    * Parse address field
    *
    * @param oct
    *    string to parse
    * @return address parsed
    */
   @SuppressWarnings ({"FeatureEnvy", "TooBroadScope"})
   private boolean
   Parse_Address (@NotNull final String oct)
   {
      boolean Retval;

      try
      {
	 String s;

	 s = oct.substring (0, 1);
	 this.Group_Read = Integer.parseInt (s, 8);
	 s = oct.substring (1, 2);
	 this.Rom_Read = Integer.parseInt (s, 8);
	 s = oct.substring (2, 5);
	 this.Pointer_Read = Integer.parseInt (s, 8);

	 Asserted.range (this.Group_Read, 0, Calculator.Max_Group, "Group_Read");
	 Asserted.range (this.Rom_Read, 0, Calculator.Max_Rom, "Rom_Read");
	 Asserted.range (this.Pointer_Read, 0, Calculator.Rom_Size, "Pointer_Read");

	 Retval = true;
      }
      catch (@NotNull final NumberFormatException Exception)
      {
	 Calculator.Logger.log (java.util.logging.Level.SEVERE, "LOG01030:Address could not be parsed", Exception);
	 Retval = false;
      } // try

      return Retval;
   } // Parse_Address

   /**
    * Parse
    *
    * @param bin
    *    line to parse
    * @return Opcode found
    */
   private boolean
   Parse_Opcode (@NotNull final CharSequence bin)
   {
      this.Opcode_Read = 0;

      boolean Retval = true;
      Search:
      for (int i = 0; i < 10; i = i + 1)
      {
	 this.Opcode_Read <<= 1;

	 if (bin.charAt (i) == '1')
	 {
	    this.Opcode_Read += 1;
	 }
	 else if (bin.charAt (i) == '.')
	 {
	    this.Opcode_Read += 0;
	 }
	 else
	 {
	    Retval = false;
	    break Search;
	 } // if
      } // for

      return Retval;
   } // Parse_Opcode

   /**
    * This reads a zip file that is expected to contain one entry which is the HP-45 firmware listing. The instructions
    * are parsed for later execution.
    *
    * @param File_Name
    *    File name to read from
    * @throws java.io.IOException
    *    file could not be read
    */
   @SuppressWarnings ({"FeatureEnvy", "MagicNumber"})
   public synchronized void
   Read_Listing_File (final String File_Name)
      throws
      java.io.IOException
   {
      this.setDisplayText ("Read ROM file… " + (Calculator.Max_Group - this.Group_Read));

      final Class<? extends Calculator> self = this.getClass ();
      final java.io.InputStream Listing_File = self.getResourceAsStream (File_Name);

      Asserted.notNull (Listing_File, "Listing_File");

      try (final java.io.InputStreamReader Listing_Reader = new java.io.InputStreamReader (Listing_File))
      {

	 Read_File:
	 for (int count = 0; ; )
	 {
	    final String Buffer = net.sourceforge.uiq3.Utils.Read_Line (Listing_Reader);

	    if (Buffer == null)
	    {
	       break Read_File;
	    }

	    Calculator.Logger.log (java.util.logging.Level.FINE, "Processing Line: {0}", Buffer);

	    if ((Buffer.length () >= 25) && (Buffer.charAt (7) == 'L') && (Buffer.charAt (13) == ':') && this
	       .Parse_Address (Buffer.substring (8)) && this.Parse_Opcode (Buffer.substring (16)))
	    {
	       net.sourceforge.uiq3.Asserted.range (this.Group_Read, 0, Calculator.Max_Group, "Group_Read");
	       net.sourceforge.uiq3.Asserted.range (this.Rom_Read, 0, Calculator.Max_Rom, "Rom_Read");
	       net.sourceforge.uiq3.Asserted.range (this.Pointer_Read, 0, Calculator.Rom_Size, "Pointer_Read");
	       net.sourceforge.uiq3.Asserted.range (this.Opcode_Read, 0, Calculator.Opcode_Size, "Opcode_Read");

	       this.ucode[this.Group_Read][this.Rom_Read][this.Pointer_Read] = (short) this.Opcode_Read;

	       count = count + 1;

	       this.setDisplayText ("Read ROM file… " + (Calculator.Max_Rom - this.Rom_Read));
	    } // if
	 } // while
      }
      catch (@NotNull final java.lang.Exception Exception)
      {
	 Calculator.Logger.log (java.util.logging.Level.SEVERE, "LOG00101: Read ROM listing failed", Exception);
	 stop ();
      } // try

      //Calculator.Logger.log (
      //java.util.logging.Level.FINE,
      //"LOG02520: Read {0} words from “{1}”",
      //new Object[] {count, File_Name});
      return;
   } // Read_Listing_File

   /**
    * Reset the calculator
    */
   @Override
   public void
   reset ()
   {
      this.setDisplayText ("Reset Calculator…");

      this.Init_Opcodes ();
      this.Display_Enable = false;
      this.Key_Flag = Calculator.Key_Flag_Idle;
      this.Program_Counter = 0;
      this.Program_Rom = 0;
      this.Program_Group = 0;
      this.del_rom = 0;
      this.del_grp = 0;

      try
      {
	 assert this.do_op_clear_reg != null : "variable is set on init";
	 assert this.do_op_clear_s != null : "variable is set on init";

	 this.do_op_clear_reg.Execute ((short) (0));
	 this.do_op_clear_s.Execute ((short) (0));
      }
      catch (@NotNull final net.sourceforge.uiq3.calculator.Error Exception)
      {
	 Calculator.Logger.log (java.util.logging.Level.SEVERE, "LOG01010:Reset Calculator failed", Exception);
      } // try

      this.p = 0;
      this.IO_Count = 0;

      return;
   } // Reset

   /**
    * <p>
    * performs a full reset by deleting ram and the cache file as well a reloading the cache file.
    * </p>
    */
   @Override public void resetAll ()
   {
      // Calculator.Logger.entering (Calculator.TAG, "resetAll");

      try
      {
	 this.Read_Listing_File (net.sourceforge.uiq3.hp45.Calculator.HP45_LST);
      }
      catch (final java.io.IOException Exception)
      {
	 Calculator.Logger.log (java.util.logging.Level.SEVERE, "LOG02530:", Exception);

	 this.displayException (Exception);
      }

      for (int i = 0; i < Calculator.Max_Ram; i++)
      {
	 java.util.Arrays.fill (ram[i], (byte) 0);
      }
      this.reset ();
      
      // Calculator.Logger.exiting (Calculator.TAG, "resetAll");
      return;
   } // resetAll

   /**
    * @param Data_Out
    *    Output stream to write calculators state to.
    * @throws java.io.IOException
    *    and I/O while writing to Data_Out
    */
   void
   Save (@NotNull final java.io.DataOutputStream Data_Out)
      throws
      java.io.IOException
   {
      // Calculator.Logger.entering (Calculator.TAG, "Save");

      if (!Constants.DEMO)
      {
	 Data_Out.writeBoolean (this.Display_Enable);
	 Data_Out.writeInt (this.Key_Flag);
	 Data_Out.writeInt (this.Program_Counter);
	 Data_Out.writeInt (this.Program_Rom);
	 Data_Out.writeInt (this.Program_Group);
	 Data_Out.writeInt (this.Return_Program_Counter);

	 Data_Out.writeInt (this.del_rom);
	 Data_Out.writeInt (this.del_grp);
	 Data_Out.write (this.a);
	 Data_Out.write (this.b);
	 Data_Out.write (this.c);
	 Data_Out.write (this.d);
	 Data_Out.write (this.e);
	 Data_Out.write (this.f);
	 Data_Out.write (this.m);
	 Data_Out.write (this.s);
	 Data_Out.writeInt (this.ram_addr);

	 for (int i = 0; i < Calculator.Max_Ram; i++)
	 {
	    Data_Out.write (this.ram[i]);
	 } // for

	 Data_Out.writeInt (this.p);
	 Data_Out.writeByte (this.carry);
	 Data_Out.writeByte (this.Previous_Carry);
	 Data_Out.writeInt (this.IO_Count);
      } // if

      // Calculator.Logger.exiting (Calculator.TAG, "Save");
      return;
   } // Save

   /**
    * add support operation
    *
    * @param x
    *    x value
    * @param y
    *    y value
    * @return x+y
    */
   byte
   do_add (final byte x, final byte y)
   {
      int res = x + y + this.carry;
      if (res > 9)
      {
	 res -= 10;
	 this.carry = 1;
      }
      else
      {
	 this.carry = 0;
      } // if
      return (byte) res;
   } // do_add

   /**
    * subtract support operation
    *
    * @param x
    *    x value
    * @param y
    *    y value
    * @return x-y
    */
   byte
   do_sub (final byte x, final byte y)
   {
      int res = (x - y) - this.carry;

      if (res < 0)
      {
	 res += 10;
	 this.carry = 1;
      }
      else
      {
	 this.carry = 0;
      } // if
      return (byte) res;
   } // do_sub

   /**
    * Start virtual CPU
    *
    * @see net.sourceforge.uiq3.calculator.ICalculator#run()
    */
   @SuppressWarnings ({"OctalInteger", "FeatureEnvy", "MagicNumber"})
   @Override
   public final void
   run ()
   {
      // Calculator.Logger.entering (Calculator.TAG, "run");

      this.setDisplayText ("Start CPU…");
      this.Save_Energy (false);
      this.Key_Time = System.nanoTime ();
      this.Calculator_Running = true;
      this.Calculator_Shutdown = false;
      this.Display_Enable = true;
      this.Load ();

      synchronized (this)
      {
	 // Calculator.Logger.log (java.util.logging.Level.FINE,
	 // "> LOG00210:Calculator initialisation finished, notify all.");

	 this.notifyAll ();
      } // synchronized

      if (this.form != null)
      {
	 this.form.setOnOff (true);
      } // if

      while (this.Calculator_Running)
      {
	 try
	 {
	    // #ifdef DEBUG
	    // # this.Previous_Program_Counter =
	    // # (this.Program_Group << 12) | (this.Program_Rom << 9) | this
	    // .Program_Counter;
	    // #endif

	    if ((this.Group_Read >= Calculator.Max_Group) || (this.Group_Read < 0) ||
	       (this.Rom_Read >= Calculator.Max_Rom) || (this.Rom_Read < 0) ||
	       (this.Pointer_Read >= Calculator.Rom_Size) || (this.Pointer_Read < 0))
	    {
	       Calculator.Logger.log (
		  java.util.logging.Level.WARNING,
		  "LOG02521: bad address Group: {0} Rom: {1} Pointer: {2}",
		  new Object[] {
		     this.Group_Read,
		     this.Rom_Read,
		     this.Pointer_Read});

	       this.Program_Group = 0;
	       this.Program_Rom = 0;
	       this.Program_Counter = 0;
	    } // if

	    Asserted.range (this.Group_Read, 0, Calculator.Max_Group, "Group_Read");
	    Asserted.range (this.Rom_Read, 0, Calculator.Max_Rom, "Rom_Read");
	    Asserted.range (this.Pointer_Read, 0, Calculator.Rom_Size, "Pointer_Read");
	    Asserted.range (this.Program_Counter, 0, Calculator.Rom_Size, "Program_Counter");

	    final short[][] Opcode_Group = this.ucode[this.Program_Group];
	    final short[] Opcode_Rom = Opcode_Group[this.Program_Rom];
	    final short Opcode = Opcode_Rom[this.Program_Counter];

	    Asserted.range (Opcode, 0, Calculator.Opcode_Size, "Opcode");

	    this.Previous_Carry = this.carry;
	    this.carry = 0;

	    if (this.Key_Flag == Calculator.Key_Flag_Pressed)
	    {
	       this.s[0] = 1;
	       this.Key_Flag = Calculator.Key_Flag_Wait;
	    } // if

	    this.Program_Counter = this.Program_Counter + 1;

	    try
	    {
	       Asserted.notNull (this.op_fcn, "op_fcn");

	       this.op_fcn[Opcode].Execute (Opcode);
	    }
	    catch (@NotNull final net.sourceforge.uiq3.calculator.Error Exception)
	    {
	       Calculator.Logger.log (java.util.logging.Level.SEVERE, "LOG01020:execution op code failed", Exception);

	       this.displayException (Exception);
	    } // try

	    if (this.Program_Counter > 0377)
	    {
	       // wrap back to beginning of the ROM
	       this.Program_Counter -= 0400;
	    } // if

	    this.IO_Count--;

	    if (this.IO_Count <= 0)
	    {
	       this.Handle_IO ();
	       // make keyboard more responsive in slow
	       // case - handle_io slow down full speed runs
	       this.IO_Count = 50;
	    } // if
	 }
	 catch (@NotNull final ArrayIndexOutOfBoundsException Exception)
	 {
	    Calculator.Logger.log (java.util.logging.Level.SEVERE, "LOG02540:Java Error in main loop", Exception);

	    this.displayException (Exception);
	    this.reset ();
	 }
	 catch (@NotNull final Throwable Exception)
	 {
	    Calculator.Logger.log (java.util.logging.Level.SEVERE, "LOG00830:Java Error in main loop", Exception);

	    this.displayException (Exception);
	    this.resetAll ();
	 } // Throwable
      } // while

      this.Save ();

      // Report off status to gui
      //
      this.setDisplayText (Calculator.Off_Text);

      if (this.form != null)
      {
	 this.form.setOnOff (false);
      } // if

      this.Calculator_Shutdown = true;

      // Calculator.Logger.exiting (Calculator.TAG, "run");
      return;
   }// run

   /**
    * Stop virtual CPU
    *
    * @see net.sourceforge.uiq3.calculator.ICalculator#stop()
    */
   @Override
   public synchronized void stop ()
   {
      // Calculator.Logger.entering (Calculator.TAG, "stop");

      this.Calculator_Running = false;

      // Calculator.Logger.exiting (Calculator.TAG, "stop");
      return;
   } // stop

   @NotNull public String
   registerToString (final int address)
   {
      String retval = REGISTER_NAMES[address];

      switch (address)
      {
	 case 0:
	 {
	    retval += Calculator.registerToString (this.a);
	    break;
	 }
	 case 1:
	 {
	    retval += Calculator.registerToString (this.b);
	    break;
	 }
	 case 2:
	 {
	    retval += Calculator.memoryToString (this.c);
	    break;
	 }
	 case 3:
	 {
	    retval += Calculator.memoryToString (this.d);
	    break;
	 }
	 case 4:
	 {
	    retval += Calculator.memoryToString (this.e);
	    break;
	 }
	 case 5:
	 {
	    retval += Calculator.memoryToString (this.f);
	    break;
	 }
	 case 6:
	 {
	    retval += Calculator.registerToString (this.m);
	    break;
	 }
	 default:
	 {
	    retval = "Illegal register address «" + address + "»";
	    break;
	 }
      }

      return retval;
   }

   @NotNull public String
   memoryToString (final int address)
   {
      return (address == 0)
	 ? "LASTx :" + Calculator.memoryToString (this.ram[0])
	 : "R" + address + "    :" + Calculator.memoryToString (this.ram[address]);
   }

   @TestOnly
   @Override
   public String toString ()
   {
      return Calculator.TAG + "{super=" + super.toString () + ", a=" + java.util.Arrays.toString (this.a) + ", b=" +
	 java.util.Arrays.toString (this.b) + ", c=" + java.util.Arrays.toString (this.c) + ", Calculator_Running=" +
	 this.Calculator_Running + ", carry=" + this.carry + ", d=" + java.util.Arrays.toString (this.d) +
	 ", del_grp=" + this.del_grp + ", del_rom=" + this.del_rom + ", Display_Enable=" + this.Display_Enable +
	 ", Display_Text=“" + this.Display_Text + '”' + ", do_op_clear_reg=" + this.do_op_clear_reg +
	 ", do_op_clear_s=" + this.do_op_clear_s + ", e=" + java.util.Arrays.toString (this.e) + ", f=" +
	 java.util.Arrays.toString (this.f) + ", Form=" + this.form + ", Group_Read=" + this.Group_Read +
	 ", IO_Count=" + this.IO_Count + ", Key_Flag=" + this.Key_Flag + ", Key_Time=" + this.Key_Time + ", m=" +
	 java.util.Arrays.toString (this.m) + ", op_fcn=" + java.util.Arrays.toString (this.op_fcn) + ", Opcode_Read" +
	 "=" +
	 this.Opcode_Read + ", p=" + this.p + ", Pointer_Read=" + this.Pointer_Read + ", Previous_Carry=" +
	 this.Previous_Carry + ", Program_Counter=" + this.Program_Counter + ", Program_Group=" + this.Program_Group +
	 ", Program_Rom=" + this.Program_Rom + ", ram=" + java.util.Arrays.toString (this.ram) + ", ram_addr=" +
	 this.ram_addr + ", Return_Program_Counter=" + this.Return_Program_Counter + ", Rom_Read=" + this.Rom_Read +
	 ", s=" + java.util.Arrays.toString (this.s) + ", ucode=" + java.util.Arrays.toString (this.ucode) + '}';
   }
} // Calculator

// vim: set nowrap tabstop=8 shiftwidth=3 softtabstop=3 expandtab textwidth=96 :
// vim: set fileencoding=utf-8 filetype=java foldmethod=syntax spell spelllang=en_gb :
