/*
 * ************************************************************ {{{1 **********
 * Copyright © 2005 … 2023  "Martin Krischik" «krischik@users.sourceforge.net»
 * **************************************************************************
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see «http://www.gnu.org/licenses»..
 * ************************************************************ }}}1 **********
 */

package net.sourceforge.uiq3.hp45

import net.sourceforge.uiq3.test.And
import net.sourceforge.uiq3.test.Async_Assert_False
import net.sourceforge.uiq3.test.Feature
import net.sourceforge.uiq3.test.Given
import net.sourceforge.uiq3.test.Info
import net.sourceforge.uiq3.test.Scenario
import net.sourceforge.uiq3.test.Second
import net.sourceforge.uiq3.test.Then
import net.sourceforge.uiq3.test.When
import org.hamcrest.CoreMatchers.notNullValue
import org.hamcrest.MatcherAssert.assertThat
import org.hamcrest.core.Is.`is` as Is
import org.hamcrest.core.IsEqual.equalTo
import org.hamcrest.number.OrderingComparison.greaterThanOrEqualTo

/**
 * <p>test the calculator engine</p>
 *
 * @author "Martin Krischik" «krischik@users.sourceforge.net»
 */
@org.junit.jupiter.api.TestInstance(org.junit.jupiter.api.TestInstance.Lifecycle.PER_CLASS)
class Calculator_Test
{
   /**
    * Wait until all keystrokes from form have been read — And Then an extra 50ms.
    *
    * @param Form form supplying the keystrokes
    */
   private fun Wait_For_Execution_Finished(
      Form: Mock_Canvas, Final_Wait: Long = Second(1.0f)
   )
   {
      Async_Assert_False(
	 Message = { "Keystrokes where not consumed" },
	 Condition = { Form.isReadKeyAvailable })

      Thread.sleep(Final_Wait)
   } // Wait_For_Execution_Finished

   companion object
   {
      @org.junit.jupiter.api.BeforeAll
      @JvmStatic
      fun Before_All()
      {
	 Feature(Calculator_Test::class.java.simpleName)
	 net.sourceforge.uiq3.test.Init_Logger(debug = true)
      }
   }

   @org.junit.jupiter.api.Nested
   @org.junit.jupiter.api.DisplayName("Feature: Initializing")
   inner class Test_Initializing
   {
      init
      {
	 Feature("Initializing")
	 Info("As a user I want to initialize And start the calculator")
      }

      @org.junit.jupiter.api.Test
      @org.junit.jupiter.api.DisplayName("Scenario: Startup without running")
      fun test_Startup_without_running()
      {
	 Scenario("Startup without running")
	 Given("A calculator instance")

	 val Canvas = Mock_Canvas()
	 val Test = Mock_Calculator.startedCalculator(Canvas)

	 And("a calculator reset is performed")

	 Test.reset()

	 Then("then no error is reported")

	 And("toString can be called")
	 val debugString = Test.toString()

	 assertThat(debugString, notNullValue())
	 Info(debugString)
      } // Scenario

      @org.junit.jupiter.api.Test
      @org.junit.jupiter.api.DisplayName("Scenario: Startup, loading listing file")
      fun `test_Startup,_loading_listing_file`()
      {
	 Scenario("Startup, loading listing file")
	 Given("An calculator instance")

	 val Canvas = Mock_Canvas()
	 val Test = Mock_Calculator.startedCalculator(Canvas)

	 And("a calculator reset is performed")

	 Test.reset()

	 Then("then the listing file can be read")

	 Test.Read_Listing_File("/rpn-45.lst")

	 And("toString can be called")
	 val debugString = Test.toString()

	 assertThat(debugString, notNullValue())
	 Info(debugString)
      } // Scenario

      @org.junit.jupiter.api.Test
      @org.junit.jupiter.api.DisplayName("Scenario: Startup force ArrayIndexOutOfBoundsException")
      fun test_Startup_force_ArrayIndexOutOfBoundsException()
      {
	 Scenario("Startup force ArrayIndexOutOfBoundsException")
	 Given("An calculator instance")

	 val Canvas = Mock_Canvas()
	 val Test = Mock_Calculator.startedCalculator(Canvas)

	 Then("then the calculator can be started")

	 Thread.sleep(Second(0.1f))

	 Test.Program_Counter = -1

	 Thread.sleep(Second(0.1f))

	 assertThat(Test.Program_Counter, greaterThanOrEqualTo(0))

	 And("toString can be called")
	 val debugString = Test.toString()

	 assertThat(debugString, notNullValue())
	 Info(debugString)
      } // Scenario

      @org.junit.jupiter.api.Test
      @org.junit.jupiter.api.DisplayName("Scenario: toString")
      fun test_toString()
      {
	 Scenario("toString")
	 Given("an Calculator instance created And started")

	 val Test = Mock_Calculator.startedCalculator(Mock_Canvas())

	 Then("toString can be called")
	 val debugString = Test.toString()

	 assertThat(debugString, notNullValue())
	 Info(debugString)

	 And("then the calculator can be stopped")

	 Test.stop()
      } // Scenario

   } // Feature

   @org.junit.jupiter.api.Nested
   @org.junit.jupiter.api.DisplayName("Feature: register to string")
   inner class Test_Register_To_String
   {
      init
      {
	 Feature("CPU and RAM register to string")
	 Info("As developer i want to have a proper string representation of the CPU and RAM register.")
      }

      @org.junit.jupiter.api.Test
      @org.junit.jupiter.api.DisplayName("Scenario: convert CPU register to string")
      fun test_registerToString_01()
      {
	 Scenario("convert CPU register to string")
	 Given("An calculator instance")

	 val Canvas = Mock_Canvas()
	 val Test = Mock_Calculator.startedCalculator(Canvas)

	 When("then the calculator can be run")

	 Wait_For_Execution_Finished(Canvas)

	 Then("All Register are zero")

	 assertThat("a", Calculator.registerToString(Test.a), Is(equalTo(" 00.000000000 000")))
	 assertThat("b", Calculator.registerToString(Test.b), Is(equalTo(" 00.000000000 000")))
	 assertThat("c", Calculator.registerToString(Test.c), Is(equalTo(" 00.000000000 000")))
	 assertThat("d", Calculator.registerToString(Test.d), Is(equalTo(" 00.000000000 000")))
	 assertThat("e", Calculator.registerToString(Test.e), Is(equalTo(" 00.000000000 000")))
	 assertThat("f", Calculator.registerToString(Test.f), Is(equalTo(" 00.000000000 000")))

	 And("calculator can be stopped stopped")

	 Test.stop()
      } // test_registerToString_01
      
      @org.junit.jupiter.api.Test
      @org.junit.jupiter.api.DisplayName("Scenario: convert RAM register to string")
      fun test_memoryToString_01()
      {
	 Scenario("convert RAM register to string")
	 Given("An calculator instance")

	 val Canvas = Mock_Canvas()
	 val Test = Mock_Calculator.startedCalculator(Canvas)

	 When("then the calculator can be run")

	 Wait_For_Execution_Finished(Canvas)

	 Then("All Register are zero")

	 assertThat("a", Calculator.memoryToString(Test.a), Is(equalTo("  0.000000000  ⁰⁰")))
	 assertThat("b", Calculator.memoryToString(Test.b), Is(equalTo("  0.000000000  ⁰⁰")))
	 assertThat("c", Calculator.memoryToString(Test.c), Is(equalTo("  0.000000000  ⁰⁰")))
	 assertThat("d", Calculator.memoryToString(Test.d), Is(equalTo("  0.000000000  ⁰⁰")))
	 assertThat("e", Calculator.memoryToString(Test.e), Is(equalTo("  0.000000000  ⁰⁰")))
	 assertThat("f", Calculator.memoryToString(Test.f), Is(equalTo("  0.000000000  ⁰⁰")))

	 And("calculator can be stopped stopped")

	 Test.stop()

      } // Scenario
      
      @org.junit.jupiter.api.Test
      @org.junit.jupiter.api.DisplayName("Scenario: convert CPU register given by index to string")
      fun test_registerAsString()
      {
	 Scenario("convert CPU register given by index to string")
	 Given("An calculator instance")

	 val Canvas = Mock_Canvas()
	 val Test = Mock_Calculator.startedCalculator(Canvas)

	 When("then the calculator can be run")

	 Wait_For_Execution_Finished(Canvas)

	 Then("All Register are zero")

	 assertThat("a", Test.registerToString(0), Is(equalTo("a     : 00.000000000 000")))
	 assertThat("b", Test.registerToString(1), Is(equalTo("b     : 00.000000000 000")))
	 assertThat("c", Test.registerToString(2), Is(equalTo("c / X :  0.000000000  ⁰⁰")))
	 assertThat("d", Test.registerToString(3), Is(equalTo("d / Y :  0.000000000  ⁰⁰")))
	 assertThat("e", Test.registerToString(4), Is(equalTo("e / Z :  0.000000000  ⁰⁰")))
	 assertThat("f", Test.registerToString(5), Is(equalTo("f / T :  0.000000000  ⁰⁰")))

	 And("calculator can be stopped stopped")

	 Test.stop()
      } // Scenario
      
      @org.junit.jupiter.api.Test
      @org.junit.jupiter.api.DisplayName("Scenario: convert RAM register given by index to string")
      fun test_memoryToString_02()
      {
	 Scenario("convert RAM register given by index to string")
	 Given("An calculator instance")

	 val Canvas = Mock_Canvas()
	 val Test = Mock_Calculator.startedCalculator(Canvas)

	 When("then the calculator can be run")

	 Wait_For_Execution_Finished(Canvas)

	 Then("All Register are zero")

	 assertThat("LASTx", Test.memoryToString(0), Is(equalTo("LASTx :  0.000000000  ⁰⁰")))
	 assertThat("R1", Test.memoryToString(1), Is(equalTo("R1    :  0.000000000  ⁰⁰")))
	 assertThat("R2", Test.memoryToString(2), Is(equalTo("R2    :  0.000000000  ⁰⁰")))
	 assertThat("R3", Test.memoryToString(3), Is(equalTo("R3    :  0.000000000  ⁰⁰")))
	 assertThat("R4", Test.memoryToString(4), Is(equalTo("R4    :  0.000000000  ⁰⁰")))
	 assertThat("R5", Test.memoryToString(5), Is(equalTo("R5    :  0.000000000  ⁰⁰")))
	 assertThat("R6", Test.memoryToString(6), Is(equalTo("R6    :  0.000000000  ⁰⁰")))
	 assertThat("R7", Test.memoryToString(7), Is(equalTo("R7    :  0.000000000  ⁰⁰")))
	 assertThat("R8", Test.memoryToString(8), Is(equalTo("R8    :  0.000000000  ⁰⁰")))
	 assertThat("R9", Test.memoryToString(9), Is(equalTo("R9    :  0.000000000  ⁰⁰")))

	 And("calculator can be stopped stopped")

	 Test.stop()
      } // test_memoryToString_02g
   }
} // Calculator_Test

// vim: set nowrap tabstop=8 shiftwidth=3 softtabstop=3 noexpandtab textwidth=96 :
// vim: set fileencoding=utf-8 filetype=kotlin foldmethod=indent spell spelllang=en_gb :
