/*
 * ************************************************************ {{{1 **********
 * Copyright © 2005 … 2024  "Martin Krischik" «krischik@users.sourceforge.net»
 * **************************************************************************
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see «http://www.gnu.org/licenses»..
 * ************************************************************ }}}1 **********
 */

package net.sourceforge.uiq3.hp45

import net.sourceforge.uiq3.test.Second
import org.jetbrains.annotations.NotNull
import java.util.logging.Level as Log_Level

/**
 * <p>Test calculator</p>
 * @author "Martin Krischik" «krischik@users.sourceforge.net»
 */
class Mock_Calculator :
   Calculator()
{
   companion object
   {
      /**
       * class logger tag.
       */
      private val TAG: String = Mock_Calculator::class.java.name

      /**
       * class logger instance
       */
      private val logger: java.util.logging.Logger = java.util.logging.Logger.getLogger(TAG)

      /**
       * Name of the file in which the calculator state is stored.
       */
      private const val RAM_FILE_NAME: String = "RPN-45-test.ram"

      /**
       * create an mock calculator , a mock canvas
       * @param canvas canvas
       * @return calculator
       */
      @Suppress("PLATFORM_CLASS_MAPPED_TO_KOTLIN")
      fun startedCalculator(canvas: Mock_Canvas): Mock_Calculator
      {
	 logger.entering(TAG, "startedCalculator", canvas)

	 val retval = Mock_Calculator()
	 retval.reset()

	 val Calculator_Thread = Thread(retval, "Calculator Thread")

	 Calculator_Thread.start()

	 synchronized(retval as Object) {
	    retval.wait(Second(0.5f))
	 }

	 while (!retval.isRunning())
	 {
	    Thread.sleep(Second(0.5f))
	 }

	 retval.setForm(canvas)

	 logger.exiting(TAG, "startedCalculator", retval)
	 return retval;
      } // constructor      

      /**
       *
       * create an mock calculator , a mock canvas and load file into calculator
       * @param file file to load
       * @param canvas canvas
       * @return calculator
       */
      @Suppress("PLATFORM_CLASS_MAPPED_TO_KOTLIN")
      fun startedCalculator(file: String, canvas: Mock_Canvas): Mock_Calculator
      {
	 logger.entering(TAG, "startedCalculator", arrayOf(file, canvas))
	 val retval = Mock_Calculator()
	 retval.reset()

	 val Calculator_Thread = java.lang.Thread(retval, "Calculator Thread")

	 Calculator_Thread.start()

	 synchronized(retval as Object) {
	    retval.wait(Second(0.5f))
	 }

	 while (!retval.isRunning())
	 {
	    Thread.sleep(Second(0.5f))
	 }

	 retval.setForm(canvas)
	 retval.Load(file)

	 logger.exiting(TAG, "startedCalculator", retval)
	 return retval;
      } // startedCalculator
   } // companion

   /**
    * Clear all alpha register. And just calling "Clear" is not enough.
    */
   override fun displayClear()
   {
      logger.entering(TAG, "displayClear")
      logger.exiting(TAG, "displayClear")
      return
   } // displayClear

   /**
    * display additional informations on the error. this is device and setup dependent.
    *
    * @param Exception Exception to display
    */
   override fun displayException(@NotNull Exception: Throwable)
   {
      logger.entering(TAG, "displayException", Exception)
      logger.exiting(TAG, "displayException")
      return
   } // displayException

   /**
    * Display value as formatted text
    *
    * @param Text
    * to be displayed
    */
   override fun Display_Formatted(Text: ShortArray)
   {
      logger.entering(TAG, "Display_Formatted", Text)
      logger.exiting(TAG, "Display_Formatted")
      return
   } // Display_Formatted

   /**
    * Display Key merge info.
    *
    * @param Text
    * info to be displayed
    */
   override fun Display_Key(Text: ShortArray)
   {
      logger.entering(TAG, "Display_Key", Text)
      logger.exiting(TAG, "Display_Key")
      return
   }

   /**
    * Reset current value Display
    */
   override fun Display_L()
   {
      logger.entering(TAG, "Display_L")
      logger.exiting(TAG, "Display_L")
      return
   }

   /**
    * Display texts as dual line message
    *
    * @param Text
    * to be displayed
    */
   override fun Display_Message(Text: ShortArray)
   {
      logger.entering(TAG, "Display_Message", Text)
      logger.exiting(TAG, "Display_Message")
      return
   }

   /**
    * Load Calculator State
    */
   override fun Load()
   {
      logger.entering(TAG, "Load")

      val filename = java.nio.file.Path.of(
	 /* first   = */ "target",
	 /* ...more = */ "surefire-reports",
	 Mock_Calculator.RAM_FILE_NAME
      )
      this.Load(filename.toFile());

      logger.exiting(TAG, "Load")
      return
   }

   /**
    * Load Calculator State
    */
   override fun Load(filename: String)
   {
      logger.entering(TAG, "Load", filename)

      try
      {
	 java.io.FileInputStream(filename).use { File_In ->
	    java.io.DataInputStream(File_In).use { Data_In ->
	       this.Load(Data_In)
	    } // Data_In
	 } // File_In
      }
      catch (Exception: Throwable)
      {
	 Mock_Calculator.logger.log(
	    Log_Level.WARNING,
	    "LOG00140:Could not load calculator state from «" + filename + "».",
	    Exception
	 )
	 this.reset()
      } // try

      logger.exiting(TAG, "Load")
      return
   }

   /**
    * Load Calculator State
    */
   override fun Save()
   {
      logger.entering(TAG, "Save")

      val filename = java.nio.file.Path.of(
	 /* first   = */ "target",
	 /* ...more = */ "surefire-reports",
	 Mock_Calculator.RAM_FILE_NAME
      )
      this.Save(filename.toFile());

      logger.exiting(TAG, "Save")
      return
   }

   /**
    * Save Calculator state
    */
   override fun Save(filename: String)
   {
      logger.entering(TAG, "Save", filename)

      try
      {
	 java.io.FileOutputStream(filename).use { File_Out ->
	    java.io.DataOutputStream(File_Out).use { Data_Out ->
	       this.Save(Data_Out)
	    } // Data_Out
	 } // File_Out
      }
      catch (Exception: java.io.IOException)
      {
	 Mock_Calculator.logger.log(
	    Log_Level.SEVERE,
	    "LOG00150:Could not save calculator state «" + filename + "»..",
	    Exception
	 )
      } // try

      logger.exiting(TAG, "Save")
      return
   }

   /**
    * Enable or disable energy savings mode. for example by reducing thread priority.
    *
    * @param enable
    * start saving energy
    */
   override fun Save_Energy(enable: Boolean)
   {
      logger.entering(TAG, "Save_Energy", enable)
      logger.exiting(TAG, "Save_Energy")
      return
   } // Save_Energy
}

// vim: set nowrap tabstop=8 shiftwidth=3 softtabstop=3 noexpandtab textwidth=96 :
// vim: set fileencoding=utf-8 filetype=kotlin foldmethod=indent spell spelllang=en_gb :
