/*
 * ************************************************************ {{{1 **********
 * Copyright © 2005 … 2023  "Martin Krischik" «krischik@users.sourceforge.net»
 * **************************************************************************
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see «http://www.gnu.org/licenses»..
 * ************************************************************ }}}1 **********
 */

package net.sourceforge.uiq3.hp45

import net.sourceforge.uiq3.ui.ICalculatorCanvas
import org.jetbrains.annotations.NotNull
import org.jetbrains.annotations.TestOnly

/**
 * <p>A Canvas Mock class</p>
 * @param keystrokes Keystrokes to send to the calculator
 * @author "Martin Krischik" «krischik@users.sourceforge.net»
 */
class Mock_Canvas(
   keystrokes: List<Short> = emptyList()
) :
   ICalculatorCanvas,
   net.sourceforge.uiq3.printer.ICanvas
{ // Mock_Canvas
   companion object
   {
      /**
       * log tag
       */
      val TAG: String = Mock_Canvas::class.java.getName()

      /**
       * log instance.
       */
      val Logger = java.util.logging.Logger.getLogger(TAG)
   }

   private var currentKey: Iterator<Short> = keystrokes.iterator()
   private var displayText: Array<ShortArray> = Array(size = 0) { ShortArray(size = 0) }
   override var printout = ""

   /**
    * Get calculator key code of the currently pressed key.
    *
    * @return calculator key code
    */
   override var pressedKey: Short = net.sourceforge.uiq3.calculator.IKey.No_Key

   /**
    * Get calculator key code of the last key (de)pressed.
    *
    * @return calculator key code
    */
   override var readKey: Short
      get()
      {
	 Logger.entering(TAG, "readKey")

	 val Retval = if (currentKey.hasNext())
	 {
	    currentKey.next()
	 }
	 else
	 {
	    net.sourceforge.uiq3.calculator.IKey.No_Key
	 } // if

	 Logger.exiting(TAG, "readKey", Retval)
	 return Retval
      } // Get_Read_Key
      set(value)
      {
	 this.keystrokes = listOf(value)
      }

   override val isReadKeyAvailable: Boolean
      get(): Boolean = currentKey.hasNext()

   /**
    * Set text to be displayed (as java string, used by RPN-45 only).
    */
   @get:TestOnly
   override var javaText: String = ""

   /**
    * Set the indicators (Shift, Alpha, Deg, Grad, Gra, etc. pp.)
    *
    * @param Indicators
    * indicators to be shown
    */
   @get:TestOnly
   override var indicators = BooleanArray (size = 0)

   private var keystrokes: List<Short> = keystrokes
      set(value)
      {
	 field = value
	 currentKey = keystrokes.iterator()
      } // Set_Keystrokes

   /**
    * <p>Set calculator Thread so the canvas can wake up the calculator after key / touch screen
    * clicks.</p>
    *
    * @param CPU_Thread
    * CPU Thread.
    * @author "Martin Krischik" «krischik@users.sourceforge.net»
    */
   override fun bindCPUThread(CPU_Thread: Thread?)
   {
      Logger.entering(TAG, "Bind_CPU_Thread", CPU_Thread)
      Logger.exiting(TAG, "Bind_CPU_Thread")
   } // Bind_CPU_Thread

   /**
    * <p>Set text to be displayed (calculator alpha codes). Null implementation for calculators
    * without alpha display.</p>
    *
    * @param Row
    * Display row to set
    * @param Text
    * to be displayed
    */
   override fun setDisplayText(Row: Int, @NotNull Text: ShortArray)
   {
      throw UnsupportedOperationException(Constants.Not_Needed)
   } // Set_Text

   /**
    * Fet text currently displayed (as one java string).
    *
    * @param Row
    * Display row to set
    * @return
    * text currently displayed
    */
   @TestOnly
   override fun getDisplayText(Row: Int): ShortArray
   {
      throw UnsupportedOperationException(Constants.Not_Needed)
   }

   /**
    * Fet text currently displayed (as one java string).
    *
    * @param Row
    * Display row to set
    * @return
    * text currently displayed
    */
   @TestOnly
   override fun getJavaText(Row: Int) = javaText

   /**
    * Fet text currently displayed (as one java string).
    *
    */
   @TestOnly

   /**
    * <p>Toggle Keyboard help on and off</p>
    */
   override fun toggleHelp()
   {
      Logger.entering(TAG, "Toggle_Help")
      Logger.exiting(TAG, "Toggle_Help")
   } // Toggle_Help

   /**
    * <p>display additional information on the error. this is device and setup dependent.</p>
    *
    * @param Exception
    *    Exception to display
    */
   override fun displayException(@NotNull Exception: Throwable)
   {
      Logger.entering(TAG, "displayException", Exception)
      Logger.exiting(TAG, "displayException")
   } // displayException

   /**
    * <p> set the calculators on of status. </p>
    *
    * @param on
    *    calculator is switched on
    */
   override fun setOnOff(on: Boolean)
   {
      Logger.entering(TAG, "Set_On_Off", on)
      Logger.exiting(TAG, "Set_On_Off")
   }
}

// vim: set nowrap tabstop=8 shiftwidth=3 softtabstop=3 noexpandtab textwidth=96 :
// vim: set fileencoding=utf-8 filetype=kotlin foldmethod=indent spell spelllang=en_gb :
