#!/opt/local/bin/zsh
########################################################### {{{1 ###########
#   Copyright (C) 2005 … 2021  Martin Krischik
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
########################################################### }}}1 ###########

setopt No_XTrace
setopt No_Verbose
setopt SH_Word_Split
setopt Err_Exit
setopt CSH_Null_Glob

typeset Git_Server="ssh://krischik@git.code.sf.net/p/uiq3/git"
typeset Source_Module="HP45"
typeset Main_Module="RPN-45"
typeset Target=${PROJECT_HOME}/target
typeset Deployed=${PROJECT_HOME}/deployed
typeset Android_Dir="${PROJECT_HOME}/Android/${Source_Module}/build/outputs"
typeset Desktop_Dir="${PROJECT_HOME}/Java/${Source_Module}/Desktop/target"

function Create_OSX ()
{
    local in_Name=${1}

    printf "\e[42m%-10.10s : \e[43m %-40.40s \e[m\n" "Package:" "create ${in_Name} DMG"

    ditto -x -j													\
	"${Main_Module}-Src-${UIQ3_VERSION}/Java/${Source_Module}/src/macosx/resources/DMG.cpio.bz2"		\
	"."

    gcp --verbose --archive											    \
	"${Main_Module}-Src-${UIQ3_VERSION}/Java/${Source_Module}/src/macosx/resources/${Main_Module}-Drive.icns"   \
	"DMG/data/.VolumeIcon.icns"

    gcp --verbose --archive											\
	--target-directory="DMG/data"										\
	"${Desktop_Dir}/${in_Name}.app"

    gln --symbolic												\
	--target-directory="DMG/data"										\
	"/Applications"

    rm ${in_Name}-OSX-${UIQ3_VERSION}.dmg || true

    hdiutil create												\
	-fs HFS+												\
	-volname "${in_Name}-OSX-${UIQ3_VERSION}"								\
	-srcfolder DMG/data											\
	"${in_Name}-OSX-${UIQ3_VERSION}"

    rm ${Deployed}/${in_Name}-OSX-${UIQ3_VERSION}.dmg || true

    hdiutil convert												\
	${in_Name}-OSX-${UIQ3_VERSION}.dmg									\
	-format UDBZ												\
	-o  ${Deployed}/${in_Name}-OSX-${UIQ3_VERSION}.dmg

    printf "\e[42m%-10.10s : \e[43m %-40.40s \e[m\n" "OSX:" "Cleanup"

    grm --recursive --force "DMG"
    grm ${in_Name}-OSX-${UIQ3_VERSION}.dmg
    return
}

gmkdir --parents   ${Deployed}

###############################################################################
# Export source files from Git
###############################################################################

pushd "${PROJECT_HOME}"
    printf "\e[42m%-10.10s : \e[43m %-40.40s \e[m\n" "Package:" "Export release directory"
    grm    --recursive "${Target}/Export" || true
    gmkdir --parents   "${Target}/Export"
    git archive "release/${UIQ3_VERSION}" | tar -x -C "${Target}/Export"
popd

###############################################################################
# Create source archives
###############################################################################

pushd ${Target}
    printf "\e[42m%-10.10s : \e[43m %-40.40s \e[m\n" "Package:" "Move relevant source directories"

    grm --recursive "${Main_Module}-Src-${UIQ3_VERSION}" || true 

    for I in			    \
	"Android/${Source_Module}"  \
	"Android/Library"	    \
	"Java/${Source_Module}"	    \
	"Java/Calculator"
    do
	printf "\e[42m%-10.10s : \e[43m %-40.40s \e[m\n" "Package:" "Move ${I}"

	gmkdir --parents "${Main_Module}-Src-${UIQ3_VERSION}/${I:h}"
	gmv --verbose "Export/${I}" "${Main_Module}-Src-${UIQ3_VERSION}/${I}"
    done; unset I

    printf "\e[42m%-10.10s : \e[43m %-40.40s \e[m\n" "Package:" "Update release directory"

    chmod -R o=g  "${Main_Module}-Src-${UIQ3_VERSION}"
    chmod -R g=u  "${Main_Module}-Src-${UIQ3_VERSION}"

    printf "\e[42m%-10.10s : \e[43m %-40.40s \e[m\n" "Package:" "Copy documentation"
    gmkdir --parent "${Main_Module}-Doc-${UIQ3_VERSION}"

    gcp --verbose --archive											\
	--target-directory="${Main_Module}-Doc-${UIQ3_VERSION}"							\
	"${Target}/Export/Downloads/Documents/HP-45"*.pdf

    7za a -tzip													\
	"${Deployed}/${Main_Module}-Doc-${UIQ3_VERSION}.zip"							\
	"${Main_Module}-Doc-${UIQ3_VERSION}"

    Create_OSX "${Main_Module}"

    printf "\e[42m%-10.10s : \e[43m %-40.40s \e[m\n" "Package:" "Remove large files from source"
    # Do not add potentionaly lage files, which are not needed for compile
    # to the Source Archive.

    grm														\
	${Main_Module}-Src-${UIQ3_VERSION}/**/Documents/*.pdf							\
	${Main_Module}-Src-${UIQ3_VERSION}/**/Documents/*.odt							\
	${Main_Module}-Src-${UIQ3_VERSION}/**/Pictures/*.png							\
	${Main_Module}-Src-${UIQ3_VERSION}/**/Pictures/*.jpg

    printf "\e[42m%-10.10s : \e[43m %-40.40s \e[m\n" "Package:" "create source tar file"
    tar --create --bzip2											\
	--file="${Deployed}/${Main_Module}-Src-${UIQ3_VERSION}.tar.bz2"						\
	"${Main_Module}-Src-${UIQ3_VERSION}"

    printf "\e[42m%-10.10s : \e[43m %-40.40s \e[m\n" "Package:" "Cleanup"
    grm --recursive --force "Export"
    grm --recursive --force "${Main_Module}-Src-${UIQ3_VERSION}"
    grm --recursive --force "${Main_Module}-Doc-${UIQ3_VERSION}"
popd

###############################################################################
# Do the actual deployment
###############################################################################

pushd "${Deployed}"
    printf "\e[42m%-10.10s : \e[43m %-40.40s \e[m\n" "Deploy:" "Copy APK"

    gcp --verbose												\
	"${Android_Dir}/apk/full/release/HP45-Droid-full-release.apk"						\
	"${Main_Module}-Droid-${UIQ3_VERSION}.apk"
    gcp --verbose												\
	"${Android_Dir}/bundle/fullRelease/HP45-Droid-full-release.aab"						\
	"${Main_Module}-Droid-${UIQ3_VERSION}.aab"

    printf "\e[42m%-10.10s : \e[43m %-40.40s \e[m\n" "Deploy:" "Copy JAR"
    gcp --verbose												\
	"${Desktop_Dir}/${Main_Module}-Desktop-${UIQ3_VERSION}.jar"						\
	"${Main_Module}-Desktop-${UIQ3_VERSION}.jar"

    printf "\e[42m%-10.10s : \e[43m %-40.40s \e[m\n" "Deploy:" "create deployment zip"
    7za a -tzip													\
	"${Main_Module}-${UIQ3_VERSION}.zip"									\
	"${Main_Module}-Desktop-${UIQ3_VERSION}.jar"								\
	"${Main_Module}-Doc-${UIQ3_VERSION}.zip"								\
	"${Main_Module}-Droid-${UIQ3_VERSION}.aab"								\
	"${Main_Module}-Droid-${UIQ3_VERSION}.apk"								\
	"${Main_Module}-OSX-${UIQ3_VERSION}.dmg"								\
	"${Main_Module}-Src-${UIQ3_VERSION}.tar.bz2"
popd

${0:h}/Upload.command

########################################################### {{{1 ###########
# vim: textwidth=0 nowrap tabstop=8 shiftwidth=4 softtabstop=4 noexpandtab :
# vim: filetype=zsh fileencoding=utf8 fileformat=unix :
